@@ -1,7 +1,8 @@
-/*  Copyright (C) 2006 yopyop
-    Copyright (C) 2006-2007 Theo Berkau
-    Copyright (C) 2007 shash
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2006-2007 Theo Berkau
+	Copyright (C) 2007 shash
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -17,6 +18,7 @@
 	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+
 #include <algorithm>
 #include <string.h>
 #include <stdlib.h>
@@ -736,6 +738,7 @@ FORCEINLINE void GPU::setFinalColorBG(u16 color, const u32 x)
 		case 5: draw = _master_setFinalBGColor<BACKDROP,Blend,true>(color,x); break;
 		case 6: draw = _master_setFinalBGColor<BACKDROP,Increase,true>(color,x); break;
 		case 7: draw = _master_setFinalBGColor<BACKDROP,Decrease,true>(color,x); break;
+		default: draw = false; break;
 	};
 
 	if(BACKDROP || draw) //backdrop must always be drawn
@@ -1184,16 +1187,16 @@ template<bool MOSAIC> FORCEINLINE void extRotBG2(GPU * gpu, s32 X, s32 Y, s16 PA
 	case BGType_AffineExt_256x1:
 		// 256 colors 
 		pal = MMU.ARM9_VMEM + gpu->core * 0x400;
-		apply_rot_fun<rot_256_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_ram[num], NULL, pal);
+		apply_rot_fun<rot_256_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_ram[num], 0, pal);
 		return;
 	case BGType_AffineExt_Direct:
 		// direct colors / BMP
-		apply_rot_fun<rot_BMP_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_ram[num], NULL, NULL);
+		apply_rot_fun<rot_BMP_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_ram[num], 0, NULL);
 		return;
 	case BGType_Large8bpp:
 		// large screen 256 colors
 		pal = MMU.ARM9_VMEM + gpu->core * 0x400;
-		apply_rot_fun<rot_256_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_large_ram[num], NULL, pal);
+		apply_rot_fun<rot_256_map<MOSAIC> >(gpu,X,Y,PA,PB,PC,PD,LG, gpu->BG_bmp_large_ram[num], 0, pal);
 		return;
 	default: break;
 	}
@@ -1291,13 +1294,13 @@ template<bool MOSAIC> void lineExtRot(GPU * gpu)
 /* if i understand it correct, and it fixes some sprite problems in chameleon shot */
 /* we have a 15 bit color, and should use the pal entry bits as alpha ?*/
 /* http://nocash.emubase.de/gbatek.htm#dsvideoobjs */
-INLINE void render_sprite_BMP (GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u16 * src, u8 * dst_alpha, u8 * typeTab, u8 * prioTab, 
-							   u8 prio, int lg, int sprX, int x, int xdir, u8 alpha) 
+INLINE void render_sprite_BMP (GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u32 srcadr, u8 * dst_alpha, u8 * typeTab, u8 * prioTab, u8 prio, int lg, int sprX, int x, int xdir, u8 alpha) 
 {
 	int i; u16 color;
 	for(i = 0; i < lg; i++, ++sprX, x+=xdir)
 	{
-		color = LE_TO_LOCAL_16(src[x]);
+		u16* src = (u16*)MMU_gpu_map(srcadr+(x<<1));
+		color = LE_TO_LOCAL_16(*src);
 
 		// alpha bit = invisible
 		if ((color&0x8000)&&(prio<prioTab[sprX]))
@@ -1311,8 +1314,7 @@ INLINE void render_sprite_BMP (GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u16 * s
 	}
 }
 
-INLINE void render_sprite_256 (	GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u8 * src, u16 * pal, 
-								u8 * dst_alpha, u8 * typeTab, u8 * prioTab, u8 prio, int lg, int sprX, int x, int xdir, u8 alpha)
+INLINE void render_sprite_256(GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u32 srcadr, u16 * pal, u8 * dst_alpha, u8 * typeTab, u8 * prioTab, u8 prio, int lg, int sprX, int x, int xdir, u8 alpha)
 {
 	int i; 
 	u8 palette_entry; 
@@ -1320,7 +1322,9 @@ INLINE void render_sprite_256 (	GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u8 * s
 
 	for(i = 0; i < lg; i++, ++sprX, x+=xdir)
 	{
-		palette_entry = src[(x&0x7) + ((x&0xFFF8)<<3)];
+		u32 adr = srcadr + (x&0x7) + ((x&0xFFF8)<<3);
+		u8* src = (u8 *)MMU_gpu_map(adr);
+		palette_entry = *src;
 		color = LE_TO_LOCAL_16(pal[palette_entry]);
 
 		// palette entry = 0 means backdrop
@@ -1335,8 +1339,7 @@ INLINE void render_sprite_256 (	GPU * gpu, u8 spriteNum, u16 l, u8 * dst, u8 * s
 	}
 }
 
-INLINE void render_sprite_16 (	GPU * gpu, u16 l, u8 * dst, u8 * src, u16 * pal, 
-								u8 * dst_alpha, u8 * typeTab, u8 * prioTab, u8 prio, int lg, int sprX, int x, int xdir, u8 alpha)
+INLINE void render_sprite_16 (	GPU * gpu, u16 l, u8 * dst, u32 srcadr, u16 * pal, u8 * dst_alpha, u8 * typeTab, u8 * prioTab, u8 prio, int lg, int sprX, int x, int xdir, u8 alpha)
 {
 	int i; 
 	u8 palette, palette_entry;
@@ -1345,7 +1348,11 @@ INLINE void render_sprite_16 (	GPU * gpu, u16 l, u8 * dst, u8 * src, u16 * pal,
 	for(i = 0; i < lg; i++, ++sprX, x+=xdir)
 	{
 		x1 = x>>1;
-		palette = src[(x1&0x3) + ((x1&0xFFFC)<<3)];
+		
+		u32 adr = srcadr + (x1&0x3) + ((x1&0xFFFC)<<3);
+		u8* src = (u8 *)MMU_gpu_map(adr);//
+		palette = *src;
+
 		if (x & 1) palette_entry = palette >> 4;
 		else       palette_entry = palette & 0xF;
 		color = LE_TO_LOCAL_16(pal[palette_entry]);
@@ -1403,8 +1410,12 @@ FORCEINLINE BOOL compute_sprite_vars(_OAM_ * spriteInfo, u16 l,
 // that tells us where the first pixel of a screenline starts in the sprite,
 // and how a step to the right in a screenline translates within the sprite
 
-	if ((l<sprY)||(l>=sprY+sprSize.y) ||	/* sprite lines outside of screen */
-		(sprX==256)||(sprX+sprSize.x<=0))	/* sprite pixels outside of line */
+	//this wasn't really tested by anything. very unlikely to get triggered
+	y = (l - sprY)&255;                        /* get the y line within sprite coords */
+	if(y >= sprSize.y)
+		return FALSE;
+
+	if((sprX==256)||(sprX+sprSize.x<=0))	/* sprite pixels outside of line */
 		return FALSE;				/* not to be drawn */
 
 	// sprite portion out of the screen (LEFT)
@@ -1418,8 +1429,6 @@ FORCEINLINE BOOL compute_sprite_vars(_OAM_ * spriteInfo, u16 l,
 	if (sprX+sprSize.x >= 256)
 		lg = 256 - sprX;
 
-	y = l - sprY;                           /* get the y line within sprite coords */
-
 	// switch TOP<-->BOTTOM
 	if (spriteInfo->VFlip)
 		y = sprSize.y - y -1;
@@ -1441,31 +1450,25 @@ FORCEINLINE BOOL compute_sprite_vars(_OAM_ * spriteInfo, u16 l,
 
 //TODO - refactor this so there isnt as much duped code between rotozoomed and non-rotozoomed versions
 
-static u8* bmp_sprite_address(GPU* gpu, _OAM_ * spriteInfo, size sprSize, s32 y)
+static u32 bmp_sprite_address(GPU* gpu, _OAM_ * spriteInfo, size sprSize, s32 y)
 {
-	u8* src;
-	if (spriteInfo->Mode == 3) //sprite is in BMP format
+	if (gpu->dispCnt().OBJ_BMP_mapping)
 	{
-		if (gpu->dispCnt().OBJ_BMP_mapping)
-		{
-			//tested by buffy sacrifice damage blood splatters in corner
-			src = (u8 *)MMU_gpu_map(gpu->sprMem + (spriteInfo->TileIndex<<gpu->sprBMPBoundary) + (y*sprSize.x*2));
-		}
-		else
-		{
-			//2d mapping:
-			//verified in rotozoomed mode by knights in the nightmare intro
-
-			if (gpu->dispCnt().OBJ_BMP_2D_dim)
-				//256*256, verified by heroes of mana FMV intro
-				src = (u8 *)MMU_gpu_map(gpu->sprMem + (((spriteInfo->TileIndex&0x3E0) * 64  + (spriteInfo->TileIndex&0x1F) *8 + ( y << 8)) << 1));
-			else 
-				//128*512, verified by harry potter and the order of the phoenix conversation portraits
-				src = (u8 *)MMU_gpu_map(gpu->sprMem + (((spriteInfo->TileIndex&0x3F0) * 64  + (spriteInfo->TileIndex&0x0F) *8 + ( y << 7)) << 1));
-		}
+		//tested by buffy sacrifice damage blood splatters in corner
+		return gpu->sprMem + (spriteInfo->TileIndex<<gpu->sprBMPBoundary) + (y*sprSize.x*2);
 	}
+	else
+	{
+		//2d mapping:
+		//verified in rotozoomed mode by knights in the nightmare intro
 
-	return src;
+		if (gpu->dispCnt().OBJ_BMP_2D_dim)
+			//256*256, verified by heroes of mana FMV intro
+			return gpu->sprMem + (((spriteInfo->TileIndex&0x3E0) * 64  + (spriteInfo->TileIndex&0x1F) *8 + ( y << 8)) << 1);
+		else 
+			//128*512, verified by harry potter and the order of the phoenix conversation portraits
+			return gpu->sprMem + (((spriteInfo->TileIndex&0x3F0) * 64  + (spriteInfo->TileIndex&0x0F) *8 + ( y << 7)) << 1);
+	}
 }
 
 
@@ -1515,6 +1518,7 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 		s32 sprX, sprY, x, y, lg;
 		int xdir;
 		u8 prio, * src;
+		u32 srcadr;
 		u16 j;
 
 		// Check if sprite is disabled before everything
@@ -1553,15 +1557,18 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 				lg <<= 1;
 			}
 
-			// Check if sprite enabled
-			if ((l   <sprY) || (l >= sprY+fieldY) ||
-				(sprX==256) || (sprX+fieldX<=0))
+			//check if the sprite is visible y-wise. unfortunately our logic for x and y is different due to our scanline based rendering
+			//tested thoroughly by many large sprites in Super Robot Wars K which wrap around the screen
+			y = (l - sprY)&255;
+			if(y >= fieldY)
+				continue;
+
+			//check if sprite is visible x-wise.
+			if((sprX==256) || (sprX+fieldX<=0))
 				continue;
 
 			cost += sprSize.x*2 + 10;
 
-			y = l - sprY;
-
 			// Get which four parameter block is assigned to this sprite
 			blockparameter = (spriteInfo->RotScalIndex + (spriteInfo->HFlip<< 3) + (spriteInfo->VFlip << 4))*4;
 
@@ -1646,7 +1653,11 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 			// Rotozoomed direct color
 			else if(spriteInfo->Mode == 3)
 			{
-				src = bmp_sprite_address(this,spriteInfo,sprSize,0);
+				//transparent (i think, dont bother to render?) if alpha is 0
+				if(spriteInfo->PaletteIndex == 0)
+					continue;
+
+				srcadr = bmp_sprite_address(this,spriteInfo,sprSize,0);
 
 				for(j = 0; j < lg; ++j, ++sprX)
 				{
@@ -1661,13 +1672,15 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 					{
 						if(dispCnt->OBJ_BMP_2D_dim)
 							//tested by knights in the nightmare
-							offset = (bmp_sprite_address(this,spriteInfo,sprSize,auxY)-src)/2+auxX;
+							offset = (bmp_sprite_address(this,spriteInfo,sprSize,auxY)-srcadr)/2+auxX;
 						else //tested by lego indiana jones (somehow?)
 							//tested by buffy sacrifice damage blood splatters in corner
 							offset = auxX + (auxY*sprSize.x);
 
 
-						colour = T1ReadWord (src, offset<<1);
+						u16* mem = (u16*)MMU_gpu_map(srcadr + (offset<<1));
+						
+						colour = T1ReadWord(mem,0);
 
 						if((colour&0x8000) && (prio<prioTab[sprX]))
 						{
@@ -1742,7 +1755,7 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 				continue;
 			}
 		}
-		else
+		else //NOT rotozoomed
 		{
 			u16 * pal;
 	
@@ -1774,48 +1787,48 @@ void GPU::_spriteRender(u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab)
 
 			if (spriteInfo->Mode == 3)              //sprite is in BMP format
 			{
-				src = bmp_sprite_address(this,spriteInfo,sprSize, y);
+				srcadr = bmp_sprite_address(this,spriteInfo,sprSize, y);
 
 				//transparent (i think, dont bother to render?) if alpha is 0
 				if(spriteInfo->PaletteIndex == 0)
 					continue;
 				
-				render_sprite_BMP (gpu, i, l, dst, (u16*)src, dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->PaletteIndex);
+				render_sprite_BMP (gpu, i, l, dst, srcadr, dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->PaletteIndex);
 				continue;
 			}
 				
-			if(spriteInfo->Depth)                   /* 256 colors */
+			if(spriteInfo->Depth) //256 colors
 			{
 				if(MODE == SPRITE_2D)
-					src = (u8 *)MMU_gpu_map(gpu->sprMem + ((spriteInfo->TileIndex)<<5) + ((y>>3)<<10) + ((y&0x7)*8));
+					srcadr = gpu->sprMem + ((spriteInfo->TileIndex)<<5) + ((y>>3)<<10) + ((y&0x7)*8);
 				else
-					src = (u8 *)MMU_gpu_map(gpu->sprMem + (spriteInfo->TileIndex<<block) + ((y>>3)*sprSize.x*8) + ((y&0x7)*8));
+					srcadr = gpu->sprMem + (spriteInfo->TileIndex<<block) + ((y>>3)*sprSize.x*8) + ((y&0x7)*8);
 				
 				if (dispCnt->ExOBJPalette_Enable)
 					pal = (u16*)(MMU.ObjExtPal[gpu->core][0]+(spriteInfo->PaletteIndex*0x200));
 				else
 					pal = (u16*)(MMU.ARM9_VMEM + 0x200 + gpu->core *0x400);
 		
-				render_sprite_256 (gpu, i, l, dst, src, pal, 
-					dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->Mode == 1);
+				render_sprite_256(gpu, i, l, dst, srcadr, pal, dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->Mode == 1);
 
 				continue;
 			}
+
 			// 16 colors 
 			if(MODE == SPRITE_2D)
 			{
-				src = (u8 *)MMU_gpu_map(gpu->sprMem + ((spriteInfo->TileIndex)<<5) + ((y>>3)<<10) + ((y&0x7)*4));
+				srcadr = gpu->sprMem + ((spriteInfo->TileIndex)<<5) + ((y>>3)<<10) + ((y&0x7)*4);
 			}
 			else
 			{
-				src = (u8 *)MMU_gpu_map(gpu->sprMem + (spriteInfo->TileIndex<<block) + ((y>>3)*sprSize.x*4) + ((y&0x7)*4));
+				srcadr = gpu->sprMem + (spriteInfo->TileIndex<<block) + ((y>>3)*sprSize.x*4) + ((y&0x7)*4);
 			}
 				
 			pal = (u16*)(MMU.ARM9_VMEM + 0x200 + gpu->core * 0x400);
 			
 			pal += (spriteInfo->PaletteIndex<<4);
 			
-			render_sprite_16 (gpu, l, dst, src, pal, dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->Mode == 1);
+			render_sprite_16 (gpu, l, dst, srcadr, pal, dst_alpha, typeTab, prioTab, prio, lg, sprX, x, xdir, spriteInfo->Mode == 1);
 		}
 	}
 
@@ -2022,20 +2035,25 @@ static void GPU_RenderLine_layer(NDS_Screen * screen, u16 l)
 	//we need to write backdrop colors in the same way as we do BG pixels in order to do correct window processing
 	//this is currently eating up 2fps or so. it is a reasonable candidate for optimization. 
 	gpu->currBgNum = 5;
-	switch(gpu->setFinalColorBck_funcNum) {
-		case 0: case 1: //for backdrops, (even with window enabled) none and blend are both the same: just copy the color
+	switch(gpu->setFinalColorBck_funcNum)
+	{
+		//for backdrops, blend isnt applied (it's illogical, isnt it?)
+		case 0: 
+		case 1: 
 			memset_u16_le<256>(gpu->currDst,backdrop_color); 
 			break;
+
+		//for backdrops, fade in and fade out can be applied if it's a 1st target screen
 		case 2:
-			//for non-windowed fade, we can just fade the color and fill
-			memset_u16_le<256>(gpu->currDst,gpu->currentFadeInColors[backdrop_color]);
+			if(gpu->BLDCNT & 0x20) //backdrop is selected for color effect
+				memset_u16_le<256>(gpu->currDst,gpu->currentFadeInColors[backdrop_color]);
 			break;
 		case 3:
-			//likewise for non-windowed fadeout
-			memset_u16_le<256>(gpu->currDst,gpu->currentFadeOutColors[backdrop_color]);
+			if(gpu->BLDCNT & 0x20) //backdrop is selected for color effect
+				memset_u16_le<256>(gpu->currDst,gpu->currentFadeOutColors[backdrop_color]);
 			break;
 
-		//windowed fades need special treatment
+		//windowed cases apparently need special treatment? why? can we not render the backdrop? how would that even work?
 		case 4: for(int x=0;x<256;x++) gpu->___setFinalColorBck<false,true,4>(backdrop_color,x,1); break;
 		case 5: for(int x=0;x<256;x++) gpu->___setFinalColorBck<false,true,5>(backdrop_color,x,1); break;
 		case 6: for(int x=0;x<256;x++) gpu->___setFinalColorBck<false,true,6>(backdrop_color,x,1); break;
