@@ -1,22 +1,20 @@
-/*	Copyright (C) 2006 yopyop
+/*
+	Copyright (C) 2006 yopyop
 	Copyright (C) 2007 shash
-	Copyright (C) 2007-2010 DeSmuME team
+	Copyright (C) 2007-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef MMU_H
@@ -204,7 +202,7 @@ public:
 	u32 saddr_user, daddr_user;
 	
 	//indicates whether the dma needs to be checked for triggering
-	BOOL check;
+	BOOL dmaCheck;
 
 	//indicates whether the dma right now is logically running
 	//(though for now we copy all the data when it triggers)
@@ -224,7 +222,7 @@ public:
 	bool loadstate(EMUFILE *f);
 
 	void exec();
-	void doCopy();
+	template<int PROCNUM> void doCopy();
 	void doPause();
 	void doStop();
 	void doSchedule();
@@ -239,7 +237,7 @@ public:
 		//by inspecting dma3 saddr when it boots
 		saddr(0), daddr(0),
 		saddr_user(0), daddr_user(0),
-		check(FALSE),
+		dmaCheck(FALSE),
 		running(FALSE),
 		paused(FALSE),
 		triggered(FALSE),
@@ -319,11 +317,14 @@ struct MMU_struct
 {
 	//ARM9 mem
 	u8 ARM9_ITCM[0x8000];
-    u8 ARM9_DTCM[0x4000];
-    u8 MAIN_MEM[0x800000]; //this has been expanded to 8MB to support debug consoles
-    u8 ARM9_REG[0x1000000];
-    u8 ARM9_BIOS[0x8000];
-    u8 ARM9_VMEM[0x800];
+	u8 ARM9_DTCM[0x4000];
+
+	//u8 MAIN_MEM[4*1024*1024]; //expanded from 4MB to 8MB to support debug consoles
+	//u8 MAIN_MEM[8*1024*1024]; //expanded from 8MB to 16MB to support dsi
+	u8 MAIN_MEM[16*1024*1024]; //expanded from 8MB to 16MB to support dsi
+	u8 ARM9_REG[0x1000000]; //this variable is evil and should be removed by correctly emulating all registers.
+	u8 ARM9_BIOS[0x8000];
+	u8 ARM9_VMEM[0x800];
 	
 	#include "PACKED.h"
 	struct {
@@ -389,6 +390,8 @@ struct MMU_struct
 	
 	//these are the user-controlled IF bits. some IF bits are generated as necessary from hardware conditions
 	u32 reg_IF_bits[2];
+	//these flags are set occasionally to indicate that an irq should have entered the pipeline, and processing will be deferred a tiny bit to help emulate things
+	u32 reg_IF_pending[2];
 
 	u32 reg_DISP3DCNT_bits;
 
@@ -419,6 +422,32 @@ struct MMU_struct
 	nds_dscard dscard[2];
 };
 
+
+//everything in here is derived from libnds behaviours. no hardware tests yet
+class DSI_TSC
+{
+public:
+	DSI_TSC();
+	void reset_command();
+	u16 write16(u16 val);
+	bool save_state(EMUFILE* os);
+	bool load_state(EMUFILE* is);
+
+private:
+	u16 read16();
+	u8 reg_selection;
+	u8 read_flag;
+	s32 state;
+	s32 readcount;
+
+	//registers[0] contains the current page.
+	//we are going to go ahead and save these out in case we want to change the way this is emulated in the future..
+	//we may want to poke registers in here at more convenient times and have the TSC dumbly pluck them out,
+	//rather than generate the values on the fly
+	u8 registers[0x80];
+};
+
+
 //this contains things which can't be memzeroed because they are smarter classes
 struct MMU_struct_new
 {
@@ -428,6 +457,7 @@ struct MMU_struct_new
 	TGXSTAT gxstat;
 	SqrtController sqrt;
 	DivController div;
+	DSI_TSC dsi_tsc;
 
 	void write_dma(const int proc, const int size, const u32 adr, const u32 val);
 	u32 read_dma(const int proc, const int size, const u32 adr);
@@ -586,13 +616,7 @@ extern u32 partie;
 extern u32 _MMU_MAIN_MEM_MASK;
 extern u32 _MMU_MAIN_MEM_MASK16;
 extern u32 _MMU_MAIN_MEM_MASK32;
-inline void SetupMMU(BOOL debugConsole) {
-	if(debugConsole) _MMU_MAIN_MEM_MASK = 0x7FFFFF;
-	else _MMU_MAIN_MEM_MASK = 0x3FFFFF;
-	_MMU_MAIN_MEM_MASK16 = _MMU_MAIN_MEM_MASK & ~1;
-	_MMU_MAIN_MEM_MASK32 = _MMU_MAIN_MEM_MASK & ~3;
-}
-
+void SetupMMU(bool debugConsole, bool dsi);
 
 FORCEINLINE void CheckMemoryDebugEvent(EDEBUG_EVENT event, const MMU_ACCESS_TYPE type, const u32 procnum, const u32 addr, const u32 size, const u32 val)
 {
