@@ -1,5 +1,6 @@
-/*	Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -38,12 +39,14 @@
 #include "debug.h"
 #include "firmware.h"
 #include "version.h"
+#include "slot1.h"
 
 #include "path.h"
 
+//int xxctr=0;
 //#define LOG_ARM9
 //#define LOG_ARM7
-//bool dolog = false;
+//#define dolog (currFrameCounter>15)
 //#define LOG_TO_FILE
 //#define LOG_TO_FILE_REGS
 
@@ -64,6 +67,7 @@ static u8	countLid = 0;
 GameInfo gameInfo;
 NDSSystem nds;
 CFIRMWARE	*firmware = NULL;
+ADVANsCEne	advsc;
 
 using std::min;
 using std::max;
@@ -304,8 +308,6 @@ RomBanner::RomBanner(bool defaultInit)
 	memset(bitmap,0,sizeof(bitmap));
 	memset(palette,0,sizeof(palette));
 	memset(titles,0,sizeof(titles));
-	for(int i=0;i<NUM_TITLES;i++)
-		wcscpy(titles[i],L"None");
 	memset(end0xFF,0,sizeof(end0xFF));
 }
 
@@ -403,8 +405,12 @@ void GameInfo::populate()
 			}
 		}*/
 
-	
+	//this may look like a poor heuristic for detecting homebrew, but it is actually pretty good.
+	//setting your own game code is stupid, so homebrew should just leave it.
+	//however, non-devkitARM-default makefiles may not have set this.
+	isHomebrew = !memcmp(header.gameCode,"####",4);
 }
+
 #ifdef _WINDOWS
 
 static std::vector<char> buffer;
@@ -426,15 +432,11 @@ static void loadrom(std::string fname) {
 	fclose(inf);
 }
 
-int NDS_LoadROM(const char *filename, const char *logicalFilename)
+static int rom_init_path(const char *filename, const char *logicalFilename)
 {
-	int					type = ROM_NDS;
-	char				buf[MAX_PATH];
+	int	type = ROM_NDS;
 
-	if (filename == NULL)
-		return -1;
-	
-    path.init(logicalFilename);
+	path.init(logicalFilename);
 
 	if ( path.isdsgba(path.path)) {
 		type = ROM_DSGBA;
@@ -465,9 +467,85 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 		return -1;
 	}
 
+	return 1;
+}
+#else
+static int rom_init_path(const char *filename, const char *logicalFilename)
+{
+	int			ret;
+	int			type;
+	ROMReader_struct	*reader;
+	void			*file;
+	u32			size;
+	char			*noext;
+
+	noext = strdup(filename);
+	reader = ROMReaderInit(&noext);
+	free(noext);
+	
+	if (logicalFilename)
+		path.init(logicalFilename);
+	else
+		path.init(filename);
+
+	if (!strcasecmp(path.extension().c_str(), "zip"))
+		type = ROM_NDS;
+	else if (!strcasecmp(path.extension().c_str(), "nds"))
+		type = ROM_NDS;
+	else if (path.isdsgba(path.path))
+		type = ROM_DSGBA;
+	else
+		type = ROM_NDS;
+
+	file = reader->Init(filename);
+	if (!file)
+	{
+		reader->DeInit(file);
+		return -1;
+	}
+
+	size = reader->Size(file);
+
+	if(type == ROM_DSGBA)
+	{
+		reader->Seek(file, DSGBA_LOADER_SIZE, SEEK_SET);
+		size -= DSGBA_LOADER_SIZE;
+	}
+
+	//check that size is at least the size of the header
+	if (size < 352) {
+		reader->DeInit(file);
+		return -1;
+	}
+
+	// Make sure old ROM is freed first(at least this way we won't be eating
+	// up a ton of ram before the old ROM is freed)
+	if(MMU.CART_ROM != MMU.UNUSED_RAM)
+		NDS_FreeROM();
+
+	gameInfo.resize(size);
+	ret = reader->Read(file, gameInfo.romdata, size);
+	gameInfo.fillGap();
+	reader->DeInit(file);
+
+	return ret;
+}
+#endif
+
+int NDS_LoadROM(const char *filename, const char *logicalFilename)
+{
+	int	ret;
+	char	buf[MAX_PATH];
+
+	if (filename == NULL)
+		return -1;
+
+	ret = rom_init_path(filename, logicalFilename);
+	if (ret < 1)
+		return ret;
+
 	//decrypt if necessary..
 	//but this is untested and suspected to fail on big endian, so lets not support this on big endian
-
 #ifndef WORDS_BIGENDIAN
 	bool okRom = DecryptSecureArea((u8*)gameInfo.romdata,gameInfo.romsize);
 
@@ -486,153 +564,59 @@ int NDS_LoadROM(const char *filename, const char *logicalFilename)
 
 	gameInfo.populate();
 	gameInfo.crc = crc32(0,(u8*)gameInfo.romdata,gameInfo.romsize);
-	INFO("\nROM crc: %08X\n", gameInfo.crc);
+	INFO("\nROM game code: %c%c%c%c\n", gameInfo.header.gameCode[0], gameInfo.header.gameCode[1], gameInfo.header.gameCode[2], gameInfo.header.gameCode[3]);
+	INFO("ROM crc: %08X\n", gameInfo.crc);
 	INFO("ROM serial: %s\n", gameInfo.ROMserial);
-	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
-	INFO("ROM game code: %c%c%c%c\n\n", gameInfo.header.gameCode[0], gameInfo.header.gameCode[1], gameInfo.header.gameCode[2], gameInfo.header.gameCode[3]);
+	INFO("ROM internal name: %s\n", gameInfo.ROMname);
+	
+	memset(buf, 0, MAX_PATH);
+	strcpy(buf, path.pathToModule);
+	strcat(buf, "desmume.ddb");							// DeSmuME database	:)
+	advsc.setDatabase(buf);
+	buf[0] = gameInfo.header.gameCode[0];
+	buf[1] = gameInfo.header.gameCode[1];
+	buf[2] = gameInfo.header.gameCode[2];
+	buf[3] = gameInfo.header.gameCode[3];
+	buf[4] = 0;
+	if (advsc.checkDB(buf))
+	{
+		u8 sv = advsc.getSaveType();
+		printf("ADVANsCEne database:\n");
+		printf("\t* ROM save type: ");
+		if (sv == 0xFF)
+			printf("Unknown");
+		else
+			if (sv == 0xFE)
+				printf("None");
+			else
+			{
+				printf("%s", save_names[sv]);
+				if (CommonSettings.autodetectBackupMethod == 1)
+					backup_setManualBackupType(sv+1);
+			}
+		printf("\n\t* ROM crc: %08X\n", advsc.getCRC32());
+	}
+	printf("\n");
 
 	//for homebrew, try auto-patching DLDI. should be benign if there is no DLDI or if it fails
-	if(!memcmp(gameInfo.header.gameCode,"####",4))
+	if(gameInfo.isHomebrew)
 		DLDI::tryPatch((void*)gameInfo.romdata, gameInfo.romsize);
 
-	NDS_Reset();
-
 	memset(buf, 0, MAX_PATH);
-
 	path.getpathnoext(path.BATTERY, buf);
-	
 	strcat(buf, ".dsv");							// DeSmuME memory card	:)
-
 	MMU_new.backupDevice.load_rom(buf);
 
 	memset(buf, 0, MAX_PATH);
-
 	path.getpathnoext(path.CHEATS, buf);
-	
 	strcat(buf, ".dct");							// DeSmuME cheat		:)
 	cheats->init(buf);
 
-	return 1;
-}
-#else
-int NDS_LoadROM(const char *filename, const char *logicalFilename)
-{
-	int					ret;
-	int					type;
-	ROMReader_struct	*reader;
-	void				*file;
-	u32					size;
-	u8					*data;
-	char				*noext;
-	char				buf[MAX_PATH];
-
-	if (filename == NULL)
-		return -1;
-
-	noext = strdup(filename);
-	reader = ROMReaderInit(&noext);
-	
-	if(logicalFilename) path.init(logicalFilename);
-	else path.init(filename);
-	if(!strcasecmp(path.extension().c_str(), "zip"))		type = ROM_NDS;
-	else if ( !strcasecmp(path.extension().c_str(), "nds"))
-		type = ROM_NDS;
-	else if ( path.isdsgba(path.path))
-		type = ROM_DSGBA;
-	else
-		type = ROM_NDS;
-
-	file = reader->Init(filename);
-	if (!file)
-	{
-		reader->DeInit(file);
-		free(noext);
-		return -1;
-	}
-
-	size = reader->Size(file);
-
-	if(type == ROM_DSGBA)
-	{
-		reader->Seek(file, DSGBA_LOADER_SIZE, SEEK_SET);
-		size -= DSGBA_LOADER_SIZE;
-	}
-
-	//check that size is at least the size of the header
-	if (size < 352) {
-		reader->DeInit(file);
-		free(noext);
-		return -1;
-	}
-
-	gameInfo.resize(size);
-
-	// Make sure old ROM is freed first(at least this way we won't be eating
-	// up a ton of ram before the old ROM is freed)
-	if(MMU.CART_ROM != MMU.UNUSED_RAM)
-		NDS_FreeROM();
-
-	data = new u8[gameInfo.mask + 1];
-	if (!data)
-	{
-		reader->DeInit(file);
-		free(noext);
-		return -1;
-	}
-
-	ret = reader->Read(file, data, size);
-	reader->DeInit(file);
-
-	//decrypt if necessary..
-	//but this is untested and suspected to fail on big endian, so lets not support this on big endian
-#ifndef WORDS_BIGENDIAN
-	bool okRom = DecryptSecureArea(data,size);
-
-	if(!okRom) {
-		printf("Specified file is not a valid rom\n");
-		return -1;
-	}
-#endif
-
-
-	if (cheatSearch)
-		cheatSearch->close();
-	MMU_unsetRom();
-	NDS_SetROM(data, gameInfo.mask);
-
-	gameInfo.populate();
-	gameInfo.crc = crc32(0,data,size);
-	INFO("\nROM crc: %08X\n", gameInfo.crc);
-	INFO("ROM serial: %s\n", gameInfo.ROMserial);
-	INFO("ROM internal name: %s\n\n", gameInfo.ROMname);
-	INFO("ROM game code: %c%c%c%c\n\n", gameInfo.header.gameCode[0], gameInfo.header.gameCode[1], gameInfo.header.gameCode[2], gameInfo.header.gameCode[3]);
-
-	//for homebrew, try auto-patching DLDI. should be benign if there is no DLDI or if it fails
-	if(!memcmp(gameInfo.header.gameCode,"####",4))
-		DLDI::tryPatch((void*)data, gameInfo.mask + 1);
-
 	NDS_Reset();
 
-	free(noext);
-
-	memset(buf, 0, MAX_PATH);
-
-	path.getpathnoext(path.BATTERY, buf);
-	
-	strcat(buf, ".dsv");							// DeSmuME memory card	:)
-
-	MMU_new.backupDevice.load_rom(buf);
-
-	memset(buf, 0, MAX_PATH);
-
-	path.getpathnoext(path.CHEATS, buf);
-	
-	strcat(buf, ".dct");							// DeSmuME cheat		:)
-	cheats->init(buf);
-
 	return ret;
 }
-#endif
+
 void NDS_FreeROM(void)
 {
 	FCEUI_StopMovie();
@@ -643,20 +627,38 @@ void NDS_FreeROM(void)
 	MMU_unsetRom();
 }
 
-
-
-int NDS_ImportSave(const char *filename)
+u32 NDS_ImportSaveSize(const char *filename)
 {
-	if (strlen(filename) < 4)
-		return 0;
+	u32 res = 0;
+	if (strlen(filename) < 4) return 0;
 
 	if (memcmp(filename+strlen(filename)-4, ".duc", 4) == 0)
-		return MMU_new.backupDevice.load_duc(filename);
+	{
+		res = MMU_new.backupDevice.get_save_duc_size(filename);
+		if (res == 0xFFFFFFFF) return 0;
+		return res;
+	}
+
+	res = MMU_new.backupDevice.get_save_nogba_size(filename);
+	if (res != 0xFFFFFFFF) return res;
+
+	res = MMU_new.backupDevice.get_save_raw_size(filename);
+	if (res != 0xFFFFFFFF) return res;
+
+	return 0;
+}
+
+int NDS_ImportSave(const char *filename, u32 force_size)
+{
+	if (strlen(filename) < 4) return 0;
+
+	if (memcmp(filename+strlen(filename)-4, ".duc", 4) == 0)
+		return MMU_new.backupDevice.load_duc(filename, force_size);
 	else
-		if (MMU_new.backupDevice.load_no_gba(filename))
+		if (MMU_new.backupDevice.load_no_gba(filename, force_size))
 			return 1;
 		else
-			return MMU_new.backupDevice.load_raw(filename);
+			return MMU_new.backupDevice.load_raw(filename, force_size);
 
 	return 0;
 }
@@ -1048,7 +1050,7 @@ void NDS_OmitFrameSkip(int force) {
 
 enum ESI_DISPCNT
 {
-	ESI_DISPCNT_HStart, ESI_DISPCNT_HBlank
+	ESI_DISPCNT_HStart, ESI_DISPCNT_HStartIRQ, ESI_DISPCNT_HDraw, ESI_DISPCNT_HBlank
 };
 
 u64 nds_timer;
@@ -1183,11 +1185,11 @@ template<int procnum, int chan> struct TSequenceItem_DMA : public TSequenceItem
 
 	FORCEINLINE bool isTriggered()
 	{
-		return (controller->check && nds_timer>= controller->nextEvent);
+		return (controller->dmaCheck && nds_timer>= controller->nextEvent);
 	}
 
 	FORCEINLINE bool isEnabled() { 
-		return controller->check?TRUE:FALSE;
+		return controller->dmaCheck?TRUE:FALSE;
 	}
 
 	FORCEINLINE u64 next()
@@ -1400,7 +1402,7 @@ static void initSchedule()
 // ARM7_CLOCK   = 33.51 mhz
 //				= 33513982 cycles per second
 // 				= 33.513982 cycles per microsecond
-const u64 kWifiCycles = 34*2;
+const u64 kWifiCycles = 67;//34*2;
 //(this isn't very precise. I don't think it needs to be)
 
 void Sequencer::init()
@@ -1503,8 +1505,8 @@ static void execHardware_hstart_vblankEnd()
 	sequencer.reschedule = true;
 
 	//turn off vblank status bit
-	T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) & 0xFFFE);
-	T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) & 0xFFFE);
+	T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) & ~1);
+	T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) & ~1);
 
 	//some emulation housekeeping
 	frameSkipper.Advance();
@@ -1514,16 +1516,13 @@ static void execHardware_hstart_vblankStart()
 {
 	//printf("--------VBLANK!!!--------\n");
 
-	//turn on vblank status bit
-	T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) | 1);
-	T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 1);
-
 	//fire vblank interrupts if necessary
-	if(T1ReadWord(MMU.ARM9_REG, 4) & 0x8) NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_VBLANK);
-	if(T1ReadWord(MMU.ARM7_REG, 4) & 0x8) NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_VBLANK);
-
-	//some emulation housekeeping
-	gfx3d_VBlankSignal();
+	for(int i=0;i<2;i++)
+		if(MMU.reg_IF_pending[i] & (1<<IRQ_BIT_LCD_VBLANK))
+		{
+			MMU.reg_IF_pending[i] &= ~(1<<IRQ_BIT_LCD_VBLANK);
+			NDS_makeIrq(i,IRQ_BIT_LCD_VBLANK);
+		}
 
 	//trigger vblank dmas
 	triggerDma(EDMAMode_VBlank);
@@ -1537,16 +1536,37 @@ static void execHardware_hstart_vblankStart()
 	nds.idleCycles[1] = 0;
 }
 
-static void execHardware_hstart_vcount()
+static u16 execHardware_gen_vmatch_goal()
 {
 	u16 vmatch = T1ReadWord(MMU.ARM9_REG, 4);
 	vmatch = ((vmatch>>8)|((vmatch<<1)&(1<<8)));
+	return vmatch;
+}
+
+static void execHardware_hstart_vcount_irq()
+{
+	//trigger pending VMATCH irqs
+	if(MMU.reg_IF_pending[ARMCPU_ARM9] & (1<<IRQ_BIT_LCD_VMATCH))
+	{
+		MMU.reg_IF_pending[ARMCPU_ARM9] &= ~(1<<IRQ_BIT_LCD_VMATCH);
+		NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_VMATCH);
+	}
+	if(MMU.reg_IF_pending[ARMCPU_ARM7] & (1<<IRQ_BIT_LCD_VMATCH))
+	{
+		MMU.reg_IF_pending[ARMCPU_ARM7] &= ~(1<<IRQ_BIT_LCD_VMATCH);
+		NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_VMATCH);
+	}
+}
+
+static void execHardware_hstart_vcount()
+{
+	u16 vmatch = execHardware_gen_vmatch_goal();
 	if(nds.VCount==vmatch)
 	{
 		//arm9 vmatch
 		T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) | 4);
 		if(T1ReadWord(MMU.ARM9_REG, 4) & 32) {
-			NDS_makeIrq(ARMCPU_ARM9,IRQ_BIT_LCD_VMATCH);
+			MMU.reg_IF_pending[ARMCPU_ARM9] |= (1<<IRQ_BIT_LCD_VMATCH);
 		}
 	}
 	else
@@ -1559,12 +1579,36 @@ static void execHardware_hstart_vcount()
 		//arm7 vmatch
 		T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 4);
 		if(T1ReadWord(MMU.ARM7_REG, 4) & 32)
-			NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_LCD_VMATCH);
+			MMU.reg_IF_pending[ARMCPU_ARM7] |= (1<<IRQ_BIT_LCD_VMATCH);
 	}
 	else
 		T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) & 0xFFFB);
 }
 
+static void execHardware_hdraw()
+{
+	//due to hacks in our selection of rendering time, we do not actually render here as intended.
+	//consider changing this if there is some problem with raster fx timing but check the documentation near the gpu rendering calls
+	//to make sure you check for regressions (nsmb, sonic classics, et al)
+}
+
+static void execHardware_hstart_irq()
+{
+	//this function very soon after the registers get updated to trigger IRQs
+	//this is necessary to fix "egokoro kyoushitsu" which idles waiting for vcount=192, which never happens due to a long vblank irq
+	//100% accurate emulation would require the read of VCOUNT to be in the pipeline already with the irq coming in behind it, thus 
+	//allowing the vcount to register as 192 occasionally (maybe about 1 out of 28 frames)
+	//the actual length of the delay is in execHardware() where the events are scheduled
+	sequencer.reschedule = true;
+	if(nds.VCount==192)
+	{
+		//when the vcount hits 192, vblank begins
+		execHardware_hstart_vblankStart();
+	}
+
+	execHardware_hstart_vcount_irq();
+}
+
 static void execHardware_hstart()
 {
 	nds.VCount++;
@@ -1580,14 +1624,28 @@ static void execHardware_hstart()
 
 	if(nds.VCount==263)
 	{
+		//when the vcount hits 263 it rolls over to 0
 		nds.VCount=0;
 	}
 	if(nds.VCount==262)
 	{
+		//when the vcount hits 262, vblank ends (oam pre-renders by one scanline)
 		execHardware_hstart_vblankEnd();
-	} else if(nds.VCount==192)
+	}
+	else if(nds.VCount==191)
 	{
-		execHardware_hstart_vblankStart();
+		//when the vcount hits 191, the 3d vblank occurs (maybe because OAM doesnt need to run anymore, so we can power it down, and give the 3d as much time as possible?)
+		gfx3d_VBlankSignal();
+	}
+	else if(nds.VCount==192)
+	{
+		//turn on vblank status bit
+		T1WriteWord(MMU.ARM9_REG, 4, T1ReadWord(MMU.ARM9_REG, 4) | 1);
+		T1WriteWord(MMU.ARM7_REG, 4, T1ReadWord(MMU.ARM7_REG, 4) | 1);
+
+		//check whether we'll need to fire vblank irqs
+		if(T1ReadWord(MMU.ARM9_REG, 4) & 0x8) MMU.reg_IF_pending[ARMCPU_ARM9] |= (1<<IRQ_BIT_LCD_VBLANK);
+		if(T1ReadWord(MMU.ARM7_REG, 4) & 0x8) MMU.reg_IF_pending[ARMCPU_ARM7] |= (1<<IRQ_BIT_LCD_VBLANK);
 	}
 
 	//write the new vcount
@@ -1689,12 +1747,32 @@ void Sequencer::execHardware()
 		{
 		case ESI_DISPCNT_HStart:
 			execHardware_hstart();
-			dispcnt.timestamp += 3168;
+			//(used to be 3168)
+			//hstart is actually 8 dots before the visible drawing begins
+			//we're going to run 1 here and then run 7 in the next case
+			dispcnt.timestamp += 1*6*2;
+			dispcnt.param = ESI_DISPCNT_HStartIRQ;
+			break;
+		case ESI_DISPCNT_HStartIRQ:
+			execHardware_hstart_irq();
+			dispcnt.timestamp += 7*6*2;
+			dispcnt.param = ESI_DISPCNT_HDraw;
+			break;
+			
+		case ESI_DISPCNT_HDraw:
+			execHardware_hdraw();
+			//duration of non-blanking period is ~1606 clocks (gbatek agrees) [but says its different on arm7]
+			//im gonna call this 267 dots = 267*6=1602
+			//so, this event lasts 267 dots minus the 8 dot preroll
+			dispcnt.timestamp += (267-8)*6*2;
 			dispcnt.param = ESI_DISPCNT_HBlank;
 			break;
+
 		case ESI_DISPCNT_HBlank:
 			execHardware_hblank();
-			dispcnt.timestamp += 1092;
+			//(once this was 1092 or 1092/12=91 dots.)
+			//there are surely 355 dots per scanline, less 267 for non-blanking period. the rest is hblank and then after that is hstart
+			dispcnt.timestamp += (355-267)*6*2;
 			dispcnt.param = ESI_DISPCNT_HStart;
 			break;
 		}
@@ -1859,13 +1937,14 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 	const u64 nds_timer_base, const s32 s32next, s32 arm9, s32 arm7)
 {
 	s32 timer = minarmtime<doarm9,doarm7>(arm9,arm7);
-	while(timer < s32next && !sequencer.reschedule)
+	while(timer < s32next && !sequencer.reschedule && execute)
 	{
 		if(doarm9 && (!doarm7 || arm9 <= timer))
 		{
 			if(!NDS_ARM9.waitIRQ&&!nds.freezeBus)
 			{
 				arm9log();
+				debug();
 				arm9 += armcpu_exec<ARMCPU_ARM9>();
 				#ifdef DEVELOPER
 					nds_debug_continuing[0] = false;
@@ -1876,7 +1955,7 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 				s32 temp = arm9;
 				arm9 = min(s32next, arm9 + kIrqWait);
 				nds.idleCycles[0] += arm9-temp;
-				if (gxFIFO.size < 255) nds.freezeBus = FALSE;
+				if (gxFIFO.size < 255) nds.freezeBus &= ~1;
 			}
 		}
 		if(doarm7 && (!doarm9 || arm7 <= timer))
@@ -1904,7 +1983,6 @@ static /*donotinline*/ std::pair<s32,s32> armInnerLoop(
 
 		timer = minarmtime<doarm9,doarm7>(arm9,arm7);
 		nds_timer = nds_timer_base + timer;
-		if (nds_timer >= MMU.gfx3dCycles) MMU_new.gxstat.sb = 0;	// clear stack busy flag
 	}
 
 	return std::make_pair(arm9, arm7);
@@ -2100,7 +2178,7 @@ void NDS_Reset()
 
 	nds.sleeping = FALSE;
 	nds.cardEjected = FALSE;
-	nds.freezeBus = FALSE;
+	nds.freezeBus = 0;
 	nds.power1.lcd = nds.power1.gpuMain = nds.power1.gfx3d_render = nds.power1.gfx3d_geometry = nds.power1.gpuSub = nds.power1.dispswap = 1;
 	nds.power2.speakers = 1;
 	nds.power2.wifi = 0;
@@ -2120,12 +2198,41 @@ void NDS_Reset()
 		TotalLagFrames = 0;
 	}
 
-	//spu must reset early on, since it will crash due to keeping a pointer into MMU memory for the sample pointers. yuck!
-	SPU_Reset();
-
+	SPU_DeInit();
 
 	MMU_Reset();
 
+
+	//put random garbage in vram for homebrew games, to help mimic the situation where libnds does not clear out junk
+	//which the card's launcher may or may not have left behind
+  //1. retail games dont clear TCM, so why should we jumble it and expect homebrew to clear it?
+  //2. some retail games _dont boot_ if main memory is jumbled. wha...?
+  //3. clearing this is not as useful as tracking uninitialized reads in dev+ builds
+  //4. the vram clearing causes lots of graphical corruptions in badly coded homebrews. this reduces compatibility substantially
+  //conclusion: disable it for now and bring it back as an option
+	//if(gameInfo.isHomebrew)
+	//{
+	//	u32 w=100000,x=99,y=117,z=19382173;
+	//	CTASSERT(sizeof(MMU.ARM9_LCD) < sizeof(MMU.MAIN_MEM));
+	//	CTASSERT(sizeof(MMU.ARM9_VMEM) < sizeof(MMU.MAIN_MEM));
+	//	CTASSERT(sizeof(MMU.ARM9_ITCM) < sizeof(MMU.MAIN_MEM));
+	//	CTASSERT(sizeof(MMU.ARM9_DTCM) < sizeof(MMU.MAIN_MEM));
+	//	for(int i=0;i<sizeof(MMU.MAIN_MEM);i++)
+	//	{
+	//		u32 t= (x^(x<<11)); 
+	//		x=y;
+	//		y=z; 
+	//		z=w;
+	//		t = (w= (w^(w>>19))^(t^(t>>8)));
+	//		//MMU.MAIN_MEM[i] = t;
+	//		if (i<sizeof(MMU.ARM9_LCD)) MMU.ARM9_LCD[i] = t;
+	//		if (i<sizeof(MMU.ARM9_VMEM)) MMU.ARM9_VMEM[i] = t;
+	//		//if (i<sizeof(MMU.ARM9_ITCM)) MMU.ARM9_ITCM[i] = t;
+	//		//if (i<sizeof(MMU.ARM9_DTCM)) MMU.ARM9_DTCM[i] = t;
+	//	}
+	//}
+
+
 	NDS_ARM7.BIOS_loaded = false;
 	NDS_ARM9.BIOS_loaded = false;
 	memset(MMU.ARM7_BIOS, 0, sizeof(MMU.ARM7_BIOS));
@@ -2369,13 +2476,15 @@ void NDS_Reset()
 	nds.wifiCycle = 0;
 	memset(nds.timerCycle, 0, sizeof(u64) * 2 * 4);
 	nds.old = 0;
-	nds.touchX = nds.touchY = 0;
+	nds.scr_touchX = nds.scr_touchY = nds.adc_touchX = nds.adc_touchY = 0;
 	nds.isTouch = 0;
-	nds.debugConsole = CommonSettings.DebugConsole;
+	nds.paddle = 0;
+	nds.ConsoleType = CommonSettings.ConsoleType;
+	nds._DebugConsole = CommonSettings.DebugConsole;
 	nds.ensataEmulation = CommonSettings.EnsataEmulation;
 	nds.ensataHandshake = ENSATA_HANDSHAKE_none;
 	nds.ensataIpcSyncCounter = 0;
-	SetupMMU(nds.debugConsole);
+	SetupMMU(nds.Is_DebugConsole(),nds.Is_DSI());
 
 	_MMU_write16<ARMCPU_ARM9>(REG_KEYINPUT, 0x3FF);
 	_MMU_write16<ARMCPU_ARM7>(REG_KEYINPUT, 0x3FF);
@@ -2398,12 +2507,20 @@ void NDS_Reset()
 		}
 	}
 
-	// Copy the whole header to Main RAM 0x27FFE00 on startup.
-	//  Reference: http://nocash.emubase.de/gbatek.htm#dscartridgeheader
-	//zero 27-jun-09 : why did this copy 0x90 more? gbatek says its not stored in ram.
-	//for (i = 0; i < ((0x170+0x90)/4); i++) {
-	for (int i = 0; i < ((0x170)/4); i++)
-		_MMU_write32<ARMCPU_ARM9>(0x027FFE00+i*4, LE_TO_LOCAL_32(((u32*)MMU.CART_ROM)[i]));
+	// Copy the whole header to Main RAM 0x27FFE00 on startup. (http://nocash.emubase.de/gbatek.htm#dscartridgeheader)
+	//once upon a time this copied 0x90 more. this was thought to be wrong, and changed.
+	if(nds.Is_DSI())
+	{
+		//dsi needs this copied later in memory. there are probably a number of things that  get copied to a later location in memory.. thats where the NDS consoles tend to stash stuff.
+		for (int i = 0; i < ((0x170)/4); i++)
+			_MMU_write32<ARMCPU_ARM9>(0x02FFFE00+i*4, LE_TO_LOCAL_32(((u32*)MMU.CART_ROM)[i]));
+	}
+	else
+	{
+		for (int i = 0; i < ((0x170)/4); i++)
+			_MMU_write32<ARMCPU_ARM9>(0x027FFE00+i*4, LE_TO_LOCAL_32(((u32*)MMU.CART_ROM)[i]));
+	}
+
 
 	// Write the header checksum to memory (the firmware needs it to see the cart)
 	_MMU_write16<ARMCPU_ARM9>(0x027FF808, T1ReadWord(MMU.CART_ROM, 0x15E));
@@ -2470,12 +2587,15 @@ void NDS_Reset()
 	Screen_Reset();
 	gfx3d_reset();
 	gpu3D->NDS_3D_Reset();
+	slot1Reset();
 
 	WIFI_Reset();
 
 	memcpy(FW_Mac, (MMU.fw.data + 0x36), 6);
 
 	initSchedule();
+
+	SPU_ReInit();
 }
 
 static std::string MakeInputDisplayString(u16 pad, const std::string* Buttons, int count) {
@@ -2511,22 +2631,22 @@ void ClearAutoHold(void) {
 	}
 }
 
-
-INLINE u16 NDS_getADCTouchPosX(u16 scrX)
+//convert a 12.4 screen coordinate to an ADC value.
+//the desmume host system will track the screen coordinate, but the hardware should be receiving the raw ADC values.
+//so we'll need to use this to simulate the ADC values corresponding to the desired screen coords, based on the current TSC calibrations
+u16 NDS_getADCTouchPosX(int scrX_lsl4)
 {
-	// this is a little iffy,
-	// we're basically adjusting the ADC results to
-	// compensate for how they will be interpreted.
-	// the actual system doesn't do this transformation.
-	int rv = (scrX - TSCal.scr.x1 + 1) * TSCal.adc.width / TSCal.scr.width + TSCal.adc.x1;
+	scrX_lsl4 >>= 4;
+	int rv = ((scrX_lsl4 - TSCal.scr.x1 + 1) * TSCal.adc.width) / TSCal.scr.width + TSCal.adc.x1;
 	rv = min(0xFFF, max(0, rv));
-	return (u16)rv;
+	return (u16)(rv);
 }
-INLINE u16 NDS_getADCTouchPosY(u16 scrY)
+u16 NDS_getADCTouchPosY(int scrY_lsl4)
 {
-	int rv = (scrY - TSCal.scr.y1 + 1) * TSCal.adc.height / TSCal.scr.height + TSCal.adc.y1;
+	scrY_lsl4 >>= 4;
+	int rv = ((scrY_lsl4 - TSCal.scr.y1 + 1) * TSCal.adc.height) / TSCal.scr.height + TSCal.adc.y1;
 	rv = min(0xFFF, max(0, rv));
-	return (u16)rv;
+	return (u16)(rv);
 }
 
 static UserInput rawUserInput = {}; // requested input, generally what the user is physically pressing
@@ -2628,27 +2748,21 @@ void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,b
 void NDS_setTouchPos(u16 x, u16 y)
 {
 	gotInputRequest();
-	rawUserInput.touch.touchX = NDS_getADCTouchPosX(x);
-	rawUserInput.touch.touchY = NDS_getADCTouchPosY(y);
+	rawUserInput.touch.touchX = x<<4;
+	rawUserInput.touch.touchY = y<<4;
 	rawUserInput.touch.isTouch = true;
 
 	if(movieMode != MOVIEMODE_INACTIVE && movieMode != MOVIEMODE_FINISHED)
 	{
 		// just in case, since the movie only stores 8 bits per touch coord
+#ifdef WORDS_BIGENDIAN
+		rawUserInput.touch.touchX &= 0xF00F;
+		rawUserInput.touch.touchY &= 0xF00F;
+#else
 		rawUserInput.touch.touchX &= 0x0FF0;
 		rawUserInput.touch.touchY &= 0x0FF0;
-	}
-
-#ifndef WIN32
-	// FIXME: this code should be deleted from here,
-	// other platforms should call NDS_beginProcessingInput,NDS_endProcessingInput once per frame instead
-	// (see the function called "StepRunLoop_Core" in src/windows/main.cpp),
-	// but I'm leaving this here for now since I can't test those other platforms myself.
-	nds.touchX = rawUserInput.touch.touchX;
-	nds.touchY = rawUserInput.touch.touchY;
-	nds.isTouch = 1;
-	MMU.ARM7_REG[0x136] &= 0xBF;
 #endif
+	}
 }
 void NDS_releaseTouch(void)
 { 
@@ -2656,17 +2770,6 @@ void NDS_releaseTouch(void)
 	rawUserInput.touch.touchX = 0;
 	rawUserInput.touch.touchY = 0;
 	rawUserInput.touch.isTouch = false;
-
-#ifndef WIN32
-	// FIXME: this code should be deleted from here,
-	// other platforms should call NDS_beginProcessingInput,NDS_endProcessingInput once per frame instead
-	// (see the function called "StepRunLoop_Core" in src/windows/main.cpp),
-	// but I'm leaving this here for now since I can't test those other platforms myself.
-	nds.touchX = 0;
-	nds.touchY = 0;
-	nds.isTouch = 0;
-	MMU.ARM7_REG[0x136] |= 0x40;
-#endif
 }
 void NDS_setMic(bool pressed)
 {
@@ -2722,6 +2825,7 @@ static void NDS_applyFinalInput()
 		((input.buttons.E ? 0 : 0x80) << 1) |
 		((input.buttons.W ? 0 : 0x80) << 2)) ;
 
+	pad = LOCAL_TO_LE_16(pad);
 	((u16 *)MMU.ARM9_REG)[0x130>>1] = (u16)pad;
 	((u16 *)MMU.ARM7_REG)[0x130>>1] = (u16)pad;
 
@@ -2758,33 +2862,32 @@ static void NDS_applyFinalInput()
 
 	if(input.touch.isTouch)
 	{
-		nds.touchX = input.touch.touchX;
-		nds.touchY = input.touch.touchY;
-		nds.isTouch = 1;
+		u16 adc_x = NDS_getADCTouchPosX(input.touch.touchX);
+		u16 adc_y = NDS_getADCTouchPosY(input.touch.touchY);
+		nds.adc_touchX = adc_x;
+		nds.adc_touchY = adc_y;
+		nds.adc_jitterctr = 0;
 
-		MMU.ARM7_REG[0x136] &= 0xBF;
+		nds.scr_touchX = input.touch.touchX;
+		nds.scr_touchY = input.touch.touchY;
+		nds.isTouch = 1;
 	}
 	else
 	{
-		nds.touchX = 0;
-		nds.touchY = 0;
+		nds.adc_touchX = 0;
+		nds.adc_touchY = 0;
+		nds.scr_touchX = 0;
+		nds.scr_touchY = 0;
 		nds.isTouch = 0;
-
-		MMU.ARM7_REG[0x136] |= 0x40;
 	}
 
-
 	if (input.buttons.F && !countLid) 
 	{
 		LidClosed = (!LidClosed) & 0x01;
 		if (!LidClosed)
 		{
-		//	SPU_Pause(FALSE);
 			NDS_makeIrq(ARMCPU_ARM7,IRQ_BIT_ARM7_FOLD);
-
 		}
-		//else
-			//SPU_Pause(TRUE);
 
 		countLid = 30;
 	}
@@ -2794,13 +2897,15 @@ static void NDS_applyFinalInput()
 			countLid--;
 	}
 
-	u16 padExt = (((u16 *)MMU.ARM7_REG)[0x136>>1] & 0x0070) |
-		((input.buttons.X ? 0 : 0x80) >> 7) |
+	u16 padExt = ((input.buttons.X ? 0 : 0x80) >> 7) |
 		((input.buttons.Y ? 0 : 0x80) >> 6) |
 		((input.buttons.G ? 0 : 0x80) >> 4) |
 		((LidClosed) << 7) |
 		0x0034;
 
+	padExt = LOCAL_TO_LE_16(padExt);
+	padExt |= (((u16 *)MMU.ARM7_REG)[0x136>>1] & 0x0070);
+	
 	((u16 *)MMU.ARM7_REG)[0x136>>1] = (u16)padExt;
 
 	InputDisplayString=MakeInputDisplayString(padExt, pad);
@@ -2823,6 +2928,12 @@ static void NDS_applyFinalInput()
 		((input.buttons.W ? 1 : 0) << 2)|
 		((input.buttons.E ? 1 : 0) << 1);
 
+	//these values are arbitrarily chosen to make arkanoid paddle control act similarly to its dpad controls
+	if(input.buttons.R)
+		nds.paddle += 5;
+	else if(input.buttons.L)
+		nds.paddle -= 5;
+
 	// TODO: low power IRQ
 }
 
@@ -2875,6 +2986,35 @@ void emu_halt() {
 #endif
 }
 
+//returns true if exmemcnt specifies satisfactory parameters for the device, which calls this function
+bool ValidateSlot2Access(u32 procnum, u32 demandSRAMSpeed, u32 demand1stROMSpeed, u32 demand2ndROMSpeed, int clockbits)
+{
+	static const u32 _sramSpeeds[] = {10,8,6,18};
+	static const u32 _rom1Speeds[] = {10,8,6,18};
+	static const u32 _rom2Speeds[] = {6,4};
+	u16 exmemcnt = T1ReadWord(MMU.MMU_MEM[procnum][0x40], 0x204);
+	u16 exmemcnt9 = T1ReadWord(MMU.MMU_MEM[ARMCPU_ARM9][0x40], 0x204);
+	u32 arm7access = (exmemcnt9 & EXMEMCNT_MASK_SLOT2_ARM7);
+	u32 sramSpeed = _sramSpeeds[(exmemcnt & EXMEMCNT_MASK_SLOT2_SRAM_TIME)];
+	u32 romSpeed1 = _rom1Speeds[(exmemcnt & EXMEMCNT_MASK_SLOT2_ROM_1ST_TIME)>>2];
+	u32 romSpeed2 = _rom2Speeds[(exmemcnt & EXMEMCNT_MASK_SLOT2_ROM_2ND_TIME)>>4];
+	u32 curclockbits = (exmemcnt & EXMEMCNT_MASK_SLOT2_CLOCKRATE)>>5;
+	
+	if(procnum==ARMCPU_ARM9 && arm7access) return false;
+	if(procnum==ARMCPU_ARM7 && !arm7access) return false;
+
+	//what we're interested in here is whether the rom/ram are too low -> too fast. then accesses won't have enough time to work.
+	//i'm not sure if this gives us enough flexibility, but it is good enough for now.
+	//should make the arguments to this function bitmasks later if we need better.
+	if(sramSpeed < demandSRAMSpeed) return false;
+	if(romSpeed1 < demand1stROMSpeed) return false;
+	if(romSpeed2 < demand2ndROMSpeed) return false;
+
+	if(clockbits != -1 && clockbits != (int)curclockbits) return false;
+
+	return true;
+}
+
 //these templates needed to be instantiated manually
 template void NDS_exec<FALSE>(s32 nb);
 template void NDS_exec<TRUE>(s32 nb);
