@@ -1,5 +1,6 @@
-/*  Copyright (C) 2006 yopyop
-    Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 #include "wifi.h"
 #include "emufile.h"
 #include "firmware.h"
+#include "types.h"
 
 #include <string>
 
@@ -155,6 +157,14 @@ enum ENSATA_HANDSHAKE
 	ENSATA_HANDSHAKE_complete = 4,
 };
 
+enum NDS_CONSOLE_TYPE
+{
+	NDS_CONSOLE_TYPE_FAT,
+	NDS_CONSOLE_TYPE_LITE,
+	NDS_CONSOLE_TYPE_IQUE,
+	NDS_CONSOLE_TYPE_DSI
+};
+
 struct NDSSystem
 {
 	s32 wifiCycle;
@@ -163,10 +173,19 @@ struct NDSSystem
 	u32 VCount;
 	u32 old;
 
-	u16 touchX;
-	u16 touchY;
+	//raw adc touch coords for old NDS
+	u16 adc_touchX;
+	u16 adc_touchY;
+	s32 adc_jitterctr;
+
+	//the DSI returns calibrated touch coords from its TSC (?), so we need to save these separately
+	u16 scr_touchX;
+	u16 scr_touchY;
+
 	BOOL isTouch;
 	u16 pad;
+	
+	u16 paddle;
 
 	u8 *FW_ARM9BootCode;
 	u8 *FW_ARM7BootCode;
@@ -177,7 +196,7 @@ struct NDSSystem
 
 	BOOL sleeping;
 	BOOL cardEjected;
-	BOOL freezeBus;
+	u32 freezeBus;
 
 	//this is not essential NDS runtime state.
 	//it was perhaps a mistake to put it here.
@@ -188,9 +207,12 @@ struct NDSSystem
 	s32 idleFrameCounter;
 	s32 cpuloopIterationCount; //counts the number of times during a frame that a reschedule happened
 
-	//if the game was booted on a debug console, this is set
-	BOOL debugConsole;
-
+	//console type must be copied in when the system boots. it can't be changed on the fly.
+	int ConsoleType;
+	bool Is_DSI() { return ConsoleType == NDS_CONSOLE_TYPE_DSI; }
+	bool Is_DebugConsole() { return _DebugConsole!=0; }
+	BOOL _DebugConsole;
+	
 	//set if the user requests ensata emulation
 	BOOL ensataEmulation;
 
@@ -222,19 +244,12 @@ struct NDS_fw_touchscreen_cal {
   u8 screen_y;
 };
 
-/** /brief The type of DS
- */
-enum nds_fw_ds_type {
-  NDS_FW_DS_TYPE_FAT,
-  NDS_FW_DS_TYPE_LITE,
-  NDS_FW_DS_TYPE_iQue
-};
-
 #define MAX_FW_NICKNAME_LENGTH 10
 #define MAX_FW_MESSAGE_LENGTH 26
 
-struct NDS_fw_config_data {
-  enum nds_fw_ds_type ds_type;
+struct NDS_fw_config_data
+{
+  NDS_CONSOLE_TYPE ds_type;
 
   u8 fav_colour;
   u8 birth_month;
@@ -248,8 +263,8 @@ struct NDS_fw_config_data {
 
   u8 language;
 
-  /* touchscreen calibration */
-  struct NDS_fw_touchscreen_cal touch_cal[2];
+  //touchscreen calibration
+  NDS_fw_touchscreen_cal touch_cal[2];
 };
 
 extern NDSSystem nds;
@@ -281,14 +296,14 @@ struct RomBanner
 	enum { NUM_TITLES = 6 };
 	union {
 		struct {
-			wchar_t title_jp[0x80]; //Title 0 Japanese (128 characters, 16bit Unicode)
-			wchar_t title_en[0x80]; //Title 1 English  ("")
-			wchar_t title_fr[0x80]; //Title 2 French   ("")
-			wchar_t title_de[0x80]; //Title 3 German   ("")
-			wchar_t title_it[0x80]; //Title 4 Italian  ("")
-			wchar_t title_es[0x80]; //Title 5 Spanish  ("")
+			u16 title_jp[0x80]; //Title 0 Japanese (128 characters, 16bit Unicode)
+			u16 title_en[0x80]; //Title 1 English  ("")
+			u16 title_fr[0x80]; //Title 2 French   ("")
+			u16 title_de[0x80]; //Title 3 German   ("")
+			u16 title_it[0x80]; //Title 4 Italian  ("")
+			u16 title_es[0x80]; //Title 5 Spanish  ("")
 		};
-		wchar_t titles[NUM_TITLES][0x80];
+		u16 titles[NUM_TITLES][0x80];
 	};
 	u8 end0xFF[0x1C0]; 
   //840h  ?    (Maybe newer/chinese firmware do also support chinese title?)
@@ -344,11 +359,12 @@ struct GameInfo
 	u32 mask;
 	const RomBanner& getRomBanner();
 	bool hasRomBanner();
+	bool isHomebrew;
 };
 
 typedef struct TSCalInfo
 {
-	struct adc
+	struct
 	{
 		u16 x1, x2;
 		u16 y1, y2;
@@ -356,7 +372,7 @@ typedef struct TSCalInfo
 		u16 height;
 	} adc;
 
-	struct scr
+	struct
 	{
 		u8 x1, x2;
 		u8 y1, y2;
@@ -429,7 +445,8 @@ void NDS_suspendProcessingInput(bool suspend);
 int NDS_LoadROM(const char *filename, const char* logicalFilename=0);
 void NDS_FreeROM(void);
 void NDS_Reset();
-int NDS_ImportSave(const char *filename);
+int NDS_ImportSave(const char *filename, u32 force_size = 0);
+u32 NDS_ImportSaveSize(const char *filename);
 bool NDS_ExportSave(const char *filename);
 
 void nds_savestate(EMUFILE* os);
@@ -469,6 +486,7 @@ extern struct TCommonSettings {
 		, GFX3D_EdgeMark(true)
 		, GFX3D_Fog(true)
 		, GFX3D_Texture(true)
+		, GFX3D_LineHack(true)
 		, GFX3D_Zelda_Shadow_Depth_Hack(0)
 		, UseExtBIOS(false)
 		, SWIFromBIOS(false)
@@ -484,8 +502,11 @@ extern struct TCommonSettings {
 		, micMode(InternalNoise)
 		, spuInterpolationMode(SPUInterpolation_Linear)
 		, manualBackupType(0)
+		, autodetectBackupMethod(0)
 		, spu_captureMuted(false)
 		, spu_advanced(false)
+		, StylusPressure(50)
+		, ConsoleType(NDS_CONSOLE_TYPE_FAT)
 	{
 		strcpy(ARM9BIOS, "biosnds9.bin");
 		strcpy(ARM7BIOS, "biosnds7.bin");
@@ -507,6 +528,7 @@ extern struct TCommonSettings {
 	bool GFX3D_EdgeMark;
 	bool GFX3D_Fog;
 	bool GFX3D_Texture;
+	bool GFX3D_LineHack;
 	int  GFX3D_Zelda_Shadow_Depth_Hack;
 
 	bool UseExtBIOS;
@@ -520,6 +542,7 @@ extern struct TCommonSettings {
 	bool BootFromFirmware;
 	struct NDS_fw_config_data InternalFirmConf;
 
+	NDS_CONSOLE_TYPE ConsoleType;
 	bool DebugConsole;
 	bool EnsataEmulation;
 	
@@ -529,6 +552,8 @@ extern struct TCommonSettings {
 	bool single_core() { return num_cores==1; }
 	bool rigorous_timing;
 
+	int StylusPressure;
+
 	bool dispLayers[2][5];
 	
 	FAST_ALIGN bool advanced_timing;
@@ -552,6 +577,7 @@ extern struct TCommonSettings {
 	//this is a temporary hack until we straighten out the flushing logic and/or gxfifo
 	//int gfx3d_flushMode;
 
+	int autodetectBackupMethod;
 	//this is the user's choice of manual backup type, for cases when the autodetection can't be trusted
 	int manualBackupType;
 
@@ -591,5 +617,9 @@ void MovieSRAM();
 
 void ClearAutoHold(void);
 
+bool ValidateSlot2Access(u32 procnum, u32 demandSRAMSpeed, u32 demand1stROMSpeed, u32 demand2ndROMSpeed, int clockbits);
+
+extern ADVANsCEne	advsc;
+
 #endif
 
