@@ -1,35 +1,35 @@
 /*
 	Copyright (C) 2006 yopyop
 	Copyright (C) 2006-2007 shash
-	Copyright (C) 2008-2010 DeSmuME team
+	Copyright (C) 2008-2011 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the 
-	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 //problem - alpha-on-alpha texture rendering might work but the dest alpha buffer isnt tracked correctly
 //due to zeromus not having any idea how to set dest alpha blending in opengl.
 //so, it doesnt composite to 2d correctly.
 //(re: new super mario brothers renders the stormclouds at the beginning)
+//!!! fixed on rev.3996
 
 #include <queue>
 
 #include "OGLRender.h"
 #include "debug.h"
 
+CACHE_ALIGN float material_8bit_to_float[255] = {0};
+
 bool (*oglrender_init)() = 0;
 bool (*oglrender_beginOpenGL)() = 0;
 void (*oglrender_endOpenGL)() = 0;
@@ -76,7 +76,7 @@ static void ENDGL() {
 #include "shaders.h"
 #include "texcache.h"
 
-static ALIGN(16) u8  GPU_screen3D			[256*192*4];
+static DS_ALIGN(16) u8  GPU_screen3D			[256*192*4];
 
 static const unsigned short map3d_cull[4] = {GL_FRONT_AND_BACK, GL_FRONT, GL_BACK, 0};
 static const int texEnv[4] = { GL_MODULATE, GL_DECAL, GL_MODULATE, GL_MODULATE };
@@ -95,6 +95,17 @@ static bool isTranslucent;
 
 static u32 textureFormat=0, texturePalette=0;
 
+// ClearImage/Rear-plane (FBO)
+GLenum	oglClearImageTextureID[2] = {0};	// 0 - image, 1 - depth
+GLuint	oglClearImageBuffers = 0;
+GLuint	oglClearImageRender[1] = {0};
+bool	oglFBOdisabled = false;
+u32		*oglClearImageColor = NULL;
+float	*oglClearImageDepth = NULL;
+u16		*oglClearImageColorTemp = NULL;
+u16		*oglClearImageDepthTemp = NULL;
+u32		oglClearImageScrollOld = 0;
+
 //------------------------------------------------------------
 
 #define OGLEXT(x,y) x y = 0;
@@ -124,10 +135,19 @@ OGLEXT(PFNGLDELETEPROGRAMPROC,glDeleteProgram)
 OGLEXT(PFNGLGETPROGRAMIVPROC,glGetProgramiv)
 OGLEXT(PFNGLGETPROGRAMINFOLOGPROC,glGetProgramInfoLog)
 OGLEXT(PFNGLVALIDATEPROGRAMPROC,glValidateProgram)
-OGLEXT(PFNGLBLENDFUNCSEPARATEEXTPROC,glBlendFuncSeparateEXT)
+OGLEXT(PFNGLBLENDFUNCSEPARATEPROC,glBlendFuncSeparate)
+OGLEXT(PFNGLBLENDEQUATIONSEPARATEPROC,glBlendEquationSeparate)
 OGLEXT(PFNGLGETUNIFORMLOCATIONPROC,glGetUniformLocation)
 OGLEXT(PFNGLUNIFORM1IPROC,glUniform1i)
 OGLEXT(PFNGLUNIFORM1IVPROC,glUniform1iv)
+// FBO
+OGLEXT(PFNGLGENFRAMEBUFFERSEXTPROC,glGenFramebuffersEXT);
+OGLEXT(PFNGLBINDFRAMEBUFFEREXTPROC,glBindFramebufferEXT);
+OGLEXT(PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC,glFramebufferRenderbufferEXT);
+OGLEXT(PFNGLFRAMEBUFFERTEXTURE2DEXTPROC,glFramebufferTexture2DEXT);
+OGLEXT(PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC,glCheckFramebufferStatusEXT);
+OGLEXT(PFNGLDELETEFRAMEBUFFERSEXTPROC,glDeleteFramebuffersEXT);
+OGLEXT(PFNGLBLITFRAMEBUFFEREXTPROC,glBlitFramebufferEXT);
 #endif
 
 #if !defined(GL_VERSION_1_3) || defined(_MSC_VER) || defined(__INTEL_COMPILER)
@@ -258,8 +278,9 @@ GLuint vertexShaderID;
 GLuint fragmentShaderID;
 GLuint shaderProgram;
 
-static GLuint hasTexLoc;
-static GLuint texBlendLoc;
+static GLint hasTexLoc;
+static GLint texBlendLoc;
+static GLint oglWBuffer;
 static bool hasTexture = false;
 
 static TexCacheItem* currTexture = NULL;
@@ -334,7 +355,7 @@ static void OGLReset()
 		glUniform1i(hasTexLoc, 0);
 		hasTexture = false;
 		glUniform1i(texBlendLoc, 0);
-		
+		glUniform1i(oglWBuffer, 0);
 	}
 
 	TexCache_Reset();
@@ -344,6 +365,12 @@ static void OGLReset()
 
 //	memset(GPU_screenStencil,0,sizeof(GPU_screenStencil));
 	memset(GPU_screen3D,0,sizeof(GPU_screen3D));
+
+	memset(oglClearImageColor, 0, 256*192*sizeof(u32));
+	memset(oglClearImageDepth, 0, 256*192*sizeof(float));
+	memset(oglClearImageColorTemp, 0, 256*192*sizeof(u16));
+	memset(oglClearImageDepthTemp, 0, 256*192*sizeof(u16));
+	oglClearImageScrollOld = 0;
 }
 
 //static class OGLTexCacheUser : public ITexCacheUser
@@ -390,7 +417,7 @@ static void expandFreeTextures()
 
 static char OGLInit(void)
 {
-	GLuint loc = 0;
+	GLint loc = 0;
 
 	if(!oglrender_init)
 		return 0;
@@ -400,6 +427,9 @@ static char OGLInit(void)
 	if(!BEGINGL())
 		return 0;
 
+	for (u8 i = 0; i < 255; i++)
+		material_8bit_to_float[i] = (float)(i<<2)/255.f;
+
 	expandFreeTextures();
 
 	glPixelStorei(GL_PACK_ALIGNMENT,8);
@@ -434,10 +464,19 @@ static char OGLInit(void)
 	INITOGLEXT(PFNGLGETPROGRAMIVPROC,glGetProgramiv)
 	INITOGLEXT(PFNGLGETPROGRAMINFOLOGPROC,glGetProgramInfoLog)
 	INITOGLEXT(PFNGLVALIDATEPROGRAMPROC,glValidateProgram)
+	// FBO
+	INITOGLEXT(PFNGLGENFRAMEBUFFERSEXTPROC,glGenFramebuffersEXT);
+	INITOGLEXT(PFNGLBINDFRAMEBUFFEREXTPROC,glBindFramebufferEXT);
+	INITOGLEXT(PFNGLFRAMEBUFFERRENDERBUFFEREXTPROC,glFramebufferRenderbufferEXT);
+	INITOGLEXT(PFNGLFRAMEBUFFERTEXTURE2DEXTPROC,glFramebufferTexture2DEXT);
+	INITOGLEXT(PFNGLCHECKFRAMEBUFFERSTATUSEXTPROC,glCheckFramebufferStatusEXT);
+	INITOGLEXT(PFNGLDELETEFRAMEBUFFERSEXTPROC,glDeleteFramebuffersEXT);
+	INITOGLEXT(PFNGLBLITFRAMEBUFFEREXTPROC,glBlitFramebufferEXT);
 #ifdef HAVE_LIBOSMESA
-	glBlendFuncSeparateEXT = NULL;
+	glBlendFuncSeparate = NULL;
 #else
-	INITOGLEXT(PFNGLBLENDFUNCSEPARATEEXTPROC,glBlendFuncSeparateEXT)
+	INITOGLEXT(PFNGLBLENDFUNCSEPARATEPROC,glBlendFuncSeparate)
+	INITOGLEXT(PFNGLBLENDEQUATIONSEPARATEPROC,glBlendEquationSeparate)
 #endif
 	INITOGLEXT(PFNGLGETUNIFORMLOCATIONPROC,glGetUniformLocation)
 	INITOGLEXT(PFNGLUNIFORM1IPROC,glUniform1i)
@@ -463,13 +502,22 @@ static char OGLInit(void)
 		hasTexLoc = glGetUniformLocation(shaderProgram, "hasTexture");
 
 		texBlendLoc = glGetUniformLocation(shaderProgram, "texBlending");
+
+		oglWBuffer = glGetUniformLocation(shaderProgram, "oglWBuffer");
 	}
 
 	//we want to use alpha destination blending so we can track the last-rendered alpha value
-	if(glBlendFuncSeparateEXT != NULL)
+	if(glBlendFuncSeparate != NULL)
 	{
-		glBlendFuncSeparateEXT(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_DST_ALPHA);
-	//	glBlendFuncSeparateEXT(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
+		if (glBlendEquationSeparate != NULL)
+		{
+			// test: new super mario brothers renders the stormclouds at the beginning 
+			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA, GL_DST_ALPHA);
+			glBlendEquationSeparate( GL_FUNC_ADD, GL_MAX );
+		}
+		else
+			glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_DST_ALPHA);
+		
 	}
 
 	if(hasShaders)
@@ -480,8 +528,63 @@ static char OGLInit(void)
 		glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 		glTexParameterf(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 		glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP); //clamp so that we dont run off the edges due to 1.0 -> [0,31] math
+
+		// Restore Toon table
+		u32 rgbToonTable[32];
+		for(int i=0;i<32;i++)
+			rgbToonTable[i] = RGB15TO32_NOALPHA(gfx3d.renderState.u16ToonTable[i]);
+		glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGBA, GL_UNSIGNED_BYTE, &rgbToonTable[0]);
+		gfx3d.state.invalidateToon = false;
 	}
 
+	// ClearImage/Rear-plane
+	glGenTextures (2, &oglClearImageTextureID[0]);
+	glActiveTexture(GL_TEXTURE2);
+	glBindTexture(GL_TEXTURE_2D, oglClearImageTextureID[0]);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 256, 192, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
+
+	glBindTexture(GL_TEXTURE_2D, oglClearImageTextureID[1]);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_BORDER);
+	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_NONE);
+	glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE, GL_LUMINANCE);
+	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, 256, 192, 0,  GL_DEPTH_COMPONENT, GL_FLOAT, NULL);
+
+	// FBO
+	oglFBOdisabled = false;
+	glGenFramebuffersEXT(1, &oglClearImageBuffers);
+
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, oglClearImageBuffers);
+	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, oglClearImageTextureID[0], 0);
+
+	//glDrawBuffer(GL_NONE);
+	//glReadBuffer(GL_NONE);
+	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_2D, oglClearImageTextureID[1], 0);
+	
+	if (glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT)==GL_FRAMEBUFFER_COMPLETE_EXT)
+		INFO("Successfully created OpenGL Framebuffer object (FBO)\n");
+	else
+	{
+		INFO("Failed to created OpenGL Framebuffer object (FBO): ClearImage disabled\n");
+		oglFBOdisabled = true;
+	}
+
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+
+	oglClearImageColor = new u32[256*192];
+	oglClearImageColorTemp = new u16[256*192];
+	oglClearImageDepth = new float[256*192];
+	oglClearImageDepthTemp = new u16[256*192];
+
+	glActiveTexture(GL_TEXTURE0);
+
 	OGLReset();
 
 	ENDGL();
@@ -520,6 +623,35 @@ static void OGLClose()
 	//glDeleteTextures(MAX_TEXTURE, &oglTempTextureID[0]);
 	glDeleteTextures(1, &oglToonTableTextureID);
 
+	// FBO
+	glDeleteTextures(2, &oglClearImageTextureID[0]);
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+	glDeleteFramebuffersEXT(1, &oglClearImageBuffers);
+
+	if (oglClearImageColor)
+	{
+		delete [] oglClearImageColor;
+		oglClearImageColor = NULL;
+	}
+
+	if (oglClearImageDepth)
+	{
+		delete [] oglClearImageDepth;
+		oglClearImageDepth = NULL;
+	}
+
+	if (oglClearImageColorTemp)
+	{
+		delete [] oglClearImageColorTemp;
+		oglClearImageColorTemp = NULL;
+	}
+
+	if (oglClearImageDepthTemp)
+	{
+		delete [] oglClearImageDepthTemp;
+		oglClearImageDepthTemp = NULL;
+	}
+
 	ENDGL();
 }
 
@@ -611,13 +743,15 @@ static void BeginRenderPoly()
 	xglDepthFunc (depthFuncMode);
 
 	// Cull face
-	if (cullingMask != 0xC0)
+	if (cullingMask == 0x03)
 	{
-		xglEnable(GL_CULL_FACE);
-		glCullFace(map3d_cull[cullingMask>>6]);
+		xglDisable(GL_CULL_FACE);
 	}
 	else
-		xglDisable(GL_CULL_FACE);
+	{
+		xglEnable(GL_CULL_FACE);
+		glCullFace(map3d_cull[cullingMask]);
+	}
 
 	if (!wireframe)
 	{
@@ -694,7 +828,7 @@ static void BeginRenderPoly()
 	xglDepthMask(enableDepthWrite?GL_TRUE:GL_FALSE);
 }
 
-static void InstallPolygonAttrib(unsigned long val)
+static void InstallPolygonAttrib(u32 val)
 {
 	// Light enable/disable
 	lightMask = (val&0xF);
@@ -709,7 +843,7 @@ static void InstallPolygonAttrib(unsigned long val)
 	depthFuncMode = depthFunc[BIT14(val)];
 
 	// back face culling
-	cullingMask = (val&0xC0);
+	cullingMask = (val >> 6) & 0x03;
 
 	alpha31 = ((val>>16)&0x1F)==31;
 	
@@ -812,9 +946,75 @@ static void GL_ReadFramebuffer()
 #endif
 }
 
+// TODO: optimize
+// Tested:	Sonic Chronicles Dark Brotherhood
+//			The Chronicles of Narnia - The Lion, The Witch and The Wardrobe
+//			Harry Potter and the Order of the Phoenix
+static void oglClearImageFBO()
+{
+	//printf("enableClearImage\n");
+	u16* clearImage = (u16*)MMU.texInfo.textureSlotAddr[2];
+	u16* clearDepth = (u16*)MMU.texInfo.textureSlotAddr[3];
+	u16 scroll = T1ReadWord(MMU.ARM9_REG,0x356); //CLRIMAGE_OFFSET
+
+	if ((oglClearImageScrollOld != scroll)||
+		(memcmp(clearImage, oglClearImageColorTemp, 256*192*2) != 0) ||
+			(memcmp(clearDepth, oglClearImageDepthTemp, 256*192*2) != 0))
+	{
+		oglClearImageScrollOld = scroll;
+		memcpy(oglClearImageColorTemp, clearImage, 256*192*2);
+		memcpy(oglClearImageDepthTemp, clearDepth, 256*192*2);
+		
+		u16 xscroll = scroll&0xFF;
+		u16 yscroll = (scroll>>8)&0xFF;
+
+		u32 dd = 256*192-256;
+		for(int iy=0;iy<192;iy++) 
+		{
+			int y = ((iy + yscroll)&255)<<8;
+			for(int ix=0;ix<256;ix++)
+			{
+				int x = (ix + xscroll)&255;
+				int adr = y + x;
+				
+				u16 col = clearImage[adr];
+				oglClearImageColor[dd] = RGB15TO32(col,255*(col>>15));
+				
+				u16 depth = clearDepth[adr] & 0x7FFF;
+
+				oglClearImageDepth[dd] = (float)gfx3d_extendDepth_15_to_24(depth) / (float)0x00FFFFFF;
+				dd++;
+			}
+			dd-=256*2;
+		}
+
+		// color
+		glActiveTexture(GL_TEXTURE2);
+		glBindTexture(GL_TEXTURE_2D, oglClearImageTextureID[0]);
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 256, 192, 0,  GL_RGBA, GL_UNSIGNED_BYTE, &oglClearImageColor[0]);
+
+		// depth
+		glBindTexture(GL_TEXTURE_2D, oglClearImageTextureID[1]);
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT, 256, 192, 0,  GL_DEPTH_COMPONENT, GL_FLOAT, &oglClearImageDepth[0]);
+
+		glActiveTexture(GL_TEXTURE0);
+	}
+	
+
+	// color & depth
+	glBindFramebufferEXT(GL_READ_FRAMEBUFFER_EXT, oglClearImageBuffers);
+	glBindFramebufferEXT(GL_DRAW_FRAMEBUFFER_EXT, 0);
+	glBlitFramebufferEXT(0,0,256,192,0,0,256,192,GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT,GL_LINEAR);
+
+	glBindFramebufferEXT(GL_READ_FRAMEBUFFER_EXT, 0);
+	glBindFramebufferEXT(GL_DRAW_FRAMEBUFFER_EXT, 0);
+
+}
 
 static void OGLRender()
 {
+	float alpha = 1.0f;
+
 	if(!BEGINGL()) return;
 
 	Control();
@@ -837,20 +1037,31 @@ static void OGLRender()
 			glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, 32, 0, GL_RGBA, GL_UNSIGNED_BYTE, &rgbToonTable[0]);
 			gfx3d.state.invalidateToon = false;
 		}
+
+		glUniform1i(oglWBuffer, gfx3d.renderState.wbuffer);
 	}
 
 	xglDepthMask(GL_TRUE);
 
-	float clearColor[4] = {
-		((float)(gfx3d.renderState.clearColor&0x1F))/31.0f,
-		((float)((gfx3d.renderState.clearColor>>5)&0x1F))/31.0f,
-		((float)((gfx3d.renderState.clearColor>>10)&0x1F))/31.0f,
-		((float)((gfx3d.renderState.clearColor>>16)&0x1F))/31.0f,
-	};
-	glClearColor(clearColor[0],clearColor[1],clearColor[2],clearColor[3]);
-	glClearDepth(gfx3d.renderState.clearDepth);
 	glClearStencil((gfx3d.renderState.clearColor >> 24) & 0x3F);
-	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
+	u32 clearFlag = GL_STENCIL_BUFFER_BIT;
+
+	if (!oglFBOdisabled && gfx3d.renderState.enableClearImage)
+			oglClearImageFBO();
+	else
+	{
+		float clearColor[4] = {
+			((float)(gfx3d.renderState.clearColor&0x1F))/31.0f,
+			((float)((gfx3d.renderState.clearColor>>5)&0x1F))/31.0f,
+			((float)((gfx3d.renderState.clearColor>>10)&0x1F))/31.0f,
+			((float)((gfx3d.renderState.clearColor>>16)&0x1F))/31.0f,
+		};
+		glClearColor(clearColor[0],clearColor[1],clearColor[2],clearColor[3]);
+		glClearDepth((float)gfx3d.renderState.clearDepth / (float)0x00FFFFFF);
+		clearFlag |= GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT;
+	}
+	
+	glClear(clearFlag);
 
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
@@ -872,47 +1083,13 @@ static void OGLRender()
 			if(i==0 || lastTextureFormat != poly->texParam || lastTexturePalette != poly->texPalette || lastPolyAttr != poly->polyAttr)
 			{
 				isTranslucent = poly->isTranslucent();
-				InstallPolygonAttrib(lastPolyAttr=poly->polyAttr);
+				InstallPolygonAttrib(poly->polyAttr);
 				lastTextureFormat = textureFormat = poly->texParam;
 				lastTexturePalette = texturePalette = poly->texPalette;
 				lastPolyAttr = poly->polyAttr;
 				BeginRenderPoly();
 			}
 			
-			//since we havent got the whole pipeline working yet, lets use opengl for the projection
-		/*	if(lastProjIndex != poly->projIndex) {
-				glMatrixMode(GL_PROJECTION);
-				glLoadMatrixf(gfx3d.projlist->projMatrix[poly->projIndex]);
-				lastProjIndex = poly->projIndex;
-			}*/
-
-		/*	glBegin(type==3?GL_TRIANGLES:GL_QUADS);
-			for(int j=0;j<type;j++) {
-				VERT* vert = &gfx3d.vertlist->list[poly->vertIndexes[j]];
-				u8 color[4] = {
-					material_5bit_to_8bit[vert->color[0]],
-					material_5bit_to_8bit[vert->color[1]],
-					material_5bit_to_8bit[vert->color[2]],
-					material_5bit_to_8bit[vert->color[3]]
-				};
-				
-				//float tempCoord[4];
-				//Vector4Copy(tempCoord, vert->coord);
-				//we havent got the whole pipeline working yet, so we cant do this
-				////convert from ds device coords to opengl
-				//tempCoord[0] *= 2;
-				//tempCoord[1] *= 2;
-				//tempCoord[0] -= 1;
-				//tempCoord[1] -= 1;
-
-				//todo - edge flag?
-				glTexCoord2fv(vert->texcoord);
-				glColor4ubv((GLubyte*)color);
-				//glVertex4fv(tempCoord);
-				glVertex4fv(vert->coord);
-			}
-			glEnd();*/
-		
 			if(lastViewport != poly->viewport)
 			{
 				VIEWPORT viewport;
@@ -921,50 +1098,25 @@ static void OGLRender()
 				lastViewport = poly->viewport;
 			}
 
-			glBegin(GL_TRIANGLES);
+			if(wireframe || !isTranslucent) alpha = 1.0f;
+			else 
+				alpha = poly->getAlpha()/31.0f;
 
-			VERT *vert0 = &gfx3d.vertlist->list[poly->vertIndexes[0]];
-			float alpha = poly->getAlpha()/31.0f;
-			if(wireframe) alpha = 1.0;
-			float color0[4] = {
-					(vert0->color[0]<<2)/255.0f,
-					(vert0->color[1]<<2)/255.0f,
-					(vert0->color[2]<<2)/255.0f,
-					alpha
-				};
+			GLenum frm[] = {GL_TRIANGLES, GL_QUADS, GL_TRIANGLE_STRIP, GL_QUADS,	//TODO: GL_QUAD_STRIP
+							GL_LINE_LOOP, GL_LINE_LOOP, GL_LINE_STRIP, GL_LINE_STRIP};
 
-			//this draws things as a fan to prepare for the day when the clipping is done in gfx3d
-			//and funny shaped polys find their way into here.
-			//of course it could really be drawn as a fan, i suppose.. i dont remember why we did it this way
-			for(int j = 1; j < (type-1); j++)
+			glBegin(frm[poly->vtxFormat]);
+
+			for(int j = 0; j < type; j++)
 			{
-				VERT *vert1 = &gfx3d.vertlist->list[poly->vertIndexes[j]];
-				VERT *vert2 = &gfx3d.vertlist->list[poly->vertIndexes[j+1]];
+				VERT *vert = &gfx3d.vertlist->list[poly->vertIndexes[j]];
 				
-				float color1[4] = {
-					(vert1->color[0]<<2)/255.0f,
-					(vert1->color[1]<<2)/255.0f,
-					(vert1->color[2]<<2)/255.0f,
-					alpha
-				};
-				float color2[4] = {
-					(vert2->color[0]<<2)/255.0f,
-					(vert2->color[1]<<2)/255.0f,
-					(vert2->color[2]<<2)/255.0f,
-					alpha
-				};
-
-				glTexCoord2fv(vert0->texcoord);
-				glColor4fv((GLfloat*)color0);
-				glVertex4fv(vert0->coord);
-
-				glTexCoord2fv(vert1->texcoord);
-				glColor4fv((GLfloat*)color1);
-				glVertex4fv(vert1->coord);
-
-				glTexCoord2fv(vert2->texcoord);
-				glColor4fv((GLfloat*)color2);
-				glVertex4fv(vert2->coord);
+				glTexCoord2fv(vert->texcoord);
+				glColor4f(material_8bit_to_float[vert->color[0]], 
+							material_8bit_to_float[vert->color[1]], 
+							material_8bit_to_float[vert->color[2]],
+							alpha);
+				glVertex4fv(vert->coord);
 			}
 
 			glEnd();
