@@ -1,27 +1,25 @@
-/*  SPU.cpp
-    Copyright (C) 2006 yopyop
-    Copyright (C) 2006 Theo Berkau
-    Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2006 Theo Berkau
+	Copyright (C) 2008-2012 DeSmuME team
 
-    Ideas borrowed from Stephane Dallongeville's SCSP core
+	Ideas borrowed from Stephane Dallongeville's SCSP core
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+
 #define _USE_MATH_DEFINES
 #include <math.h>
 #ifndef M_PI
@@ -45,6 +43,7 @@
 #include "metaspu/metaspu.h"
 
 #define K_ADPCM_LOOPING_RECOVERY_INDEX 99999
+#define COSINE_INTERPOLATION_RESOLUTION 8192
 
 //#ifdef FASTBUILD
 	#undef FORCEINLINE
@@ -60,9 +59,11 @@ int SPU_currentCoreNum = SNDCORE_DUMMY;
 static int volume = 100;
 
 
+static int buffersize = 0;
 static ESynchMode synchmode = ESynchMode_DualSynchAsynch;
 static ESynchMethod synchmethod = ESynchMethod_N;
 
+static int SNDCoreId=-1;
 static SoundInterface_struct *SNDCore=NULL;
 extern SoundInterface_struct *SNDCoreList[];
 
@@ -101,6 +102,7 @@ static const s16 wavedutytbl[8][8] = {
 
 static s32 precalcdifftbl[89][16];
 static u8 precalcindextbl[89][8];
+static double cos_lut[COSINE_INTERPOLATION_RESOLUTION];
 
 static const double ARM7_CLOCK = 33513982;
 
@@ -125,7 +127,9 @@ int SPU_ChangeSoundCore(int coreid, int buffersize)
 {
 	int i;
 
-	delete SPU_user; SPU_user = 0;
+	::buffersize = buffersize;
+
+	delete SPU_user; SPU_user = NULL;
 
 	// Make sure the old core is freed
 	if (SNDCore)
@@ -148,6 +152,8 @@ int SPU_ChangeSoundCore(int coreid, int buffersize)
 		}
 	}
 
+	SNDCoreId = coreid;
+
 	//If the user picked the dummy core, disable the user spu
 	if(SNDCore == &SNDDummy)
 		return 0;
@@ -163,11 +169,10 @@ int SPU_ChangeSoundCore(int coreid, int buffersize)
 		return -1;
 	}
 
-	//enable the user spu
-	SPU_user = new SPU_struct(buffersize);
-
 	SNDCore->SetVolume(volume);
 
+	SPU_SetSynchMode(synchmode,synchmethod);
+
 	return 0;
 }
 
@@ -176,14 +181,18 @@ SoundInterface_struct *SPU_SoundCore()
 	return SNDCore;
 }
 
-//static double cos_lut[256];
+void SPU_ReInit()
+{
+	SPU_Init(SNDCoreId, buffersize);
+}
+
 int SPU_Init(int coreid, int buffersize)
 {
 	int i, j;
-
-	//for some reason we dont use the cos lut anymore... did someone decide it was slow?
-	//for(int i=0;i<256;i++)
-	//	cos_lut[i] = cos(i/256.0*M_PI);
+	
+	// Build the cosine interpolation LUT
+	for(unsigned int i = 0; i < COSINE_INTERPOLATION_RESOLUTION; i++)
+		cos_lut[i] = (1.0 - cos(((double)i/(double)COSINE_INTERPOLATION_RESOLUTION) * M_PI)) * 0.5;
 
 	SPU_core = new SPU_struct((int)ceil(samples_per_hline));
 	SPU_Reset();
@@ -218,6 +227,15 @@ void SPU_Pause(int pause)
 		SNDCore->UnMuteAudio();
 }
 
+void SPU_CloneUser()
+{
+	if(SPU_user) {
+		memcpy(SPU_user->channels,SPU_core->channels,sizeof(SPU_core->channels));
+		SPU_user->regs = SPU_core->regs;
+	}
+}
+
+
 void SPU_SetSynchMode(int mode, int method)
 {
 	synchmode = (ESynchMode)mode;
@@ -229,6 +247,15 @@ void SPU_SetSynchMode(int mode, int method)
 		  // or maybe not, maybe the platform-specific code that calls this function can deal with it.
 		synchronizer = metaspu_construct(synchmethod);
 	}
+
+	delete SPU_user;
+	SPU_user = NULL;
+		
+	if(synchmode == ESynchMode_DualSynchAsynch)
+	{
+		SPU_user = new SPU_struct(buffersize);
+		SPU_CloneUser();
+	}
 }
 
 void SPU_ClearOutputBuffer()
@@ -759,28 +786,38 @@ void SPU_WriteLong(u32 addr, u32 val)
 	addr &= 0xFFF;
 
 	SPU_core->WriteLong(addr,val);
-	if(SPU_user) SPU_user->WriteLong(addr,val);
+	if(SPU_user) 
+		SPU_user->WriteLong(addr,val);
 }
 
 //////////////////////////////////////////////////////////////////////////////
 
-template<SPUInterpolationMode INTERPOLATE_MODE> static FORCEINLINE s32 Interpolate(s32 a, s32 b, double _ratio)
+template<SPUInterpolationMode INTERPOLATE_MODE> static FORCEINLINE s32 Interpolate(s32 a, s32 b, double ratio)
 {
-	float ratio = (float)_ratio;
-	if(INTERPOLATE_MODE == SPUInterpolation_Cosine)
+	double sampleA = (double)a;
+	double sampleB = (double)b;
+	ratio = ratio - sputrunc(ratio);
+	
+	switch (INTERPOLATE_MODE)
 	{
-		//why did we change it away from the lookup table? somebody should research that
-		ratio = ratio - (int)ratio;
-		double ratio2 = ((1.0 - cos(ratio * M_PI)) * 0.5);
-		//double ratio2 = (1.0f - cos_lut[((int)(ratio*256.0))&0xFF]) / 2.0f;
-		return (s32)(((1-ratio2)*a) + (ratio2*b));
-	}
-	else
-	{
-		//linear interpolation
-		ratio = ratio - sputrunc(ratio);
-		return s32floor((1-ratio)*a + ratio*b);
+		case SPUInterpolation_Cosine:
+			// Cosine Interpolation Formula:
+			// ratio2 = (1 - cos(ratio * M_PI)) / 2
+			// sampleI = sampleA * (1 - ratio2) + sampleB * ratio2
+			return s32floor((cos_lut[(unsigned int)(ratio * (double)COSINE_INTERPOLATION_RESOLUTION)] * (sampleB - sampleA)) + sampleA);
+			break;
+			
+		case SPUInterpolation_Linear:
+			// Linear Interpolation Formula:
+			// sampleI = sampleA * (1 - ratio) + sampleB * ratio
+			return s32floor((ratio * (sampleB - sampleA)) + sampleA);
+			break;
+			
+		default:
+			break;
 	}
+	
+	return a;
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -1360,35 +1397,43 @@ void SPU_Emulate_core()
 	bool mix = driver->AVI_IsRecording() || driver->WAV_IsRecording() || synchronize;
 
 	SPU_MixAudio(mix,SPU_core,spu_core_samples);
-	if(synchronize && SPU_user)
+	if(synchronize && SPU_currentCoreNum != SNDCORE_DUMMY)
 		synchronizer->enqueue_samples(SPU_core->outbuf, spu_core_samples);
 }
 
 void SPU_Emulate_user(bool mix)
 {
-	if(!SPU_user)
-		return;
-
 	u32 audiosize;
 
 	// Check to see how much free space there is
 	// If there is some, fill up the buffer
+	if(!SNDCore) return;
 	audiosize = SNDCore->GetAudioSpace();
 
 	if (audiosize > 0)
 	{
 		//printf("mix %i samples\n", audiosize);
-		if (audiosize > SPU_user->bufsize)
-			audiosize = SPU_user->bufsize;
+		if (audiosize > buffersize)
+			audiosize = buffersize;
 
+		s16* outbuf;
 		int samplesOutput;
 		if(synchmode == ESynchMode_Synchronous)
-			samplesOutput = synchronizer->output_samples(SPU_user->outbuf, audiosize);
-		else
+		{
+			static std::vector<s16> tempbuf;
+			if(tempbuf.size() < audiosize*2) tempbuf.resize(audiosize*2);
+			outbuf = &tempbuf[0];
+			samplesOutput = synchronizer->output_samples(outbuf, audiosize);
+		}
+		else if(SPU_user != NULL)
+		{
+			outbuf = SPU_user->outbuf;
 			samplesOutput = (SPU_MixAudio(mix,SPU_user,audiosize), audiosize);
+		}
+		else return;
 
-		SNDCore->UpdateAudio(SPU_user->outbuf, samplesOutput);
-		WAV_WavSoundUpdate(SPU_user->outbuf, samplesOutput, WAVMODE_USER);
+		SNDCore->UpdateAudio(outbuf, samplesOutput);
+		WAV_WavSoundUpdate(outbuf, samplesOutput, WAVMODE_USER);
 	}
 }
 
@@ -1721,12 +1766,8 @@ bool spu_loadstate(EMUFILE* is, int size)
 		spu->regs.masteren = BIT15(T1ReadWord(MMU.ARM7_REG, 0x500));
 	}
 
-
 	//copy the core spu (the more accurate) to the user spu
-	if(SPU_user) {
-		memcpy(SPU_user->channels,SPU_core->channels,sizeof(SPU_core->channels));
-		SPU_user->regs = SPU_core->regs;
-	}
+	SPU_CloneUser();
 
 	return true;
 }
