@@ -1,22 +1,19 @@
-/*  Copyright (C) 2009-2010 DeSmuME team
+/*
+	Copyright (C) 2009-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
-
 #include "addons.h"
 #include <string>
 
@@ -36,6 +33,7 @@ extern ADDONINTERFACE addonGBAgame;
 extern ADDONINTERFACE addonGuitarGrip;
 extern ADDONINTERFACE addonExpMemory;
 extern ADDONINTERFACE addonPiano;
+extern ADDONINTERFACE addonPaddle;
 //extern ADDONINTERFACE addonExternalMic;
 
 ADDONINTERFACE addonList[NDS_ADDON_COUNT] = {
@@ -46,6 +44,7 @@ ADDONINTERFACE addonList[NDS_ADDON_COUNT] = {
 		addonGuitarGrip,
 		addonExpMemory,
 		addonPiano,
+		addonPaddle
 };
 
 ADDONINTERFACE	addon = addonCFlash;		// default none pak
@@ -68,10 +67,10 @@ void addonsReset()
 
 BOOL addonsChangePak(NDS_ADDON_TYPE type)
 {
-	printf("addonsChangePak\n");
 	if (type > NDS_ADDON_COUNT) return FALSE;
 	addon.close();
 	addon = addonList[type];
 	addon_type = type;
+	printf("Slot 2: %s\n", addon.name);
 	return addon.init();
 }
