@@ -1,9 +1,10 @@
-/*  Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008-2012 DeSmuME team
 	
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation, either version 3 of the License, or
+	the Free Software Foundation, either version 2 of the License, or
 	(at your option) any later version.
 
 	This file is distributed in the hope that it will be useful,
@@ -192,6 +193,8 @@ TEMPLATE static u32 bios_nop()
 
 TEMPLATE static u32 WaitByLoop()
 {
+	u32 elapsed;
+	
 	//printf("%lld waitbyloop\n",nds_timer);
 	//INFO("ARM%c: SWI 0x03 (WaitByLoop)\n", PROCNUM?'7':'9');
 	if (PROCNUM == ARMCPU_ARM9)
@@ -199,12 +202,14 @@ TEMPLATE static u32 WaitByLoop()
 		armcp15_t *cp = (armcp15_t*)(cpu->coproc[15]);
 
 		if (cp->ctrl & ((1<<16)|(1<<18)))		// DTCM or ITCM is on (cache)
-			return cpu->R[0] * 2;
+			elapsed = cpu->R[0] * 2;
 		else
-			return cpu->R[0] * 8;
+			elapsed = cpu->R[0] * 8;
 	}
-    
-	return cpu->R[0] * 4;
+	else
+		elapsed = cpu->R[0] * 4;
+	cpu->R[0] = 0;
+	return elapsed;
 }
 
 TEMPLATE static u32 wait4IRQ()
@@ -1049,12 +1054,13 @@ TEMPLATE static u32 getCRC16()
 	u16 crc = (u16)cpu->R[0];
 	u32 datap = cpu->R[1];
 	u32 size = cpu->R[2]>>1;
+	u16 currVal = 0;
 
 	const u16 val[] = { 0x0000,0xCC01,0xD801,0x1400,0xF001,0x3C00,0x2800,0xE401,0xA001,0x6C00,0x7800,0xB401,0x5000,0x9C01,0x8801,0x4400};
 
 	for(u32 i = 0; i < size; i++)
 	{
-		u16 currVal = _MMU_read16<PROCNUM>(datap + i*2);
+		currVal = _MMU_read16<PROCNUM>(datap + i*2);
 
 		for(int j=0;j<4;j++)
 		{
@@ -1069,17 +1075,18 @@ TEMPLATE static u32 getCRC16()
 	}
 
 	cpu->R[0] = crc;
+	// R3 contains the last processed halfword 
+	cpu->R[3] = currVal;
 	return 1;
 }
 
 TEMPLATE static u32 isDebugger()
 {
 	//gbatek has additional specifications which are not emulated here
-#ifdef DEVELOPER
-	cpu->R[0] = 1;
-#else
-	cpu->R[0] = 0;
-#endif
+	if(nds.Is_DebugConsole())
+		cpu->R[0] = 1;
+	else
+		cpu->R[0] = 0;
 	return 1;
 }
 
