@@ -71,8 +71,7 @@ static float nds_screen_size_ratio = 1.0f;
 #define NUM_FRAMES_TO_TIME 60
 #endif
 
-
-#define FPS_LIMITER_FRAME_PERIOD 8
+#define FPS_LIMITER_FPS 60
 
 static SDL_Surface * surface;
 
@@ -129,7 +128,6 @@ class configured_features : public CommandLine
 public:
   int auto_pause;
   int frameskip;
-  int fps_limiter_frame_period;
 
   int engine_3d;
   int savetype;
@@ -147,7 +145,6 @@ init_config( class configured_features *config) {
 
   config->auto_pause = 0;
   config->frameskip = 0;
-  config->fps_limiter_frame_period = FPS_LIMITER_FRAME_PERIOD;
 
   config->engine_3d = 1;
   config->savetype = 0;
@@ -168,7 +165,6 @@ fill_config( class configured_features *config,
   GOptionEntry options[] = {
     { "auto-pause", 0, 0, G_OPTION_ARG_NONE, &config->auto_pause, "Pause emulation if focus is lost", NULL},
     { "frameskip", 0, 0, G_OPTION_ARG_INT, &config->frameskip, "Set frameskip", "FRAMESKIP"},
-    { "limiter-period", 0, 0, G_OPTION_ARG_INT, &config->fps_limiter_frame_period, "Set frame period of the fps limiter", "LIMITER"},
     { "3d-engine", 0, 0, G_OPTION_ARG_INT, &config->engine_3d, "Select 3d rendering engine. Available engines:\n"
         "\t\t\t\t\t\t  0 = 3d disabled\n"
         "\t\t\t\t\t\t  1 = internal rasterizer (default)\n"
@@ -225,11 +221,6 @@ fill_config( class configured_features *config,
     goto error;
   }
 
-  if (config->fps_limiter_frame_period < 0 || config->fps_limiter_frame_period > 30) {
-    g_printerr("FPS limiter period must be >= 0 and <= 30.\n");
-    goto error;
-  }
-
   if (config->nds_file == "") {
     g_printerr("Need to specify file to load.\n");
     goto error;
@@ -275,30 +266,6 @@ joinThread_gdb( void *thread_handle) {
 }
 #endif
 
-
-
-/** 
- * A SDL timer callback function. Signals the supplied SDL semaphore
- * if its value is small.
- * 
- * @param interval The interval since the last call (in ms)
- * @param param The pointer to the semaphore.
- * 
- * @return The interval to the next call (required by SDL)
- */
-static Uint32
-fps_limiter_fn( Uint32 interval, void *param) {
-  SDL_sem *sdl_semaphore = (SDL_sem *)param;
-
-  /* signal the semaphore if it is getting low */
-  if ( SDL_SemValue( sdl_semaphore) < 4) {
-    SDL_SemPost( sdl_semaphore);
-  }
-
-  return interval;
-}
-
-
 #ifdef INCLUDE_OPENGL_2D
 /* initialization openGL function */
 static int
@@ -351,11 +318,19 @@ initGL( GLuint *screen_texture) {
 }
 
 static void
-resizeWindow( u16 width, u16 height) {
+resizeWindow( u16 width, u16 height, GLuint *screen_texture) {
+
   int comp_width = 3 * width;
   int comp_height = 2 * height;
   GLenum errCode;
 
+  surface = SDL_SetVideoMode(width, height, 32, sdl_videoFlags);
+  initGL(screen_texture);
+
+#ifdef HAVE_LIBAGG
+  Hud.reset();
+#endif
+
   if ( comp_width > comp_height) {
     width = 2*height/3;
   }
@@ -386,9 +361,6 @@ resizeWindow( u16 width, u16 height) {
     errString = gluErrorString(errCode);
     fprintf( stderr, "GL resize failed: %s\n", errString);
   }
-
-  surface = SDL_SetVideoMode( width, height, 32,
-                              sdl_videoFlags );
 }
 
 
@@ -455,9 +427,15 @@ opengl_Draw( GLuint *texture, int software_convert) {
 #endif
 
 /* this is a stub for resizeWindow_stub in the case of no gl headers or no opengl 2d */
+#ifdef INCLUDE_OPENGL_2D
 static void
-resizeWindow_stub (u16 width, u16 height) {
+resizeWindow_stub (u16 width, u16 height, GLuint *screen_texture) {
 }
+#else
+static void
+resizeWindow_stub (u16 width, u16 height, void *screen_texture) {
+}
+#endif
 
 static void
 Draw( void) {
@@ -518,17 +496,17 @@ int main(int argc, char ** argv) {
 #endif
 
   int limiter_frame_counter = 0;
-  SDL_sem *fps_limiter_semaphore = NULL;
-  SDL_TimerID limiter_timer = NULL;
+  int limiter_tick0 = 0;
   int error;
 
   GKeyFile *keyfile;
 
+  int now;
+
 #ifdef DISPLAY_FPS
   u32 fps_timing = 0;
   u32 fps_frame_counter = 0;
   u32 fps_previous_time = 0;
-  u32 fps_temp_time;
 #endif
 
 #ifdef INCLUDE_OPENGL_2D
@@ -714,7 +692,7 @@ int main(int argc, char ** argv) {
 
   /* set the initial window size */
   if ( my_config.opengl_2d) {
-    resizeWindow( 256, 192*2);
+    resizeWindow( 256, 192*2, screen_texture);
   }
 #endif
 
@@ -726,27 +704,6 @@ int main(int argc, char ** argv) {
   /* Since gtk has a different mapping the keys stop to work with the saved configuration :| */
   load_default_config(cli_kb_cfg);
 
-  if ( !my_config.disable_limiter) {
-    /* create the semaphore used for fps limiting */
-    fps_limiter_semaphore = SDL_CreateSemaphore( 1);
-
-    /* start a SDL timer for every FPS_LIMITER_FRAME_PERIOD frames to keep us at 60 fps */
-    if ( fps_limiter_semaphore != NULL) {
-      limiter_timer = SDL_AddTimer( 16 * my_config.fps_limiter_frame_period,
-                                  fps_limiter_fn, fps_limiter_semaphore);
-    }
-
-    if ( limiter_timer == NULL) {
-      fprintf( stderr, "Error trying to start FPS limiter timer: %s\n",
-               SDL_GetError());
-      if ( fps_limiter_semaphore != NULL) {
-        SDL_DestroySemaphore( fps_limiter_semaphore);
-        fps_limiter_semaphore = NULL;
-      }
-      return 1;
-    }
-  }
-
   if(my_config.load_slot != -1){
     loadstate_slot(my_config.load_slot);
   }
@@ -763,6 +720,11 @@ int main(int argc, char ** argv) {
   ctrls_cfg.focused = 1;
   ctrls_cfg.fake_mic = 0;
   ctrls_cfg.keypad = 0;
+#ifdef INCLUDE_OPENGL_2D
+  ctrls_cfg.screen_texture = screen_texture;
+#else
+  ctrls_cfg.screen_texture = NULL;
+#endif
   ctrls_cfg.resize_cb = &resizeWindow_stub;
 
   while(!ctrls_cfg.sdl_quit) {
@@ -785,21 +747,28 @@ int main(int argc, char ** argv) {
         desmume_cycle(&ctrls_cfg);
     }
 
+#ifdef DISPLAY_FPS
+    now = SDL_GetTicks();
+#endif
     if ( !my_config.disable_limiter && !ctrls_cfg.boost) {
-      limiter_frame_counter += 1 + my_config.frameskip;
-      if ( limiter_frame_counter >= my_config.fps_limiter_frame_period) {
+#ifndef DISPLAY_FPS
+      now = SDL_GetTicks();
+#endif
+      int delay =  (limiter_tick0 + limiter_frame_counter*1000/FPS_LIMITER_FPS) - now;
+      if (delay < -500 || delay > 100) { // reset if we fall too far behind don't want to run super fast until we catch up
+        limiter_tick0 = now;
         limiter_frame_counter = 0;
-
-        /* wait for the timer to expire */
-        SDL_SemWait( fps_limiter_semaphore);
+      } else if (delay > 0) {
+        SDL_Delay(delay);
       }
     }
+    // always count frames, we'll mess up if the limiter gets turned on later otherwise
+    limiter_frame_counter += 1 + my_config.frameskip;
 
 #ifdef DISPLAY_FPS
     fps_frame_counter += 1;
-    fps_temp_time = SDL_GetTicks();
-    fps_timing += fps_temp_time - fps_previous_time;
-    fps_previous_time = fps_temp_time;
+    fps_timing += now - fps_previous_time;
+    fps_previous_time = now;
 
     if ( fps_frame_counter == NUM_FRAMES_TO_TIME) {
       char win_title[20];
@@ -818,12 +787,6 @@ int main(int argc, char ** argv) {
   /* Unload joystick */
   uninit_joy();
 
-  if ( !my_config.disable_limiter) {
-    /* tidy up the FPS limiter timer and semaphore */
-    SDL_RemoveTimer( limiter_timer);
-    SDL_DestroySemaphore( fps_limiter_semaphore);
-  }
-
   SDL_Quit();
   NDS_DeInit();
 
