@@ -1,28 +1,24 @@
-/*  commandline.cpp
+/*
+	Copyright (C) 2009-2012 DeSmuME team
 
-    Copyright (C) 2009-2010 DeSmuME team
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    This file is part of DeSmuME
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 //windows note: make sure this file gets compiled with _cdecl
 
 #include <glib.h>
-
+#include <algorithm>
 #include <stdio.h>
 #include "commandline.h"
 #include "types.h"
@@ -52,11 +48,14 @@ CommandLine::CommandLine()
 , _rigorous_timing(0)
 , _advanced_timing(-1)
 , _slot1(NULL)
+, _slot1_fat_dir(NULL)
+, _console_type(NULL)
 , depth_threshold(-1)
 , load_slot(-1)
 , arm9_gdb_port(0)
 , arm7_gdb_port(0)
 , start_paused(FALSE)
+, autodetect_method(-1)
 {
 #ifndef _MSC_VER
 	disable_sound = 0;
@@ -94,7 +93,9 @@ void CommandLine::loadCommonOptions()
 		{ "rigorous-timing", 0, 0, G_OPTION_ARG_INT, &_rigorous_timing, "Use some rigorous timings instead of unrealistically generous (default 0)", "RIGOROUS_TIMING"},
 		{ "advanced-timing", 0, 0, G_OPTION_ARG_INT, &_advanced_timing, "Use advanced BUS-level timing (default 1)", "ADVANCED_TIMING"},
 		{ "slot1", 0, 0, G_OPTION_ARG_STRING, &_slot1, "Device to load in slot 1 (default retail)", "SLOT1"},
+		{ "slot1-fat-dir", 0, 0, G_OPTION_ARG_STRING, &_slot1_fat_dir, "Directory to scan for slot 1", "SLOT1_DIR"},
 		{ "depth-threshold", 0, 0, G_OPTION_ARG_INT, &depth_threshold, "Depth comparison threshold (default 0)", "DEPTHTHRESHOLD"},
+		{ "console-type", 0, 0, G_OPTION_ARG_STRING, &_console_type, "Select console type: {fat,lite,ique,debug,dsi}", "CONSOLETYPE" },
 #ifndef _MSC_VER
 		{ "disable-sound", 0, 0, G_OPTION_ARG_NONE, &disable_sound, "Disables the sound emulation", NULL},
 		{ "disable-limiter", 0, 0, G_OPTION_ARG_NONE, &disable_limiter, "Disables the 60fps limiter", NULL},
@@ -104,12 +105,15 @@ void CommandLine::loadCommonOptions()
 		{ "arm9gdb", 0, 0, G_OPTION_ARG_INT, &arm9_gdb_port, "Enable the ARM9 GDB stub on the given port", "PORT_NUM"},
 		{ "arm7gdb", 0, 0, G_OPTION_ARG_INT, &arm7_gdb_port, "Enable the ARM7 GDB stub on the given port", "PORT_NUM"},
 #endif
+		{ "autodetect_method", 0, 0, G_OPTION_ARG_INT, &autodetect_method, "Autodetect backup method (0 - internal, 1 - from database)", "AUTODETECT_METHOD"},
 		{ NULL }
 	};
 
 	g_option_context_add_main_entries (ctx, options, "options");
 }
 
+static char mytoupper(char c) { return ::toupper(c); }
+
 bool CommandLine::parse(int argc,char **argv)
 {
 	g_option_context_parse (ctx, &argc, &argv, &error);
@@ -119,7 +123,9 @@ bool CommandLine::parse(int argc,char **argv)
 		return false;
 	}
 
+	if(_slot1_fat_dir) slot1_fat_dir = _slot1_fat_dir;
 	if(_slot1) slot1 = _slot1;
+	if(_console_type) console_type = _console_type;
 	if(slot1.size() != 0) str_lcase((char*)&slot1[0]);
 	if(_play_movie_file) play_movie_file = _play_movie_file;
 	if(_record_movie_file) record_movie_file = _record_movie_file;
@@ -133,7 +139,26 @@ bool CommandLine::parse(int argc,char **argv)
 	if(depth_threshold != -1)
 		CommonSettings.GFX3D_Zelda_Shadow_Depth_Hack = depth_threshold;
 
-	//TODO MAX PRIORITY! change ARM9BIOS etc to be a std::string
+
+	//process console type
+	CommonSettings.DebugConsole = false;
+	CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_FAT;
+	std::transform(console_type.begin(), console_type.end(), console_type.begin(), ::mytoupper);
+	if(console_type == "") {}
+	else if(console_type == "FAT") CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_FAT;
+	else if(console_type == "LITE") CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_LITE;
+	else if(console_type == "IQUE") CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_IQUE;
+	else if(console_type == "DSI") CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_DSI;
+	else if(console_type == "DEBUG")
+	{
+		CommonSettings.ConsoleType = NDS_CONSOLE_TYPE_FAT;
+		CommonSettings.DebugConsole = true;
+	}
+
+	if (autodetect_method != -1)
+		CommonSettings.autodetectBackupMethod = autodetect_method;
+
+	//TODO NOT MAX PRIORITY! change ARM9BIOS etc to be a std::string
 	if(_bios_arm9) { CommonSettings.UseExtBIOS = true; strcpy(CommonSettings.ARM9BIOS,_bios_arm9); }
 	if(_bios_arm7) { CommonSettings.UseExtBIOS = true; strcpy(CommonSettings.ARM7BIOS,_bios_arm7); }
 	if(_bios_swi) CommonSettings.SWIFromBIOS = true;
@@ -149,6 +174,8 @@ bool CommandLine::parse(int argc,char **argv)
 
 bool CommandLine::validate()
 {
+
+
 	if(slot1 != "")
 	{
 		if(slot1 != "r4" && slot1 != "retail" && slot1 != "none") {
@@ -190,6 +217,11 @@ bool CommandLine::validate()
 		g_printerr("Cannot specify both cflash and gbaslot rom (both occupy SLOT-2)\n");
 	}
 
+	if (autodetect_method < -1 || autodetect_method > 1) {
+		g_printerr("Invalid autodetect save method (0 - internal, 1 - from database)\n");
+	}
+
+
 	return true;
 }
 
@@ -227,10 +259,14 @@ void CommandLine::process_addonCommands()
 		is_cflash_configured = true;
     }
 
+	if(slot1_fat_dir != "")
+		slot1SetFatDir(slot1_fat_dir);
+
 	if(slot1 == "retail")
 		slot1Change(NDS_SLOT1_RETAIL);
 	else if(slot1 == "r4")
 		slot1Change(NDS_SLOT1_R4);
-
+		else if(slot1 == "retailNAND")
+			slot1Change(NDS_SLOT1_RETAIL_NAND);
 }
 
