@@ -1,22 +1,18 @@
-/*  commandline.h
+/*
+	Copyright (C) 2009-2011 DeSmuME team
 
-    Copyright (C) 2009-2010 DeSmuME team
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    This file is part of DeSmuME
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
-
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _COMMANDLINE_H_
@@ -41,6 +37,7 @@ public:
 	//actual options: these may move to another sturct
 	int load_slot;
 	int depth_threshold;
+	int autodetect_method;
 	std::string nds_file;
 	std::string play_movie_file;
 	std::string record_movie_file;
@@ -50,6 +47,8 @@ public:
 	std::string cflash_path;
 	std::string gbaslot_rom;
 	std::string slot1;
+	std::string console_type;
+	std::string slot1_fat_dir;
 #ifndef _MSC_VER
 	int disable_sound;
 	int disable_limiter;
@@ -91,6 +90,8 @@ private:
 	int _rigorous_timing;
 	int _advanced_timing;
 	char* _slot1;
+	char *_slot1_fat_dir;
+	char* _console_type;
 };
 
 #endif
