@@ -1,20 +1,18 @@
-/*	Copyright (C) 2009 DeSmuME Team
+/*	
+	Copyright (C) 2009-2011 DeSmuME Team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "firmware.h"
@@ -566,11 +564,11 @@ bool CFIRMWARE::load()
 		INFO("   * ARM7 unpacked size:         0x%08X (%i) bytes\n", size7, size7);
 	}
 
-	PathInfo path;
-	path.init(CommonSettings.Firmware);
-	path.getpathnoext(path.FIRMWARE, &MMU.fw.userfile[0]);
-	strcat(MMU.fw.userfile, ".dfc");		// DeSmuME Firmware Config
+	// Generate the path for the external firmware config file.
+	std::string extFilePath = CFIRMWARE::GetExternalFilePath();
+	strncpy(MMU.fw.userfile, extFilePath.c_str(), MAX_PATH);
 
+	fclose(fp);
 	fp = fopen(MMU.fw.userfile, "rb");
 	if (fp)
 	{
@@ -609,6 +607,15 @@ bool CFIRMWARE::load()
 	return true;
 }
 
+std::string CFIRMWARE::GetExternalFilePath()
+{
+	std::string fwPath = CommonSettings.Firmware;
+	std::string fwFileName = Path::GetFileNameFromPathWithoutExt(fwPath);
+	std::string configPath = path.pathToBattery;
+	std::string finalPath = configPath + DIRECTORY_DELIMITER_CHAR + fwFileName + FILE_EXT_DELIMITER_CHAR + FW_CONFIG_FILE_EXT;
+
+	return finalPath;
+}
 
 //=====================================================================================================
 static u32
@@ -789,7 +796,7 @@ int NDS_CreateDummyFirmware( struct NDS_fw_config_data *user_settings)
 	MMU.fw.data[0x8 + 3] = 'P';
 
 	// DS type
-	if ( user_settings->ds_type == NDS_FW_DS_TYPE_LITE)
+	if ( user_settings->ds_type == NDS_CONSOLE_TYPE_LITE)
 		MMU.fw.data[0x1d] = 0x20;
 	else
 		MMU.fw.data[0x1d] = 0xff;
@@ -872,7 +879,7 @@ void NDS_FillDefaultFirmwareConfigData( struct NDS_fw_config_data *fw_config) {
 	int str_length;
 
 	memset( fw_config, 0, sizeof( struct NDS_fw_config_data));
-	fw_config->ds_type = NDS_FW_DS_TYPE_FAT;
+	fw_config->ds_type = NDS_CONSOLE_TYPE_FAT;
 
 	fw_config->fav_colour = 7;
 
@@ -891,19 +898,32 @@ void NDS_FillDefaultFirmwareConfigData( struct NDS_fw_config_data *fw_config) {
 	}
 	fw_config->message_len = str_length;
 
-	/* default to English */
+	//default to English
 	fw_config->language = 1;
 
-	/* default touchscreen calibration */
+	// default touchscreen calibration
+
+	//ANCIENT DESMUME VALUES
 	fw_config->touch_cal[0].adc_x = 0x200;
 	fw_config->touch_cal[0].adc_y = 0x200;
 	fw_config->touch_cal[0].screen_x = 0x20 + 1; // calibration screen coords are 1-based,
 	fw_config->touch_cal[0].screen_y = 0x20 + 1; // either that or NDS_getADCTouchPosX/Y are wrong.
+	//VALUES FROM NOCASH
+	//fw_config->touch_cal[0].adc_x = 0x02DF;
+	//fw_config->touch_cal[0].adc_y = 0x032C;
+	//fw_config->touch_cal[0].screen_x = 0x20;
+	//fw_config->touch_cal[0].screen_y = 0x20;
 
+	//ANCIENT DESMUME VALUES
 	fw_config->touch_cal[1].adc_x = 0xe00;
 	fw_config->touch_cal[1].adc_y = 0x800;
 	fw_config->touch_cal[1].screen_x = 0xe0 + 1;
 	fw_config->touch_cal[1].screen_y = 0x80 + 1;
+	//VALUES FROM NOCASH
+	//fw_config->touch_cal[1].adc_x = 0x0D3B;
+	//fw_config->touch_cal[1].adc_y = 0x0CE7;
+	//fw_config->touch_cal[1].screen_x = 0xE0;
+	//fw_config->touch_cal[1].screen_y = 0xA0;
 }
 
 void NDS_PatchFirmwareMAC()
