@@ -1,5 +1,6 @@
-/*	Copyright (C) 2006 yopyop
-    Copyright (C) 2008-2010 DeSmuME team
+/*	
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -47,6 +48,13 @@
 #include "movie.h" //only for currframecounter which really ought to be moved into the core emu....
 #include <queue>
 
+//#define _SHOW_VTX_COUNTERS	// show polygon/vertex counters on screen
+#ifdef _SHOW_VTX_COUNTERS
+u32 max_polys, max_verts;
+#include "GPU_OSD.h"
+#endif
+
+
 /*
 thoughts on flush timing:
 I think a flush is supposed to queue up and wait to happen during vblank sometime.
@@ -64,31 +72,29 @@ But since we're not sure how we'll eventually want this, I am leaving it sort of
 in this function: */
 static void gfx3d_doFlush();
 
-#define INVALID_COMMAND 0xFF
-#define UNDEFINED_COMMAND 0xCC
+#define GFX_NOARG_COMMAND 0x00
+#define GFX_INVALID_COMMAND 0xFF
+#define GFX_UNDEFINED_COMMAND 0xCC
 static const u8 gfx3d_commandTypes[] = {
-	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, //invalid commands; no parameters
-	0x01,0x00,0x01,0x01,0x01,0x00,0x10,0x0C, 0x10,0x0C,0x09,0x03,0x03,0xCC,0xCC,0xCC, //matrix commands
-	0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x01, 0x01,0x01,0x01,0x01,0xCC,0xCC,0xCC,0xCC, //vertex and per-vertex material commands
-	0x01,0x01,0x01,0x01,0x20,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //lighting engine material commands
-	0x01,0x00,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //begin and end
-	0x01,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //swapbuffers
-	0x01,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //viewport
-	0x03,0x02,0x01,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //tests
+	/* 00 */ 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF, //invalid commands; no parameters
+	/* 10 */ 0x01,0x00,0x01,0x01,0x01,0x00,0x10,0x0C, 0x10,0x0C,0x09,0x03,0x03,0xCC,0xCC,0xCC, //matrix commands
+	/* 20 */ 0x01,0x01,0x01,0x02,0x01,0x01,0x01,0x01, 0x01,0x01,0x01,0x01,0xCC,0xCC,0xCC,0xCC, //vertex and per-vertex material commands
+	/* 30 */ 0x01,0x01,0x01,0x01,0x20,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //lighting engine material commands
+	/* 40 */ 0x01,0x00,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //begin and end
+	/* 50 */ 0x01,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //swapbuffers
+	/* 60 */ 0x01,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //viewport
+	/* 70 */ 0x03,0x02,0x01,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, //tests
 	//0x80:
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
-	0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC
+	/* 80 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* 90 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* A0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* B0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* C0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* D0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* E0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,
+	/* F0 */ 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC, 0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC,0xCC
 };
 
-
-//todo: it would be nice to make some kind of state machine that more closely models the hardware
-//but this is servicable for now
 class GXF_Hardware
 {
 public:
@@ -103,105 +109,72 @@ public:
 			commandsPending[i].command = 0;
 			commandsPending[i].countdown = 0;
 		}
+		size = 0;
 	}
-	//todo - things in here other than the very first thing involving GFX3D_NOP_NOARG_HACK I am not too sure about.
-	void receive(u32 val) {
-		bool hack = false;
-		if(size()>0 && val != 0 && 
-			(front().command == 0x15 || front().command == 0x11 || front().command == 0x41
-			|| front().command == GFX3D_NOP_NOARG_HACK  //nintendogs dalmatian sends these I think.
-			))
+
+	void receive(u32 val) 
+	{
+		if (size > 0)
 		{
-			//apparently a zero is swallowed in this case but if another value is sent 
-			//processing will continue
-			if(front().command == GFX3D_NOP_NOARG_HACK) { }
-			else
+			GFX_FIFOsend(front().command, val);
+			front().countdown--;
+			if (front().countdown == 0) 
 			{
-				//printf("gxf: sending hack %02X: (dummy=0)\n", front().command);
-				GFX_FIFOsend(front().command,0);
-			}
-			hack = true;
-			goto hackTrigger;
-		}
-		if(countdown>0) {
-			//received a parameter
-			//printf("gxf: sending %02X: %08X\n", front().command,val);
-			//if(commandsPending.front() == GFX3D_NOP_NOARG_HACK)
-			//{}
-			//else 
-				GFX_FIFOsend(front().command,val);
-			hackTrigger:
-			countdown--;
-			while(countdown==0) {
+				size--; 
+				if (size == 0) return;
 				dequeue();
-			trigger:
-				//dont set hack to false if you jumped from below! it needs to be true for when you jump down from above.
-				//oh my what a mess.
-				if(size()==0) break;
-				countdown = front().countdown;
-				if(!countdown) {
-					if(front().command != INVALID_COMMAND 
-						&& front().command != GFX3D_NOP_NOARG_HACK //g.i. joe sends these in the form of 0xFF commands 
-						){
-						//printf("[%06d]gxf: sending %02X: (dummy=0)\n", currFrameCounter,front().command);
-						GFX_FIFOsend(front().command,0);
-					}
+
+				while (gfx3d_commandTypes[front().command] == GFX_NOARG_COMMAND)
+				{
+					GFX_FIFOsend(front().command, 0);
+					size--; 
+					if (size == 0) break;
+					dequeue();
 				}
 			}
-			if(hack) goto decode;
-		} else {
-			//decode a packed command
-decode:
-			//printf("[%05d] gxf: decoding %08X\n",currFrameCounter,val);
+		}
+		else
+		{
 			if (val == 0) return;	// nop
 
 			const u8 commands[] = { val&0xFF, (val>>8)&0xFF, (val>>16)&0xFF, (val>>24)&0xFF };
-			const u8 commandTypes[] = { gfx3d_commandTypes[commands[0]], gfx3d_commandTypes[commands[1]],gfx3d_commandTypes[commands[2]], gfx3d_commandTypes[commands[3]] };
-
-			for(int i=0;i<4;i++) {
-				u8 cmd = commands[i];
-				u8 type = commandTypes[i];
-
-				if(type == INVALID_COMMAND) {
-					commandsPending[i].command = INVALID_COMMAND;
-				} else {
-					if(type == UNDEFINED_COMMAND) 
-					{
-						commandsPending[i].command = GFX3D_NOP_NOARG_HACK;  //enqueue a single undefined command we know how to handle
-					}
-					else commandsPending[i].command = cmd;
-				}
-				if(type == UNDEFINED_COMMAND
-					//|| type == 0x00 //DON'T DO THIS: galactik football will break if you do (ingame character portraits etc.) as well as nintendogs dalmatian
-					) {
-					//these are valid commands with no parameters. they might need special handling
-					//as long as there is a subsequent defined command with parameters, we're safe
-					bool safe = false;
-					if (i == 3)
-						safe = true;
-					else
-						// start loop from (i+1) always skip last command
-					for(u8 j=i+1;j<4;j++) {
-						if((commandTypes[j] != INVALID_COMMAND)) {
-							safe = true;
-							break;
-						}
-					}
-					if(safe) {
-						commandsPending[i].countdown = 0;
-					} else {
-						//we need to receive a dummy parameter in this case
-						commandsPending[i].countdown = 1;
-					}
-				} else if(type != INVALID_COMMAND) {
-					commandsPending[i].countdown = type;
-				} else commandsPending[i].countdown = 0;
-			}
+			const u8 commandTypes[] = { gfx3d_commandTypes[commands[0]], gfx3d_commandTypes[commands[1]], gfx3d_commandTypes[commands[2]], gfx3d_commandTypes[commands[3]] };
 
 			commandCursor = 0;
-			countdown = front().countdown;
-			if(countdown==0) 
-				goto trigger;
+			size = 0;
+
+			// A command without parameters is one of the four following commands:
+			// - PushMatrix
+			// - LoadIdentity
+			// - End
+			// - Commands undefined within the region between 0x10 and 0xFF
+
+			for (u8 i = 0; i < 4; i++)
+			{
+				if (commandTypes[i] == GFX_INVALID_COMMAND)
+				{
+					//printf("gfx3D: invalid command (%02X)\n", commands[i]);
+					continue;
+				}
+
+				if (commandTypes[i] == GFX_UNDEFINED_COMMAND)
+				{
+					//printf("gfx3D: undefined command (%02X)\n", commands[i]);
+					continue;
+				}
+				commandsPending[size].command = commands[i];
+				commandsPending[size].countdown = commandTypes[i];
+				//printf("%i: CMD %02X size %i\n", i, commands[i], commandsPending[size].countdown);
+				if ((commandsPending[size].countdown == 0) && (size == 0)) // cmd 0x11, 0x15, 0x41 - no params
+				{
+					GFX_FIFOsend(commands[i], 0);
+
+					while ((i < 4) && commands[i+1] != 0 && gfx3d_commandTypes[commands[i+1]] == GFX_NOARG_COMMAND)
+						GFX_FIFOsend(commands[++i], 0);
+				}
+				else
+					size++;
+			}
 		}
 	}
 
@@ -215,17 +188,17 @@ decode:
 private:
 	void dequeue() { commandCursor++; }
 	CommandItem& front() { return commandsPending[commandCursor]; }
-	u32 size() { return 4-commandCursor; }
+	u32 size;
 public:
 
 	void savestate(EMUFILE *f)
 	{
 		//TODO - next time we invalidate savestates, simplify this format.
-		write32le(0,f); //version
-		write32le(size(),f);
-		for(u32 i=commandCursor;i<4;i++) write8le(commandsPending[i].command,f);
-		write32le(0,f);
-		for(u32 i=commandCursor;i<4;i++) write8le(commandsPending[i].countdown,f);
+		write32le(1,f); //version
+		write32le(size,f);
+		write32le(commandCursor,f);
+		for(u32 i=0;i<4;i++) write8le(commandsPending[i].command,f);
+		for(u32 i=0;i<4;i++) write8le(commandsPending[i].countdown,f);
 		write8le(countdown,f);
 	}
 	
@@ -233,16 +206,25 @@ public:
 	{
 		u32 version;
 		if(read32le(&version,f) != 1) return false;
-		if(version != 0) return false;
+		if(version > 1) return false;
 
-		u32 tempsize;
-		read32le(&tempsize,f);
-		commandCursor = 4-tempsize;
-		for(u32 i=0;i<commandCursor;i++) commandsPending[i].command = 0;
-		for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i].command,f);
-		read32le(&tempsize,f);
-		for(u32 i=0;i<commandCursor;i++) commandsPending[i].countdown = 0;
-		for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i].countdown,f);
+		if (version == 0)
+		{
+			read32le(&size,f);
+			commandCursor = 4-size;
+			for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i-commandCursor].command,f);
+			read32le(&size,f);
+			size = 4-commandCursor;
+			for(u32 i=commandCursor;i<4;i++) read8le(&commandsPending[i-commandCursor].countdown,f);
+		}
+		else
+		if (version == 1)
+		{
+			read32le(&size,f);
+			read32le(&commandCursor,f);
+			for(u32 i=0;i<4;i++) read8le(&commandsPending[i].command,f);
+			for(u32 i=0;i<4;i++) read8le(&commandsPending[i].countdown,f);
+		}
 
 		read8le(&countdown,f);
 
@@ -368,7 +350,7 @@ BOOL isSwapBuffers = FALSE;
 
 static u32 BTind = 0;
 static u32 PTind = 0;
-static u16 BTcoords[6] = {0, 0, 0, 0, 0, 0};
+static CACHE_ALIGN u16 BTcoords[6] = {0, 0, 0, 0, 0, 0};
 static CACHE_ALIGN float PTcoords[4] = {0.0, 0.0, 0.0, 1.0};
 
 //raw ds format poly attributes
@@ -503,6 +485,10 @@ void gfx3d_init()
 
 void gfx3d_reset()
 {
+#ifdef _SHOW_VTX_COUNTERS
+	max_polys = max_verts = 0;
+#endif
+
 	reconstruct(&gfx3d);
 	delete viewer3d_state;
 	viewer3d_state = new Viewer3d_State();
@@ -512,8 +498,9 @@ void gfx3d_reset()
 	control = 0;
 	drawPending = FALSE;
 	flushPending = FALSE;
-	memset(polylists, 0, sizeof(polylists));
-	memset(vertlists, 0, sizeof(vertlists));
+	memset(polylists, 0, sizeof(POLYLIST)*2);
+	memset(vertlists, 0, sizeof(VERTLIST)*2);
+	gfx3d.state.invalidateToon = true;
 	listTwiddle = 1;
 	twiddleLists();
 	gfx3d.polylist = polylist;
@@ -595,7 +582,6 @@ FORCEINLINE s32 vec3dot_fixed32(const s32* a, const s32* b) {
 	return fx32_shiftdown(fx32_mul(a[0],b[0]) + fx32_mul(a[1],b[1]) + fx32_mul(a[2],b[2]));
 }
 
-
 #define SUBMITVERTEX(ii, nn) polylist->list[polylist->count].vertIndexes[ii] = tempVertInfo.map[nn];
 //Submit a vertex to the GE
 static void SetVertex()
@@ -606,7 +592,7 @@ static void SetVertex()
 		s16coord[2]
 	};
 
-	ALIGN(16) s32 coordTransformed[4] = { coord[0], coord[1], coord[2], (1<<12) };
+	DS_ALIGN(16) s32 coordTransformed[4] = { coord[0], coord[1], coord[2], (1<<12) };
 
 	if (texCoordinateTransform == 3)
 	{
@@ -735,9 +721,9 @@ static void SetVertex()
 					break;
 				polygonListCompleted = 1;
 				SUBMITVERTEX(0,0);
-				SUBMITVERTEX(1,1);
-				SUBMITVERTEX(2,3);
-				SUBMITVERTEX(3,2);
+				SUBMITVERTEX(1,1);	// TODO:
+				SUBMITVERTEX(2,3);	// OpenGL Quad_Strip must be	: 3,3
+				SUBMITVERTEX(3,2);	//								: 2,2
 				polylist->list[polylist->count].type = 4;
 				tempVertInfo.map[0] = vertlist->count+2-continuation;
 				tempVertInfo.map[1] = vertlist->count+3-continuation;
@@ -754,6 +740,30 @@ static void SetVertex()
 		if(polygonListCompleted == 1)
 		{
 			POLY &poly = polylist->list[polylist->count];
+			
+			poly.vtxFormat = vtxFormat;
+
+			// Line segment detect
+			// Tested" Castlevania POR - warp stone, trajectory of ricochet, "Eye of Decay"
+			if (!(textureFormat & (7 << 26)))	// no texture
+			{
+				bool duplicated = false;
+				VERT &vert0 = vertlist->list[poly.vertIndexes[0]];
+				VERT &vert1 = vertlist->list[poly.vertIndexes[1]];
+				VERT &vert2 = vertlist->list[poly.vertIndexes[2]];
+				if ( (vert0.x == vert1.x) && (vert0.y == vert1.y) ) duplicated = true;
+				else
+					if ( (vert1.x == vert2.x) && (vert1.y == vert2.y) ) duplicated = true;
+					else
+						if ( (vert0.y == vert1.y) && (vert1.y == vert2.y) ) duplicated = true;
+						else
+							if ( (vert0.x == vert1.x) && (vert1.x == vert2.x) ) duplicated = true;
+				if (duplicated)
+				{
+					//printf("Line Segmet detected (poly type %i, mode %i, texparam %08X)\n", poly.type, poly.vtxFormat, textureFormat);
+					poly.vtxFormat = vtxFormat + 4;
+				}
+			}
 
 			poly.polyAttr = polyAttr;
 			poly.texParam = textureFormat;
@@ -842,8 +852,6 @@ static void gfx3d_glPushMatrix()
 
 	if(mymode==2)
 		MatrixStackPushMatrix (&mtxStack[1], mtxCurrent[1]);
-
-	MMU_new.gxstat.sb = 1; // set busy
 }
 
 static void gfx3d_glPopMatrix(s32 i)
@@ -855,9 +863,13 @@ static void gfx3d_glPopMatrix(s32 i)
 	//this command always works on both pos and vector when either pos or pos-vector are the current mtx mode
 	short mymode = (mode==1?2:mode);
 
-	//6 bits, sign extended
-	//this was necessary to fix sims apartment pets
 	//i = (i<<26)>>26;
+	//previously, we sign extended here. that isnt really necessary since the stacks are apparently modularly addressed. so i am somewhat doubtful that this is a real concept.
+	//example:
+	//suppose we had a -30 that would be %100010.
+	//which is the same as adding 34. if our stack was at 17 then one way is 17-30(+32)=19 and the other way is 17+34(-32)=19
+	
+	//please note that our ability to skip treating this as signed is dependent on the modular addressing later. if that ever changes, we need to change this back.
 
 	MatrixStackPopMatrix(mtxCurrent[mymode], &mtxStack[mymode], i);
 
@@ -865,8 +877,6 @@ static void gfx3d_glPopMatrix(s32 i)
 
 	if (mymode == 2)
 		MatrixStackPopMatrix(mtxCurrent[1], &mtxStack[1], i);
-
-	MMU_new.gxstat.sb = 1; // set busy
 }
 
 static void gfx3d_glStoreMatrix(u32 v)
@@ -879,7 +889,14 @@ static void gfx3d_glStoreMatrix(u32 v)
 	if(mymode==0 || mymode==3)
 		v = 0;
 
-	if (v >= 31) return;
+	v &= 31;
+
+	//according to gbatek, 31 works but sets the stack overflow flag
+	//spider-man 2 tests this on the spiderman model (and elsewhere)
+	//i am somewhat skeptical of this, but we'll leave it this way for now.
+	//a test shouldnt be too hard
+	if(v==31)
+		MMU_new.gxstat.se = 1;
 
 	MatrixStackLoadMatrix (&mtxStack[mymode], v, mtxCurrent[mymode]);
 
@@ -899,7 +916,15 @@ static void gfx3d_glRestoreMatrix(u32 v)
 	if(mymode==0 || mymode==3)
 		v = 0;
 
-	if (v >= 31) return;
+	v &= 31;
+
+	//according to gbatek, 31 works but sets the stack overflow flag
+	//spider-man 2 tests this on the spiderman model (and elsewhere)
+	//i am somewhat skeptical of this, but we'll leave it this way for now.
+	//a test shouldnt be too hard
+	if(v==31)
+		MMU_new.gxstat.se = 1;
+
 
 	MatrixCopy (mtxCurrent[mymode], MatrixStackGetPos(&mtxStack[mymode], v));
 
@@ -907,9 +932,6 @@ static void gfx3d_glRestoreMatrix(u32 v)
 
 	if (mymode == 2)
 		MatrixCopy (mtxCurrent[1], MatrixStackGetPos(&mtxStack[1], v));
-
-	//printf("restore: matrix %d to: \n",mymode); MatrixPrint(mtxCurrent[1]);
-
 }
 
 static void gfx3d_glLoadIdentity()
@@ -1119,15 +1141,16 @@ static void gfx3d_glNormal(s32 v)
 	s16 ny = ((v<<12)>>22)<<3;
 	s16 nz = ((v<<2)>>22)<<3;
 
+	CACHE_ALIGN s32 normal[4] =  { nx,ny,nz,(1<<12) };
+
 	if (texCoordinateTransform == 2)
 	{
 		//SM64 highlight rendered star in main menu tests this
-		last_s = (s32)(((s64)nx * mtxCurrent[3][0] + (s64)ny * mtxCurrent[3][4] + (s64)nz * mtxCurrent[3][8] + (_s<<24))>>24);
-		last_t = (s32)(((s64)nx * mtxCurrent[3][1] + (s64)ny * mtxCurrent[3][5] + (s64)nz * mtxCurrent[3][9] + (_t<<24))>>24);
+		//also smackdown 2010 player textures tested this (needed cast on _s and _t)
+		last_s = (s32)(((s64)normal[0] * mtxCurrent[3][0] + (s64)normal[1] * mtxCurrent[3][4] + (s64)normal[2] * mtxCurrent[3][8] + (((s64)_s)<<24))>>24);
+		last_t = (s32)(((s64)normal[0] * mtxCurrent[3][1] + (s64)normal[1] * mtxCurrent[3][5] + (s64)normal[2] * mtxCurrent[3][9] + (((s64)_t)<<24))>>24);
 	}
 
-	CACHE_ALIGN s32 normal[4] =  { nx,ny,nz,(1<<12) };
-
 	MatrixMultVec3x3_fixed(mtxCurrent[2],normal);
 
 	//apply lighting model
@@ -1393,8 +1416,8 @@ static void gfx3d_glBegin(u32 v)
 
 static void gfx3d_glEnd(void)
 {
-	inBegin = FALSE;
 	tempVertInfo.count = 0;
+	inBegin = FALSE;
 	GFX_DELAY(1);
 }
 
@@ -1662,7 +1685,7 @@ s32 gfx3d_GetClipMatrix (unsigned int index)
 {
 	s32 val = MatrixGetMultipliedIndex (index, mtxCurrent[0], mtxCurrent[1]);
 
-	//val *= (1<<12);
+	//printf("reading clip matrix: %d\n",index);
 
 	return (s32)val;
 }
@@ -1817,7 +1840,6 @@ static void gfx3d_execute(u8 cmd, u32 param)
 	log3D(cmd, param);
 #endif
 
-	MMU_new.gxstat.sb = 0;	// clear stack busy flag
 	switch (cmd)
 	{
 		case 0x10:		// MTX_MODE - Set Matrix Mode (W)
@@ -1991,6 +2013,8 @@ void gfx3d_glFlush(u32 v)
 #endif
 	
 	isSwapBuffers = TRUE;
+
+	//printf("%05d:%03d:%12lld: FLUSH\n",currFrameCounter, nds.VCount, nds_timer);
 	
 	//well, the game wanted us to flush.
 	//it may be badly timed. lets just flush it.
@@ -2055,6 +2079,12 @@ static void gfx3d_doFlush()
 	gfx3d.state.activeFlushCommand = gfx3d.state.pendingFlushCommand;
 
 	int polycount = polylist->count;
+#ifdef _SHOW_VTX_COUNTERS
+	max_polys = max((u32)polycount, max_polys);
+	max_verts = max((u32)vertlist->count, max_verts);
+	osd->addFixed(180, 20, "%i/%i", polycount, vertlist->count);		// current
+	osd->addFixed(180, 35, "%i/%i", max_polys, max_verts);		// max
+#endif
 
 	//find the min and max y values for each poly.
 	//TODO - this could be a small waste of time if we are manual sorting the translucent polys
@@ -2157,6 +2187,8 @@ void gfx3d_VBlankEndSignal(bool skipFrame)
 
 void gfx3d_sendCommandToFIFO(u32 val)
 {
+	//printf("gxFIFO: send val=0x%08X, size=%03i (fifo)\n", val, gxFIFO.size);
+
 	gxf_hardware.receive(val);
 }
 
@@ -2322,6 +2354,7 @@ SFORMAT SF_GFX3D[]={
 	{ "GFHE", 4, 1, &gxFIFO.head},
 	{ "GFTA", 4, 1, &gxFIFO.tail},
 	{ "GFSZ", 4, 1, &gxFIFO.size},
+	{ "GFMS", 4, 1, &gxFIFO.matrix_stack_op_size},
 	{ "GFCM", 1, HACK_GXIFO_SIZE, &gxFIFO.cmd[0]},
 	{ "GFPM", 4, HACK_GXIFO_SIZE, &gxFIFO.param[0]},
 	{ "GPHE", 1, 1, &gxPIPE.head},
@@ -2801,3 +2834,70 @@ void GFX3D_Clipper::clipPoly(POLY* poly, VERT** verts)
 }
 #endif
 
+// "Workaround" for line poly
+bool gfx3d_IsLinePoly(POLY *poly)
+{
+	if(!CommonSettings.GFX3D_LineHack) return false;
+
+	int type = poly->type;
+	VERT *vert1, *vert2;
+
+	if (type <= 2)
+		return true;
+	else if (type > 10)
+		return false;
+
+	// Method 1:
+	// Castlevania Portrait of Ruin - trajectory of ricochet
+	bool duplicatedVert[10];
+	for(int i = 0; i < type; i++)
+		duplicatedVert[i] = false;
+	for(int i = 0; i < type - 1; i++)
+	{
+		vert1 = &gfx3d.vertlist->list[poly->vertIndexes[i]];
+		for(int j = i + 1; j < type; j++)
+		{
+			vert2 = &gfx3d.vertlist->list[poly->vertIndexes[j]];
+			if (vert1->x == vert2->x && vert1->y == vert2->y
+				//&& vert1->z == vert2->z
+				)
+			{
+				duplicatedVert[j] = true;
+			}
+		}
+	}
+	int vertCount = type;
+	for(int i = 0; i < type; i++)
+	{
+		if (duplicatedVert[i])
+			vertCount--;
+	}
+	if (vertCount <= 2)
+		return true;
+
+	// Method 2:
+	// Castlevania Portrait of Ruin - warp stone
+	bool horizontalLine = true;
+	bool verticalLine = true;
+	vert1 = &gfx3d.vertlist->list[poly->vertIndexes[0]];
+	for(int i = 1; i < type && (horizontalLine || verticalLine); i++)
+	{
+		vert2 = &gfx3d.vertlist->list[poly->vertIndexes[i]];
+		if (vert1->coord[0] != vert2->coord[0])
+		{
+			verticalLine = false;
+		}
+		if (vert1->coord[1] != vert2->coord[1])
+		{
+			horizontalLine = false;
+		}
+
+		//the Z is different, and this method isn't even meant to catch that
+		//if (vert1->coord[2] != vert2->coord[2])
+		//	return false;
+	}
+	if (horizontalLine || verticalLine)
+		return true;
+
+	return false;
+}
