@@ -1,22 +1,19 @@
-/*	gfx3d.h
+/*
 	Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2010 DeSmuME team
+	Copyright (C) 2008-2011 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef _GFX3D_H_
@@ -139,6 +136,7 @@ void gfx3d_reset();
 
 struct POLY {
 	int type; //tri or quad
+	u8 vtxFormat;
 	u16 vertIndexes[4]; //up to four verts can be referenced by this poly
 	u32 polyAttr, texParam, texPalette; //the hardware rendering params
 	u32 viewport;
@@ -192,7 +190,6 @@ struct POLY {
 };
 
 #define POLYLIST_SIZE 100000
-//#define POLYLIST_SIZE 2048
 struct POLYLIST {
 	POLY list[POLYLIST_SIZE];
 	int count;
@@ -450,4 +447,6 @@ bool gfx3d_loadstate(EMUFILE* is, int size);
 
 void gfx3d_ClearStack();
 
+bool gfx3d_IsLinePoly(POLY *poly);
+
 #endif //_GFX3D_H_
