@@ -1,4 +1,4 @@
-/* $Id: gdk_3Demu.h,v 1.1 2007/04/21 19:45:07 evilynux Exp $
+/* $Id: gdk_3Demu.h,v 1.1 2007-04-21 19:45:07 evilynux Exp $
  */
 /*  
 	Copyright (C) 2006-2007 Ben Jaques
@@ -27,6 +27,6 @@
  */
 
 int
-init_opengl_gdk_3Demu( void);
+init_opengl_gdk_3Demu( GdkDrawable * drawable);
 
 #endif
