@@ -66,16 +66,14 @@
 
 #include "DeSmuME.xpm"
 
+#undef GPOINTER_TO_INT
+#define GPOINTER_TO_INT(p) ((gint)  (glong) (p))
+
+#undef GPOINTER_TO_UINT
+#define GPOINTER_TO_UINT(p) ((guint)  (glong) (p))
+
 #define EMULOOP_PRIO (G_PRIORITY_HIGH_IDLE + 20)
 
-#if GTK_CHECK_VERSION(2,10,0)
-#define HAVE_RECENT_FILES 1
-#endif
-
-#if GTK_CHECK_VERSION(2,14,0)
-#define HAVE_TIMEOUT 1
-#endif
-
 #define SCREENS_PIXEL_SIZE (256*192*2)
 #define SCREEN_BYTES_PER_PIXEL 3
 #define GAP_SIZE 50
@@ -136,9 +134,7 @@ static const char *ui_description =
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
 "      <menuitem action='open'/>"
-#ifdef HAVE_RECENT_FILES
 "      <menu action='RecentMenu'/>"
-#endif
 "      <separator/>"
 "      <menuitem action='savestateto'/>"
 "      <menuitem action='loadstatefrom'/>"
@@ -185,6 +181,14 @@ static const char *ui_description =
 #ifdef FAKE_MIC
 "      <menuitem action='micnoise'/>"
 #endif
+"      <menu action='SPUModeMenu'>"
+"        <menuitem action='SPUModeDualASync'/>"
+"        <menuitem action='SPUModeSyncN'/>"
+"        <menuitem action='SPUModeSyncZ'/>"
+#ifdef HAVE_LIBSOUNDTOUCH
+"        <menuitem action='SPUModeSyncP'/>"
+#endif
+"      </menu>"
 "      <menu action='FrameskipMenu'>"
 "        <menuitem action='frameskipA'/>"
 "        <menuitem action='frameskip0'/>"
@@ -263,9 +267,7 @@ static const char *ui_description =
 static const GtkActionEntry action_entries[] = {
     { "FileMenu", NULL, "_File" },
       { "open",          "gtk-open",    "_Open",         "<Ctrl>o",  NULL,   OpenNdsDialog },
-#ifdef HAVE_RECENT_FILES
       { "RecentMenu", NULL, "Open _recent" },
-#endif
       { "savestateto",    NULL,         "Save state _to ...",         NULL,  NULL,   SaveStateDialog },
       { "loadstatefrom",  NULL,         "Load state _from ...",         NULL,  NULL,   LoadStateDialog },
       { "recordmovie",  NULL,         "Record movie _to ...",         NULL,  NULL,   RecordMovieDialog },
@@ -283,6 +285,7 @@ static const GtkActionEntry action_entries[] = {
       { "run",        "gtk-media-play",   "_Run",          "<Ctrl>r",  NULL,   Launch },
       { "pause",      "gtk-media-pause",  "_Pause",        "<Ctrl>p",  NULL,   Pause },
       { "reset",      "gtk-refresh",      "Re_set",        NULL,       NULL,   Reset },
+      { "SPUModeMenu", NULL, "_SPU Mode" },
       { "FrameskipMenu", NULL, "_Frameskip" },
       { "LayersMenu", NULL, "_Layers" },
       { "CheatMenu", NULL, "_Cheat" },
@@ -354,6 +357,22 @@ const struct screen_size_t screen_size[ORIENT_N] = {
     {256, 192}
 };
 
+enum spumode_enum {
+    SPUMODE_DUALASYNC = 0,
+    SPUMODE_SYNCN = 1,
+    SPUMODE_SYNCZ = 2,
+    SPUMODE_SYNCP = 3
+};
+
+static const GtkRadioActionEntry spumode_entries[] = {
+    { "SPUModeSyncN", NULL, "Synchronous (N)", NULL, NULL, SPUMODE_SYNCN},
+    { "SPUModeSyncZ", NULL, "Synchronous (Z)", NULL, NULL, SPUMODE_SYNCZ},
+#ifdef HAVE_LIBSOUNDTOUCH
+    { "SPUModeSyncP", NULL, "Synchronous (P)", NULL, NULL, SPUMODE_SYNCP},
+#endif
+    { "SPUModeDualASync", NULL, "Dual Asynchronous", NULL, NULL, SPUMODE_DUALASYNC}
+};
+
 enum frameskip_enum {
     FRAMESKIP_0 = 0,
     FRAMESKIP_1 = 1,
@@ -432,9 +451,7 @@ public:
 
   int firmware_language;
 
-#ifdef HAVE_TIMEOUT
   int timeout;
-#endif
 };
 
 static void
@@ -444,9 +461,7 @@ init_configured_features( class configured_features *config)
 
   config->savetype = 0;
 
-#ifdef HAVE_TIMEOUT
   config->timeout = 0;
-#endif
 
   /* use the default language */
   config->firmware_language = -1;
@@ -481,9 +496,7 @@ fill_configured_features( class configured_features *config,
                                     "\t\t\t\t  4 = Italian\n"
                                     "\t\t\t\t  5 = Spanish\n",
                                     "LANG"},
-#ifdef HAVE_TIMEOUT
     { "timeout", 0, 0, G_OPTION_ARG_INT, &config->timeout, "Quit desmume after the specified seconds for testing purpose.", "SECONDS"},
-#endif
     { NULL }
   };
 
@@ -563,6 +576,7 @@ joinThread_gdb( void *thread_handle) {
 
 /************************ GTK *******************************/
 
+uint SPUMode = SPUMODE_DUALASYNC;
 uint Frameskip = 0;
 bool autoframeskip = false;
 GdkInterpType Interpolation = GDK_INTERP_BILINEAR;
@@ -966,7 +980,6 @@ static void OpenNdsDialog()
             gtk_dialog_run(GTK_DIALOG(pDialog));
             gtk_widget_destroy(pDialog);
         } else {
-#ifdef HAVE_RECENT_FILES
             GtkRecentData recentData;
             gchar *uri;
             memset(&recentData, 0, sizeof(GtkRecentData));
@@ -982,7 +995,6 @@ static void OpenNdsDialog()
             g_free(uri);
             g_free(recentData.app_name);
             g_free(recentData.app_exec);
-#endif
             gtk_action_set_sensitive(gtk_action_group_get_action(action_group, "run"), TRUE);
         }
 
@@ -994,13 +1006,15 @@ static void OpenNdsDialog()
     gtk_widget_destroy(pFileSelection);
 }
 
-#ifdef HAVE_RECENT_FILES
 static void OpenRecent(GtkRecentChooser *chooser, gpointer user_data)
 {
     GtkRecentManager *recent_manager = gtk_recent_manager_get_default();
     gchar *uri, *romname;
     int ret;
 
+    if (desmume_running())
+        Pause();
+
     uri = gtk_recent_chooser_get_current_uri(chooser);
     romname = g_filename_from_uri(uri, NULL, NULL);
     ret = Open(romname);
@@ -1020,14 +1034,11 @@ static void OpenRecent(GtkRecentChooser *chooser, gpointer user_data)
     g_free(uri);
     g_free(romname);
 }
-#endif
 
 static void Reset()
 {
     NDS_Reset();
-    desmume_resume();
-
-    UpdateStatusBar("Running ...");
+    Launch();
 }
 
 
@@ -1129,19 +1140,19 @@ static gboolean ExposeDrawingArea (GtkWidget *widget, GdkEventExpose *event, gpo
     GdkPixbuf *resizedPixbuf, *drawPixbuf;
     guchar rgb[SCREENS_PIXEL_SIZE*SCREEN_BYTES_PER_PIXEL];
     cairo_t *cr;
+    GdkWindow *window;
 
     gfloat vratio, hratio, nscreen_ratio;
     gint daW, daH, imgW, imgH, screenW, screenH, gapW, gapH;
     gint primaryOffsetX, primaryOffsetY, secondaryOffsetX, secondaryOffsetY;
-    gint secondaryPixbufOffsetX, secondaryPixbufOffsetY;
     const gboolean gap_vertical = ((nds_screen.orientation == ORIENT_VERTICAL) ^ (nds_screen.rotation_angle == 90 || nds_screen.rotation_angle == 270));
   
-#if GTK_CHECK_VERSION(2,14,0)
-    gdk_drawable_get_size(
-            gtk_widget_get_window(GTK_WIDGET(pDrawingArea)), &daW, &daH);
+    window = gtk_widget_get_window(GTK_WIDGET(pDrawingArea));
+#if GTK_CHECK_VERSION(2,24,0)
+    daW = gdk_window_get_width(window);
+    daH = gdk_window_get_height(window);
 #else
-    gdk_drawable_get_size(
-            (GTK_WIDGET(pDrawingArea))->window, &daW, &daH);
+    gdk_drawable_get_size(window, &daW, &daH);
 #endif
 
     if (nds_screen.rotation_angle == 0 || nds_screen.rotation_angle == 180) {
@@ -1177,15 +1188,11 @@ static gboolean ExposeDrawingArea (GtkWidget *widget, GdkEventExpose *event, gpo
         screenH = (int)(vratio*(float)imgH*nscreen_ratio);
         secondaryOffsetX = primaryOffsetX;
         secondaryOffsetY = primaryOffsetY + screenH + gapH;
-        secondaryPixbufOffsetX = 0;
-        secondaryPixbufOffsetY = screenH;
     } else {
         screenW = (int)(hratio*(float)imgW*nscreen_ratio);
         screenH = (int)(vratio*(float)imgH);
         secondaryOffsetX = primaryOffsetX + screenW + gapW;
         secondaryOffsetY = primaryOffsetY;
-        secondaryPixbufOffsetX = screenW;
-        secondaryPixbufOffsetY = 0;
     }
 
     if ((nds_screen.swap) ^
@@ -1222,8 +1229,8 @@ static gboolean ExposeDrawingArea (GtkWidget *widget, GdkEventExpose *event, gpo
     if (nds_screen.orientation != ORIENT_SINGLE) {
         gdk_cairo_set_source_pixbuf(cr, drawPixbuf, primaryOffsetX, primaryOffsetY);
     }
-
-    g_object_unref(drawPixbuf);
+    
+    g_object_unref(drawPixbuf); //drawPixbuf was never unref'd, so its ref count stayed above 0 and it was never freed
 
     cairo_paint(cr);
     cairo_destroy(cr);
@@ -1417,7 +1424,7 @@ static void AcceptNewInputKey(GtkWidget *w, GdkEventKey *e, struct modify_key_ct
 
     ctx->mk_key_chosen = e->keyval;
     YouPressed = g_strdup_printf("You pressed : %s\nClick OK to keep this key.", gdk_keyval_name(e->keyval));
-    gtk_label_set(GTK_LABEL(ctx->label), YouPressed);
+    gtk_label_set_text(GTK_LABEL(ctx->label), YouPressed);
     g_free(YouPressed);
 }
 
@@ -1510,7 +1517,7 @@ static void AcceptNewJoyKey(GtkWidget *w, GdkEventFocus *e, struct modify_key_ct
     ctx->mk_key_chosen = get_joy_key(ctx->key_id);
 
     YouPressed = g_strdup_printf("You pressed : %d\nClick OK to keep this key.", ctx->mk_key_chosen);
-    gtk_label_set(GTK_LABEL(ctx->label), YouPressed);
+    gtk_label_set_text(GTK_LABEL(ctx->label), YouPressed);
     g_free(YouPressed);
 }
 
@@ -1669,11 +1676,7 @@ static void Printscreen()
                           NULL);
 
     filen = g_strdup_printf("desmume-screenshot-%d.png", seq);
-#if GLIB_CHECK_VERSION(2,14,0)
     filename = g_build_filename(g_get_user_special_dir(G_USER_DIRECTORY_PICTURES), filen, NULL);
-#else
-    filename = g_build_filename("./", filen, NULL);
-#endif
 
     gdk_pixbuf_save(screenshot, filename, "png", &error, NULL);
     if (error) {
@@ -1748,6 +1751,28 @@ static void Modify_Interpolation(GtkAction *action, GtkRadioAction *current)
     Interpolation = (i == 0 ? GDK_INTERP_NEAREST : GDK_INTERP_BILINEAR);
 }
 
+static void Modify_SPUMode(GtkAction *action, GtkRadioAction *current)
+{
+    const uint mode = gtk_radio_action_get_current_value(current);
+
+    switch (mode) {
+    case SPUMODE_SYNCN:
+    case SPUMODE_SYNCZ:
+#ifdef HAVE_LIBSOUNDTOUCH
+    case SPUMODE_SYNCP:
+#endif
+        SPUMode = mode;
+        SPU_SetSynchMode(1, mode-1);
+        break;
+
+    case SPUMODE_DUALASYNC:
+    default:
+        SPUMode = SPUMODE_DUALASYNC;
+        SPU_SetSynchMode(0, 0);
+        break;
+    }
+}
+
 static void Modify_Frameskip(GtkAction *action, GtkRadioAction *current)
 {
     Frameskip = gtk_radio_action_get_current_value(current) ;
@@ -1966,10 +1991,10 @@ static void ToggleAudio (GtkToggleAction *action)
 {
     if (gtk_toggle_action_get_active(action) == TRUE) {
         SPU_ChangeSoundCore(SNDCORE_SDL, 735 * 4);
-        osd->addLine("Audio enabled\n");
+        osd->addLine("Audio enabled");
     } else {
         SPU_ChangeSoundCore(0, 0);
-        osd->addLine("Audio disabled\n");
+        osd->addLine("Audio disabled");
     }
 }
 
@@ -1980,9 +2005,9 @@ static void ToggleMicNoise (GtkToggleAction *action)
 
     Mic_DoNoise(doNoise);
     if (doNoise)
-       osd->addLine("Fake mic enabled\n");
+       osd->addLine("Fake mic enabled");
     else
-       osd->addLine("Fake mic disabled\n");
+       osd->addLine("Fake mic disabled");
 }
 #endif
 
@@ -1997,7 +2022,6 @@ static void desmume_gtk_menu_tools (GtkActionGroup *ag)
     }
 }
 
-#ifdef HAVE_TIMEOUT
 static gboolean timeout_exit_cb(gpointer data)
 {
     DoQuit();
@@ -2005,8 +2029,6 @@ static gboolean timeout_exit_cb(gpointer data)
 
     return FALSE;
 }
-#endif
-
 
 static int
 common_gtk_main( class configured_features *my_config)
@@ -2019,7 +2041,6 @@ common_gtk_main( class configured_features *my_config)
     GtkWidget *pVBox;
     GtkWidget *pMenuBar;
     GtkWidget *pToolBar;
-    gint pStatusBar_Ctx;
 
 #ifdef GDB_STUB
     gdbstub_handle_t arm9_gdb_stub;
@@ -2163,6 +2184,8 @@ common_gtk_main( class configured_features *my_config)
             my_config->savetype, G_CALLBACK(changesavetype), NULL);
     gtk_action_group_add_radio_actions(action_group, interpolation_entries, G_N_ELEMENTS(interpolation_entries), 
             1, G_CALLBACK(Modify_Interpolation), NULL);
+    gtk_action_group_add_radio_actions(action_group, spumode_entries, G_N_ELEMENTS(spumode_entries),
+            0, G_CALLBACK(Modify_SPUMode), NULL);
     gtk_action_group_add_radio_actions(action_group, frameskip_entries, G_N_ELEMENTS(frameskip_entries), 
             0, G_CALLBACK(Modify_Frameskip), NULL);
     gtk_action_group_add_radio_actions(action_group, rotation_entries, G_N_ELEMENTS(rotation_entries), 
@@ -2193,7 +2216,6 @@ common_gtk_main( class configured_features *my_config)
     pToolBar = gtk_ui_manager_get_widget (ui_manager, "/ToolBar");
     gtk_box_pack_start (GTK_BOX(pVBox), pToolBar, FALSE, FALSE, 0);
 
-#ifdef HAVE_RECENT_FILES
     {
         GtkWidget * recentMenu = gtk_ui_manager_get_widget (ui_manager, "/MainMenu/FileMenu/RecentMenu");
         GtkWidget * recentFiles = gtk_recent_chooser_menu_new();
@@ -2203,7 +2225,6 @@ common_gtk_main( class configured_features *my_config)
         gtk_menu_item_set_submenu(GTK_MENU_ITEM(recentMenu), recentFiles);
         g_signal_connect(G_OBJECT(recentFiles), "item-activated", G_CALLBACK(OpenRecent), NULL);
     }
-#endif
 
     /* Creating the place for showing DS screens */
     pDrawingArea = gtk_drawing_area_new();
@@ -2229,7 +2250,6 @@ common_gtk_main( class configured_features *my_config)
 
     /* Status bar */
     pStatusBar = gtk_statusbar_new();
-    pStatusBar_Ctx = gtk_statusbar_get_context_id(GTK_STATUSBAR(pStatusBar), "Global");
     UpdateStatusBar("Desmume");
     gtk_box_pack_end(GTK_BOX(pVBox), pStatusBar, FALSE, FALSE, 0);
 
@@ -2289,11 +2309,9 @@ common_gtk_main( class configured_features *my_config)
         }
     }
 
-#ifdef HAVE_TIMEOUT
     if (my_config->timeout > 0) {
         g_timeout_add_seconds(my_config->timeout, timeout_exit_cb, GINT_TO_POINTER(my_config->timeout));
     }
-#endif
 
     /* Main loop */
     gtk_main();
