@@ -1,22 +1,20 @@
-/*  Copyright (C) 2006 thoduv
-    Copyright (C) 2006-2007 Theo Berkau
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 thoduv
+	Copyright (C) 2006-2007 Theo Berkau
+	Copyright (C) 2008-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include <stdlib.h>
@@ -27,6 +25,8 @@
 #include "movie.h"
 #include "readwrite.h"
 #include "NDSSystem.h"
+#define TIXML_USE_STL
+#include "utils/tinyxml/tinyxml.h"
 
 //temporary hack until we have better error reporting facilities
 #ifdef _WINDOWS
@@ -87,7 +87,7 @@ static const u32 saveSizes[] = {512,			// 4k
 static const u32 saveSizes_count = ARRAY_SIZE(saveSizes);
 
 //the lookup table from user save types to save parameters
-static const int save_types[][2] = {
+const int save_types[][2] = {
         {MC_TYPE_AUTODETECT,1},
         {MC_TYPE_EEPROM1,MC_SIZE_4KBITS},
         {MC_TYPE_EEPROM2,MC_SIZE_64KBITS},
@@ -201,11 +201,15 @@ void fw_reset_com(memory_chip_t *mc)
 			if (fp)
 			{
 				if (fwrite(&mc->data[0x3FF00], 1, 0x100, fp) == 0x100)		// User Settings
+				{
 					if (fwrite(&mc->data[0x0002A], 1, 0x1D6, fp) == 0x1D6)  // WiFi Settings
+					{
 						if (fwrite(&mc->data[0x3FA00], 1, 0x300, fp) == 0x300)  // WiFi AP Settings
 							printf(" - done\n");
 						else
 							printf(" - failed\n");
+					}
+				}
 				fclose(fp);
 			}
 			else
@@ -407,8 +411,9 @@ void BackupDevice::reset_hardware()
 
 void BackupDevice::reset()
 {
+	memset(&info, 0, sizeof(info));
 	reset_hardware();
-	data.resize(0);
+	resize(0);
 	data_autodetect.resize(0);
 	addr_size = 0;
 	loadfile();
@@ -420,7 +425,7 @@ void BackupDevice::reset()
 		int savetype = save_types[CommonSettings.manualBackupType][0];
 		int savesize = save_types[CommonSettings.manualBackupType][1];
 		ensure((u32)savesize); //expand properly if necessary
-		data.resize(savesize); //truncate if necessary
+		resize(savesize); //truncate if necessary
 		addr_size = addr_size_for_old_save_type(savetype);
 		flush();
 	}
@@ -431,6 +436,16 @@ void BackupDevice::close_rom()
 	flush();
 }
 
+u8 BackupDevice::searchFileSaveType(u32 size)
+{
+	for (u8 i = 1; i < MAX_SAVE_TYPES; i++)
+	{
+		if (size == save_types[i][1])
+			return (i-1);
+	}
+	return 0xFF;
+}
+
 void BackupDevice::reset_command()
 {
 	//printf("MC RESET\n");
@@ -484,6 +499,7 @@ void BackupDevice::reset_command()
 			default:
 				//the archaic case: write the address and then some modulo-4 number of bytes
 				//why modulo 4? who knows.
+				//SM64 (KOR) makes it here with autodetect_size=11 and nothing interesting in the buffer
 				addr_size = autodetect_size & 3;
 				break;
 			}
@@ -675,12 +691,17 @@ void BackupDevice::ensure(u32 addr)
 	u32 size = data.size();
 	if(size<addr)
 	{
-		data.resize(addr);
-		for(u32 i=size;i<addr;i++)
-			data[i] = kUninitializedSaveDataValue;
+		resize(addr);
 	}
 }
 
+void BackupDevice::resize(u32 size)
+{
+	size_t old_size = data.size();
+	data.resize(size);
+	for(u32 i=old_size;i<size;i++)
+		data[i] = kUninitializedSaveDataValue;
+}
 
 u32 BackupDevice::addr_size_for_old_save_size(int bupmem_size)
 {
@@ -724,7 +745,7 @@ void BackupDevice::load_old_state(u32 addr_size, u8* data, u32 datasize)
 {
 	state = RUNNING;
 	this->addr_size = addr_size;
-	this->data.resize(datasize);
+	resize(datasize);
 	memcpy(&this->data[0],data,datasize);
 
 	//dump back out as a dsv, just to keep things sane
@@ -734,11 +755,42 @@ void BackupDevice::load_old_state(u32 addr_size, u8* data, u32 datasize)
 //======================================================================= no$GBA
 //=======================================================================
 //=======================================================================
+const char no_GBA_HEADER_ID[] = "NocashGbaBackupMediaSavDataFile";
+const char no_GBA_HEADER_SRAM_ID[] = "SRAM";
+
+u32 BackupDevice::get_save_nogba_size(const char* fname)
+{
+	FILE *fsrc = fopen(fname, "rb");
+	if (fsrc)
+	{
+		char src[0x50] = {0};
+		u32 fsize = 0;
+		fseek(fsrc, 0, SEEK_END);
+		fsize = ftell(fsrc);
+		fseek(fsrc, 0, SEEK_SET);
+		if (fsize < 0x50) { fclose(fsrc); return 0xFFFFFFFF; }
+		memset(&src[0], 0, sizeof(src));
+		if (fread(src, 1, sizeof(src), fsrc) != sizeof(src))  { fclose(fsrc); return 0xFFFFFFFF; }
+
+		for (u8 i = 0; i < 0x1F; i++)
+			if (src[i] != no_GBA_HEADER_ID[i]) { fclose(fsrc); return 0xFFFFFFFF; }
+		if (src[0x1F] != 0x1A) { fclose(fsrc); return 0xFFFFFFFF; }
+		for (int i = 0; i < 0x4; i++)
+			if (src[i+0x40] != no_GBA_HEADER_SRAM_ID[i]) { fclose(fsrc); return 0xFFFFFFFF; }
+		
+		u32 compressMethod = *((u32*)(src+0x44));
+		if (compressMethod == 0)
+			{ fclose(fsrc); return *((u32*)(src+0x48)); }
+		else
+			if (compressMethod == 1)
+				{ fclose(fsrc); return *((u32*)(src+0x4C)); }
+		fclose(fsrc);
+	}
+	return 0xFFFFFFFF;
+}
 
 static int no_gba_unpackSAV(void *in_buf, u32 fsize, void *out_buf, u32 &size)
 {
-	const char no_GBA_HEADER_ID[] = "NocashGbaBackupMediaSavDataFile";
-	const char no_GBA_HEADER_SRAM_ID[] = "SRAM";
 	u8	*src = (u8 *)in_buf;
 	u8	*dst = (u8 *)out_buf;
 	u32 src_pos = 0;
@@ -850,7 +902,7 @@ static u32 no_gba_fillLeft(u32 size)
 	return size;
 }
 
-bool BackupDevice::load_no_gba(const char *fname)
+bool BackupDevice::load_no_gba(const char *fname, u32 force_size)
 {
 	FILE	*fsrc = fopen(fname, "rb");
 	u8		*in_buf = NULL;
@@ -874,13 +926,15 @@ bool BackupDevice::load_no_gba(const char *fname)
 			memset(out_buf, 0xFF, 8 * 1024 * 1024 / 8);
 			if (no_gba_unpackSAV(in_buf, fsize, out_buf, size) == 0)
 			{
+				if (force_size > 0)
+					size = force_size;
 				//printf("New size %i byte(s)\n", size);
 				size = no_gba_savTrim(out_buf, size);
 				//printf("--- new size after trim %i byte(s)\n", size);
 				size = no_gba_fillLeft(size);
 				//printf("--- new size after fill %i byte(s)\n", size);
-				raw_applyUserSettings(size);
-				data.resize(size);
+				raw_applyUserSettings(size, (force_size > 0));
+				resize(size);
 				for (u32 tt = 0; tt < size; tt++)
 					data[tt] = out_buf[tt];
 
@@ -985,24 +1039,56 @@ void BackupDevice::loadfile()
 			return;
 		}
 		inf->fseek(-24, SEEK_CUR);
-		struct {
-			u32 size,padSize,type,addr_size,mem_size;
-		} info;
 		read32le(&info.size,inf);
 		read32le(&info.padSize,inf);
 		read32le(&info.type,inf);
 		read32le(&info.addr_size,inf);
 		read32le(&info.mem_size,inf);
 
+		u32 left = 0;
+		if (CommonSettings.autodetectBackupMethod == 1)
+		{
+			if (advsc.isLoaded())
+			{
+				info.type = advsc.getSaveType();
+				if (info.type != 0xFF || info.type != 0xFE)
+				{
+					u32 adv_size = save_types[info.type+1][1];
+					if (info.size > adv_size)
+						info.size = adv_size;
+					else
+						if (info.size < adv_size)
+						{
+							left = adv_size - info.size;
+							info.size = adv_size;
+						}
+				}
+			}
+		}
 		//establish the save data
-		data.resize(info.size);
+		resize(info.size);
 		inf->fseek(0, SEEK_SET);
 		if(info.size>0)
-			inf->fread(&data[0],info.size); //read all the raw data we have
+			inf->fread(&data[0],info.size - left); //read all the raw data we have
 		state = RUNNING;
 		addr_size = info.addr_size;
 		//none of the other fields are used right now
 
+
+		if (CommonSettings.autodetectBackupMethod != 1 && info.type == 0) 
+		{
+			info.type = searchFileSaveType(info.size);
+			if (info.type == 0xFF) info.type = 0;
+		}
+		u32 ss = info.size * 8 / 1024;
+		if (ss >= 1024)
+		{
+			ss /= 1024;
+			printf("Backup size: %i Mbit\n", ss);
+		}
+		else
+			printf("Backup size: %i Kbit\n", ss);
+
 		delete inf;
 	}
 }
@@ -1049,6 +1135,8 @@ void BackupDevice::flush()
 	//never use save files if we are in movie mode
 	if(isMovieMode) return;
 
+	if (filename.length() == 0) return;
+
 	EMUFILE* outf = new EMUFILE_FILE(filename.c_str(),"wb");
 	if(!outf->fail())
 	{
@@ -1071,9 +1159,9 @@ void BackupDevice::flush()
 		//and now the actual footer
 		write32le(size,outf); //the size of data that has actually been written
 		write32le(padSize,outf); //the size we padded it to
-		write32le(0,outf); //save memory type
+		write32le(info.type,outf); //save memory type
 		write32le(addr_size,outf);
-		write32le(0,outf); //save memory size
+		write32le(info.size,outf); //save memory size
 		write32le(0,outf); //version number
 		outf->fprintf("%s", kDesmumeSaveCookie); //this is what we'll use to recognize the desmume format save
 
@@ -1082,41 +1170,74 @@ void BackupDevice::flush()
 	else
 	{
 		delete outf;
-		printf("Unable to open savefile %s\n",filename.c_str());
+		printf("Unable to open savefile %s\n", filename.c_str());
 	}
 }
 
-void BackupDevice::raw_applyUserSettings(u32& size)
+void BackupDevice::raw_applyUserSettings(u32& size, bool manual)
 {
 	//respect the user's choice of backup memory type
-	if(CommonSettings.manualBackupType == MC_TYPE_AUTODETECT)
+	if(CommonSettings.manualBackupType == MC_TYPE_AUTODETECT && !manual)
 	{
 		addr_size = addr_size_for_old_save_size(size);
-		data.resize(size);
+		resize(size);
 	}
 	else
 	{
-		int savetype = save_types[CommonSettings.manualBackupType][0];
-		int savesize = save_types[CommonSettings.manualBackupType][1];
+		u32 type = CommonSettings.manualBackupType;
+		if (manual)
+		{
+			u32 res = searchFileSaveType(size);
+			if (res != 0xFF) type = (res + 1); // +1 - skip autodetect
+		}
+		int savetype = save_types[type][0];
+		int savesize = save_types[type][1];
 		addr_size = addr_size_for_old_save_type(savetype);
 		if((u32)savesize<size) size = savesize;
-		data.resize(savesize);
+		resize(savesize);
 	}
 
 	state = RUNNING;
 }
 
-
-bool BackupDevice::load_raw(const char* filename)
+u32 BackupDevice::get_save_raw_size(const char* fname)
 {
-	FILE* inf = fopen(filename,"rb");
+	FILE* inf = fopen(fname,"rb");
+	if (!inf) return 0xFFFFFFFF;
+
 	fseek(inf, 0, SEEK_END);
 	u32 size = (u32)ftell(inf);
+	fclose(inf);
+	return size;
+}
+
+bool BackupDevice::load_raw(const char* filename, u32 force_size)
+{
+	FILE* inf = fopen(filename,"rb");
+
+	if (!inf) return false;
+
+	fseek(inf, 0, SEEK_END);
+	u32 size = (u32)ftell(inf);
+	u32 left = 0;
+	
+	if (force_size > 0)
+	{
+		if (size > force_size)
+			size = force_size;
+		else
+			if (size < force_size)
+			{
+				left = force_size - size;
+				size = force_size;
+			}
+	}
+
 	fseek(inf, 0, SEEK_SET);
 
-	raw_applyUserSettings(size);
+	raw_applyUserSettings(size, (force_size > 0));
 
-	fread(&data[0],1,size,inf);
+	fread(&data[0],1,size-left,inf);
 	fclose(inf);
 
 	//dump back out as a dsv, just to keep things sane
@@ -1125,14 +1246,25 @@ bool BackupDevice::load_raw(const char* filename)
 	return true;
 }
 
+u32 BackupDevice::get_save_duc_size(const char* fname)
+{
+	FILE* inf = fopen(fname,"rb");
+	if (!inf) return 0xFFFFFFFF;
 
-bool BackupDevice::load_duc(const char* filename)
+	fseek(inf, 0, SEEK_END);
+	u32 size = (u32)ftell(inf);
+	fclose(inf);
+	if (size < 500) return 0xFFFFFFFF;
+	return (size - 500);
+}
+
+bool BackupDevice::load_duc(const char* filename, u32 force_size)
 {
   u32 size;
    char id[16];
    FILE* file = fopen(filename, "rb");
-   if(file == NULL)
-      return false;
+   
+   if(!file) return false;
 
    fseek(file, 0, SEEK_END);
    size = (u32)ftell(file) - 500;
@@ -1143,18 +1275,31 @@ bool BackupDevice::load_duc(const char* filename)
 
    if (memcmp(id, "ARDS000000000001", 16) != 0)
    {
-	   printf("Not recognized as a valid DUC file\n");
-      fclose(file);
-      return false;
+		printf("Not recognized as a valid DUC file\n");
+		fclose(file);
+		return false;
    }
    // Skip the rest of the header since we don't need it
    fseek(file, 500, SEEK_SET);
 
-   raw_applyUserSettings(size);
+	u32 left = 0;
+	if (force_size > 0)
+	{
+		if (size > force_size)
+			size = force_size;
+		else
+			if (size < force_size)
+			{
+				left = force_size - size;
+				size = force_size;
+			}
+	}
+
+   raw_applyUserSettings(size, (force_size > 0));
 
    ensure((u32)size);
 
-   fread(&data[0],1,size,file);
+   fread(&data[0],1,size-left,file);
    fclose(file);
 
    //choose 
@@ -1191,7 +1336,7 @@ bool BackupDevice::load_movie(EMUFILE* is) {
 	is->fread((char*)&info.mem_size,4);
 
 	//establish the save data
-	data.resize(info.size);
+	resize(info.size);
 	is->fseek(0, SEEK_SET);
 	if(info.size>0)
 		is->fread((char*)&data[0],info.size);
@@ -1207,4 +1352,193 @@ void BackupDevice::forceManualBackupType()
 {
 	addr_size = addr_size_for_old_save_size(save_types[CommonSettings.manualBackupType][1]);
 	state = RUNNING;
-}
\ No newline at end of file
+}
+
+// ============================================= ADVANsCEne
+u8 ADVANsCEne::checkDB(const char *serial)
+{
+	loaded = false;
+	FILE *fp = fopen(database_path, "rb");
+	if (fp)
+	{
+		char buf[64];
+		memset(buf, 0, sizeof(buf));
+		if (fread(buf, 1, strlen(_ADVANsCEne_BASE_ID), fp) == strlen(_ADVANsCEne_BASE_ID))
+		{
+			//printf("ID: %s\n", buf);
+			if (strcmp(buf, _ADVANsCEne_BASE_ID) == 0)
+			{
+				if (fread(&versionBase[0], 1, 2, fp) == 2)
+				{
+					//printf("Version base: %i.%i\n", versionBase[0], versionBase[1]);
+					if (fread(&version[0], 1, 4, fp) == 4)
+					{
+						//printf("Version: %c%c%c%c\n", version[3], version[2], version[1], version[0]);
+						if (fread(&createTime, 1, sizeof(time_t), fp) == sizeof(time_t))
+						{
+							memset(buf, 0,sizeof(buf));
+							// serial(8) + crc32(4) + save_type(1) = 13 + reserved(8) = 21
+							while (true)
+							{
+								if (fread(buf, 1, 21, fp) != 21) break;
+								if (memcmp(&buf[4], serial, 4) == 0)
+								{
+									memcpy(&crc32, &buf[8], 4);
+									//printf("%s founded: crc32=%04X, save type %02X\n", serial, crc32, buf[12]);
+									saveType = buf[12];
+									fclose(fp);
+									loaded = true;
+									return true;
+								}
+							}
+						}
+					}
+				}
+			}
+		}
+		fclose(fp);
+	}
+	return false;
+}
+
+bool ADVANsCEne::getXMLConfig(const char *in_filaname)
+{
+	TiXmlDocument	*xml = NULL;
+	TiXmlElement	*el = NULL;
+	TiXmlElement	*el_configuration = NULL;
+	TiXmlElement	*el_newDat = NULL;
+	
+	xml = new TiXmlDocument();
+	if (!xml) return false;
+	if (!xml->LoadFile(in_filaname)) return false;
+	el = xml->FirstChildElement("dat");
+	if (!el) return false;
+	el_configuration = el->FirstChildElement("configuration");
+	if (!el_configuration) return false;
+	el = el_configuration->FirstChildElement("datName"); if (el) { datName = el->GetText(); }
+	el = el_configuration->FirstChildElement("datVersion"); if (el) { datVersion = el->GetText(); }
+	
+	el_newDat = el_configuration->FirstChildElement("newDat");
+	if (!el_newDat) return false;
+	el = el_newDat->FirstChildElement("datVersionURL"); if (el) { urlVersion = el->GetText(); }
+	el = el_newDat->FirstChildElement("datURL"); if (el) { urlDat = el->GetText(); }
+
+	delete xml;
+	return true;
+}
+
+u32 ADVANsCEne::convertDB(const char *in_filaname)
+{
+	const char *saveTypeNames[] = {	"Eeprom - 4 kbit",		// EEPROM 4kbit
+									"Eeprom - 64 kbit",		// EEPROM 64kbit
+									"Eeprom - 512 kbit",	// EEPROM 512kbit
+									"Fram - 256 kbit",		// FRAM 256kbit !
+									"Flash - 2 mbit",		// FLASH 2Mbit
+									"Flash - 4 mbit",		// FLASH 4Mbit
+									"Flash - 8 mbit",		// FLASH 8Mbit
+									"Flash - 16 mbit",		// FLASH 16Mbit !
+									"Flash - 32 mbit",		// FLASH 32Mbit !
+									"Flash - 64 mbit",		// FLASH 64Mbit
+									"Flash - 128 mbit",		// FLASH 128Mbit !
+									"Flash - 256 mbit",		// FLASH 256Mbit !
+									"Flash - 512 mbit"		// FLASH 512Mbit !
+	};
+
+	TiXmlDocument	*xml = NULL;
+	TiXmlElement	*el = NULL;
+	TiXmlElement	*el_serial = NULL;
+	TiXmlElement	*el_games = NULL;
+	TiXmlElement	*el_crc32 = NULL;
+	TiXmlElement	*el_saveType = NULL;
+	FILE			*fp;
+	u32				crc32 = 0;
+	u32				reserved = 0;
+
+	printf("Converting DB...\n");
+	if (getXMLConfig(in_filaname))
+	{
+		if (!datName) return 0;
+		if (strcmp(datName, _ADVANsCEne_BASE_NAME) != 0) return 0;
+	}
+
+	fp = fopen(database_path, "wb");
+	if (!fp) return 0;
+
+	// Header
+	fwrite(_ADVANsCEne_BASE_ID, 1, strlen(_ADVANsCEne_BASE_ID), fp);
+	fputc(_ADVANsCEne_BASE_VERSION_MAJOR, fp);
+	fputc(_ADVANsCEne_BASE_VERSION_MINOR, fp);
+	if (datVersion) 
+		fwrite(datVersion, 1, strlen(datVersion), fp);
+	else
+		fputc(0, fp);
+	time_t __time = time(NULL);
+	fwrite(&__time, 1, sizeof(time_t), fp);
+
+	xml = new TiXmlDocument();
+	if (!xml) { fclose(fp); return 0; }
+	if (!xml->LoadFile(in_filaname)) { fclose(fp); return 0; }
+	el = xml->FirstChildElement("dat");
+	if (!el) { fclose(fp); return 0; }
+	el_games = el->FirstChildElement("games");
+	if (!el_games) { fclose(fp); return 0; }
+	el = el_games->FirstChildElement("game");
+	if (!el) { fclose(fp); return 0; }
+	u32 count = 0;
+	while (el)
+	{
+		el_serial = el->FirstChildElement("serial");
+		if (fwrite(el_serial->GetText(), 1, 8, fp) != 8)
+		{
+			fclose(fp); return 0;
+		}
+		// CRC32
+		el_crc32 = el->FirstChildElement("files"); 
+		sscanf_s(el_crc32->FirstChildElement("romCRC")->GetText(), "%x", &crc32);
+		if (fwrite(&crc32, 1, sizeof(u32), fp) != sizeof(u32))
+		{
+			fclose(fp); return 0;
+		}
+		// Save type
+		el_saveType = el->FirstChildElement("saveType"); 
+		if (el_saveType)
+		{
+			const char *tmp = el_saveType->GetText();
+			if (tmp)
+			{
+				if (strcmp(tmp, "None")  == 0)
+					fputc(0xFE, fp);
+				else
+				{
+					bool bUnknown = true;
+					for (u8 i = 0; i < MAX_SAVE_TYPES; i++)
+					{
+						if (strcmp(saveTypeNames[i], "") == 0) continue;
+						if (strcasecmp(tmp, saveTypeNames[i]) == 0)
+						{
+							fputc(i, fp);
+							bUnknown = false;
+							break;
+						}
+					}
+					if (bUnknown)
+						fputc(0xFF, fp);	// Unknown
+				}
+			}
+			else
+				fputc(0xFF, fp);	// Unknown
+		}
+		fwrite(&reserved, 1, sizeof(u32), fp);
+		fwrite(&reserved, 1, sizeof(u32), fp);
+		count++;
+		el = el->NextSiblingElement("game");
+	}
+	delete xml;
+	fclose(fp);
+	if (count > 0) 
+		printf("done\n");
+	else
+		printf("error\n");
+	printf("ADVANsCEne converter: %i found\n", count);
+	return count;
+}
