@@ -1,22 +1,20 @@
-/*  Copyright (C) 2006 thoduv
-    Copyright (C) 2006 Theo Berkau
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 thoduv
+	Copyright (C) 2006 Theo Berkau
+	Copyright (C) 2008-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef __FW_H__
@@ -27,6 +25,8 @@
 #include <string>
 #include "types.h"
 #include "emufile.h"
+#include "common.h"
+#include "utils/tinyxml/tinyxml.h"
 
 #define MAX_SAVE_TYPES 13
 #define MC_TYPE_AUTODETECT      0x0
@@ -50,6 +50,46 @@
 #define MC_SIZE_256MBITS                0x2000000
 #define MC_SIZE_512MBITS                0x4000000
 
+// ============================================= ADVANsCEne
+#define _ADVANsCEne_BASE_ID "DeSmuME database (ADVANsCEne)\0x1A"
+#define _ADVANsCEne_BASE_VERSION_MAJOR 1
+#define _ADVANsCEne_BASE_VERSION_MINOR 0
+#define _ADVANsCEne_BASE_NAME "ADVANsCEne Nintendo DS Collection"
+class ADVANsCEne
+{
+private:
+	char			database_path[MAX_PATH];		// DeSmuME save types 
+	u8				versionBase[2];
+	char			version[4];
+	time_t			createTime;
+	u8				saveType;
+	u32				crc32;
+	bool			loaded;
+
+	// XML
+	const char		*datName;
+	const char		*datVersion;
+	const char		*urlVersion;
+	const char		*urlDat;
+	bool getXMLConfig(const char *in_filaname);
+public:
+	ADVANsCEne() :	saveType(0xFF),
+					crc32(0),
+					loaded(false)
+	{
+		memset(database_path, 0, sizeof(database_path));
+		memset(versionBase, 0, sizeof(versionBase));
+		memset(version, 0, sizeof(version));
+	}
+	void setDatabase(const char *path) { loaded = false; strcpy(database_path, path); }
+	u32 convertDB(const char *in_filaname);
+	u8 checkDB(const char *serial);
+	u32 getSaveType() { return saveType; }
+	u32 getCRC32() { return crc32; }
+	bool isLoaded() { return loaded; }
+};
+
+
 struct memory_chip_t
 {
 	u8 com;	//persistent command actually handled
@@ -70,7 +110,7 @@ struct memory_chip_t
 	
 	// needs only for firmware
 	bool isFirmware;
-	char userfile[260];
+	char userfile[MAX_PATH];
 };
 
 //the new backup system by zeromus
@@ -88,6 +128,8 @@ public:
 	void close_rom();
 	void forceManualBackupType();
 	void reset_hardware();
+	std::string getFilename() { return filename; }
+	u8 searchFileSaveType(u32 size);
 
 	bool save_state(EMUFILE* os);
 	bool load_state(EMUFILE* is);
@@ -109,14 +151,17 @@ public:
 	static u32 addr_size_for_old_save_type(int bupmem_type);
 
 	static u32 pad_up_size(u32 startSize);
-	void raw_applyUserSettings(u32& size);
+	void raw_applyUserSettings(u32& size, bool manual = false);
 
-	bool load_duc(const char* filename);
-	bool load_no_gba(const char *fname);
+	bool load_duc(const char* filename, u32 force_size = 0);
+	bool load_no_gba(const char *fname, u32 force_size = 0);
 	bool save_no_gba(const char* fname);
-	bool load_raw(const char* filename);
+	bool load_raw(const char* filename, u32 force_size = 0);
 	bool save_raw(const char* filename);
 	bool load_movie(EMUFILE* is);
+	u32 get_save_duc_size(const char* filename);
+	u32 get_save_nogba_size(const char* filename);
+	u32 get_save_raw_size(const char* filename);
 
 	//call me once a second or so to lazy flush the save data
 	//here's the reason for this system: we want to dump save files when theyre READ
@@ -125,17 +170,19 @@ public:
 	void lazy_flush();
 	void flush();
 
-public: //SHOULD BE PRIVATE!!!!!!!!
-	std::string filename;
-	bool isMovieMode;
+	struct {
+			u32 size,padSize,type,addr_size,mem_size;
+		} info;
 
+	bool isMovieMode;
 private:
+	std::string filename;
+	
 	bool write_enable;	//is write enabled?
 	u32 com;	//persistent command actually handled
 	u32 addr_size, addr_counter;
 	u32 addr;
 
-	
 	std::vector<u8> data_autodetect;
 	enum STATE {
 		DETECTING = 0, RUNNING = 1
@@ -157,9 +204,11 @@ private:
 	void loadfile();
 	bool _loadfile(const char *fname);
 	void ensure(u32 addr);
-	
 
 	bool flushPending, lazyFlushPending;
+
+private:
+	void resize(u32 size);
 };
 
 #define NDS_FW_SIZE_V1 (256 * 1024)		/* size of fw memory on nds v1 */
@@ -177,6 +226,7 @@ void backup_setManualBackupType(int type);
 void backup_forceManualBackupType();
 
 extern const char *save_names[];
+extern const int save_types[][2];
 
 #endif /*__FW_H__*/
 
