@@ -1,20 +1,18 @@
-/*  Copyright 2009-2010 DeSmuME team
+/*
+	Copyright (C) 2009-2012 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 //nothing in this file should be assumed to be accurate
@@ -35,6 +33,10 @@
 #include <math.h>
 #include <string.h>
 
+#ifndef _MSC_VER
+#include <stdint.h>
+#endif
+
 #include "bits.h"
 #include "common.h"
 #include "matrix.h"
@@ -223,16 +225,30 @@ FORCEINLINE edge_fx_fl::edge_fx_fl(int Top, int Bottom, VERT** verts, bool& fail
 	Y = Ceil28_4((fixed28_4)verts[Top]->y);
 	int YEnd = Ceil28_4((fixed28_4)verts[Bottom]->y);
 	Height = YEnd - Y;
+	X = Ceil28_4((fixed28_4)verts[Top]->x);
+	int XEnd = Ceil28_4((fixed28_4)verts[Bottom]->x);
+	int Width = XEnd - X; // can be negative
 
-	if(Height)
+	// even if Height == 0, give some info for horizontal line poly
+	if(Height != 0 || Width != 0)
 	{
 		long dN = long(verts[Bottom]->y - verts[Top]->y);
 		long dM = long(verts[Bottom]->x - verts[Top]->x);
-	
-		long InitialNumerator = (long)(dM*16*Y - dM*verts[Top]->y + dN*verts[Top]->x - 1 + dN*16);
-		FloorDivMod(InitialNumerator,dN*16,X,ErrorTerm,failure);
-		FloorDivMod(dM*16,dN*16,XStep,Numerator,failure);
-		Denominator = dN*16;
+		if (dN != 0)
+		{
+			long InitialNumerator = (long)(dM*16*Y - dM*verts[Top]->y + dN*verts[Top]->x - 1 + dN*16);
+			FloorDivMod(InitialNumerator,dN*16,X,ErrorTerm,failure);
+			FloorDivMod(dM*16,dN*16,XStep,Numerator,failure);
+			Denominator = dN*16;
+		}
+		else
+		{
+			XStep = Width;
+			Numerator = 0;
+			ErrorTerm = 0;
+			Denominator = 1;
+			dN = 1;
+		}
 	
 		float YPrestep = Fixed28_4ToFloat((fixed28_4)(Y*16 - verts[Top]->y));
 		float XPrestep = Fixed28_4ToFloat((fixed28_4)(X*16 - verts[Top]->x));
@@ -247,6 +263,11 @@ FORCEINLINE edge_fx_fl::edge_fx_fl(int Top, int Bottom, VERT** verts, bool& fail
 		for(int i=0;i<3;i++)
 			color[i].initialize(verts[Top]->fcolor[i],verts[Bottom]->fcolor[i],dx,dy,XStep,XPrestep,YPrestep);
 	}
+	else
+	{
+		memset(this, 0, sizeof(*this));
+		this->verts = verts;
+	}
 }
 
 FORCEINLINE int edge_fx_fl::Step() {
@@ -268,10 +289,11 @@ static FORCEINLINE void alphaBlend(FragmentColor & dst, const FragmentColor & sr
 {
 	if(gfx3d.renderState.enableAlphaBlending)
 	{
-		if(src.a == 0 || dst.a == 0)
+		if(src.a == 31 || dst.a == 0)
 		{
 			dst = src;
 		}
+		//else if(src.a == 0) { } //this is not necessary since it was handled earlier
 		else
 		{
 			u8 alpha = src.a+1;
@@ -296,6 +318,7 @@ static FORCEINLINE void alphaBlend(FragmentColor & dst, const FragmentColor & sr
 	}
 }
 
+// TODO: wire-frame
 struct PolyAttr
 {
 	u32 val;
@@ -434,7 +457,7 @@ public:
 
 	FORCEINLINE FragmentColor sample(float u, float v)
 	{
-		static FragmentColor white = MakeFragmentColor(63,63,63,31);
+		static const FragmentColor white = MakeFragmentColor(63,63,63,31);
 		if(!sampler.enabled) return white;
 
 		//finally, we can use floor here. but, it is slower than we want.
@@ -687,11 +710,23 @@ public:
 	}
 
 	//draws a single scanline
-	FORCEINLINE void drawscanline(edge_fx_fl *pLeft, edge_fx_fl *pRight)
+	FORCEINLINE void drawscanline(edge_fx_fl *pLeft, edge_fx_fl *pRight, bool lineHack)
 	{
 		int XStart = pLeft->X;
 		int width = pRight->X - XStart;
 
+		// HACK: workaround for vertical/slant line poly
+		if (lineHack && width == 0)
+		{
+			int leftWidth = pLeft->XStep;
+			if (pLeft->ErrorTerm + pLeft->Numerator >= pLeft->Denominator)
+				leftWidth++;
+			int rightWidth = pRight->XStep;
+			if (pRight->ErrorTerm + pRight->Numerator >= pRight->Denominator)
+				rightWidth++;
+			width = max(1, max(abs(leftWidth), abs(rightWidth)));
+		}
+
 		//these are the starting values, taken from the left edge
 		float invw = pLeft->invw.curr;
 		float u = pLeft->u.curr;
@@ -728,16 +763,36 @@ public:
 
 		int x = XStart;
 
+		if(x<0)
+		{
+			if(RENDERER && !lineHack)
+			{
+				printf("rasterizer rendering at x=%d! oops!\n",x);
+				return;
+			}
+			invw += dinvw_dx * -x;
+			u += du_dx * -x;
+			v += dv_dx * -x;
+			z += dz_dx * -x;
+			color[0] += dc_dx[0] * -x;
+			color[1] += dc_dx[1] * -x;
+			color[2] += dc_dx[2] * -x;
+			adr += -x;
+			width -= -x;
+			x = 0;
+		}
+		if(x+width > (RENDERER?256:engine->width))
+		{
+			if(RENDERER && !lineHack)
+			{
+				printf("rasterizer rendering at x=%d! oops!\n",x+width-1);
+				return;
+			}
+			width = (RENDERER?256:engine->width)-x;
+		}
+
 		while(width-- > 0)
 		{
-			if(RENDERER && (x<0 || x>255)) {
-				printf("rasterizer rendering at x=%d! oops!\n",x);
-				return;
-			}
-			if(!RENDERER && (x<0 || x>=engine->width)) {
-				printf("rasterizer rendering at x=%d! oops!\n",x);
-				return;
-			}
 			pixel(adr,color[0],color[1],color[2],u,v,1.0f/invw,z);
 			adr++;
 			x++;
@@ -754,22 +809,28 @@ public:
 
 	//runs several scanlines, until an edge is finished
 	template<bool SLI>
-	void runscanlines(edge_fx_fl *left, edge_fx_fl *right,bool horizontal)
+	void runscanlines(edge_fx_fl *left, edge_fx_fl *right, bool horizontal, bool lineHack)
 	{
 		//oh lord, hack city for edge drawing
 
 		//do not overstep either of the edges
 		int Height = min(left->Height,right->Height);
 		bool first=true;
-		static int runctr=0;
-		runctr++;
+
+		//HACK: special handling for horizontal line poly
+		if (lineHack && left->Height == 0 && right->Height == 0 && left->Y<192 && left->Y>=0)
+		{
+			bool draw = (!SLI || (left->Y & SLI_MASK) == SLI_VALUE);
+			if(draw) drawscanline(left,right,lineHack);
+		}
+
 		while(Height--) {
 			bool draw = (!SLI || (left->Y & SLI_MASK) == SLI_VALUE);
-			if(draw) drawscanline(left,right);
+			if(draw) drawscanline(left,right,lineHack);
 			const int xl = left->X;
 			const int xr = right->X;
 			const int y = left->Y;
-			left->Step(); 
+			left->Step();
 			right->Step();
 
 			if(!RENDERER && _debug_thisPoly)
@@ -859,7 +920,15 @@ public:
 		}
 		
 		while(verts[0]->y == verts[1]->y && verts[0]->x > verts[1]->x)
+		{
 			rot_verts<type>();
+			// hack for VC++ 2010 (bug in compiler optimization?)
+			// freeze on 3D
+			// TODO: study it
+			#if defined(_MSC_VER) && _MSC_VER == 1600
+				Sleep(0); // nop
+			#endif
+		}
 		
 	}
 
@@ -868,7 +937,7 @@ public:
 	//I didnt reference anything for this algorithm but it seems like I've seen it somewhere before.
 	//Maybe it is like crow's algorithm
 	template<bool SLI>
-	void shape_engine(int type, bool backwards)
+	void shape_engine(int type, bool backwards, bool lineHack)
 	{
 		bool failure = false;
 
@@ -906,7 +975,7 @@ public:
 				return;
 
 			bool horizontal = left.Y == right.Y;
-			runscanlines<SLI>(&left,&right,horizontal);
+			runscanlines<SLI>(&left,&right,horizontal, lineHack);
 
 			//if we ran out of an edge, step to the next one
 			if(right.Height == 0) {
@@ -976,7 +1045,7 @@ public:
 
 			polyAttr.backfacing = engine->polyBackfacing[i];
 
-			shape_engine<SLI>(type,!polyAttr.backfacing);
+			shape_engine<SLI>(type,!polyAttr.backfacing, gfx3d_IsLinePoly(poly));
 		}
 	}
 
@@ -1070,7 +1139,7 @@ static char SoftRastInit(void)
 
 	TexCache_Reset();
 
-	printf("SoftRast Initialized\n");
+	printf("SoftRast Initialized with cores=%d\n",rasterizerCores);
 	return 1;
 }
 
@@ -1348,6 +1417,15 @@ void SoftRasterizerEngine::framebufferProcess()
 			destFragmentColor.a = ((128-fog)*destFragmentColor.a + a*fog)>>7;
 		}
 	}
+
+	////debug alpha channel framebuffer contents
+	//for(int i=0;i<256*192;i++)
+	//{
+	//	FragmentColor &destFragmentColor = screenColor[i];
+	//	destFragmentColor.r = destFragmentColor.a;
+	//	destFragmentColor.g = destFragmentColor.a;
+	//	destFragmentColor.b = destFragmentColor.a;
+	//}
 }
 
 void SoftRasterizerEngine::performClipping(bool hirez)
