@@ -1,7 +1,8 @@
-/*  Copyright (C) 2006 Normmatt
-    Copyright (C) 2006 Theo Berkau
-    Copyright (C) 2007 Pascal Giard
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 Normmatt
+	Copyright (C) 2006 Theo Berkau
+	Copyright (C) 2007 Pascal Giard
+	Copyright (C) 2008-2012 DeSmuME team
 
 	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -161,16 +162,21 @@ SFORMAT SF_NDS[]={
 	{ "_TCY", 8, 8, nds.timerCycle},
 	{ "_VCT", 4, 1, &nds.VCount},
 	{ "_OLD", 4, 1, &nds.old},
-	{ "_TPX", 2, 1, &nds.touchX},
-	{ "_TPY", 2, 1, &nds.touchY},
+	{ "_TPX", 2, 1, &nds.adc_touchX},
+	{ "_TPY", 2, 1, &nds.adc_touchY},
+	{ "_TPC", 2, 1, &nds.adc_jitterctr},
+	{ "_STX", 2, 1, &nds.scr_touchX},
+	{ "_STY", 2, 1, &nds.scr_touchY},
 	{ "_TPB", 4, 1, &nds.isTouch},
-	{ "_DBG", 4, 1, &nds.debugConsole},
+	{ "_DBG", 4, 1, &nds._DebugConsole},
 	{ "_ENS", 4, 1, &nds.ensataEmulation},
+	{ "_TYP", 4, 1, &nds.ConsoleType},
 	{ "_ENH", 4, 1, &nds.ensataHandshake},
 	{ "_ENI", 4, 1, &nds.ensataIpcSyncCounter},
 	{ "_SLP", 4, 1, &nds.sleeping},
 	{ "_FBS", 4, 1, &nds.freezeBus},
 	{ "_CEJ", 4, 1, &nds.cardEjected},
+	{ "_PDL", 2, 1, &nds.paddle},
 	{ "_P00", 1, 1, &nds.power1.lcd},
 	{ "_P01", 1, 1, &nds.power1.gpuMain},
 	{ "_P02", 1, 1, &nds.power1.gfx3d_render},
@@ -199,6 +205,7 @@ SFORMAT SF_MMU[]={
 	{ "MIME", 4, 2,       MMU.reg_IME},
 	{ "MIE_", 4, 2,       MMU.reg_IE},
 	{ "MIF_", 4, 2,       MMU.reg_IF_bits},
+	{ "MIFP", 4, 2,       MMU.reg_IF_pending},
 
 	{ "MGXC", 8, 1,       &MMU.gfx3dCycles},
 	
@@ -263,7 +270,7 @@ SFORMAT SF_MOVIE[]={
 
 static void mmu_savestate(EMUFILE* os)
 {
-	u32 version = 5;
+	u32 version = 6;
 	write32le(version,os);
 	
 	//version 2:
@@ -285,8 +292,12 @@ static void mmu_savestate(EMUFILE* os)
 	//version 4:
 	MMU_new.sqrt.savestate(os);
 	MMU_new.div.savestate(os);
+
+	//version 6:
+	MMU_new.dsi_tsc.save_state(os);
 }
 
+// TODO: integrate the new wifi state variables once everything is settled
 SFORMAT SF_WIFI[]={
 	{ "W000", 4, 1, &wifiMac.powerOn},
 	{ "W010", 4, 1, &wifiMac.powerOnPending},
@@ -301,17 +312,8 @@ SFORMAT SF_WIFI[]={
 	{ "W070", 2, 1, &wifiMac.wepMode},
 	{ "W080", 4, 1, &wifiMac.WEP_enable},
 
-	{ "W090", 2, 3, &wifiMac.TXSlot[0]},
 	{ "W100", 2, 1, &wifiMac.TXCnt},
-	{ "W110", 2, 1, &wifiMac.TXOpt},
 	{ "W120", 2, 1, &wifiMac.TXStat},
-	{ "W130", 2, 1, &wifiMac.BeaconAddr},
-	{ "W140", 4, 1, &wifiMac.BeaconEnable},
-	{ "W150", 1, 1, &wifiMac.txCurSlot},
-	{ "W160", 1, 3, &wifiMac.txSlotBusy[0]},
-	{ "W170", 4, 3, &wifiMac.txSlotAddr[0]},
-	{ "W180", 4, 3, &wifiMac.txSlotLen[0]},
-	{ "W190", 4, 3, &wifiMac.txSlotRemainingBytes[0]},
 
 	{ "W200", 2, 1, &wifiMac.RXCnt},
 	{ "W210", 2, 1, &wifiMac.RXCheckCounter},
@@ -366,19 +368,9 @@ SFORMAT SF_WIFI[]={
 	{ "W530", 2, 1, &wifiMac.CircBufWrEnd},
 	{ "W540", 2, 1, &wifiMac.CircBufWrSkip},
 
-	{ "W550", 4, 1, &wifiMac.curPacketSize[0]},
-	{ "W560", 4, 1, &wifiMac.curPacketPos[0]},
-	{ "W570", 4, 1, &wifiMac.curPacketSending[0]},
-
 	{ "W580", 2, 0x800, &wifiMac.IOPorts[0]},
 	{ "W590", 2, 1, &wifiMac.randomSeed},
 
-	{ "WX00", 8, 1, &SoftAP.usecCounter},
-	{ "WX10", 1, 4096, &SoftAP.curPacket[0]},
-	{ "WX20", 4, 1, &SoftAP.curPacketSize},
-	{ "WX30", 4, 1, &SoftAP.curPacketPos},
-	{ "WX40", 4, 1, &SoftAP.curPacketSending},
-
 	{ 0 }
 };
 
@@ -448,8 +440,8 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 	ok &= MMU_new.div.loadstate(is,version);
 
 	//to prevent old savestates from confusing IF bits, mask out ones which had been stored but should have been generated
-	MMU.reg_IF_bits[0] &= ~0x00260000;
-	MMU.reg_IF_bits[1] &= ~0x00060000;
+	MMU.reg_IF_bits[0] &= ~0x00200000;
+	MMU.reg_IF_bits[1] &= ~0x00000000;
 
 	MMU_new.gxstat.fifo_low = gxFIFO.size <= 127;
 	MMU_new.gxstat.fifo_empty = gxFIFO.size == 0;
@@ -457,6 +449,10 @@ static bool mmu_loadstate(EMUFILE* is, int size)
 	if(version < 5)
 		MMU.reg_DISP3DCNT_bits = T1ReadWord(MMU.ARM9_REG,0x60);
 
+	if(version < 6) return ok;
+
+	MMU_new.dsi_tsc.load_state(is);
+
 	return ok;
 }
 
@@ -1073,7 +1069,7 @@ static void loadstate()
 		_MMU_write16<ARMCPU_ARM9>(REG_BASE_DISPB+subRegenAddr[i], _MMU_read16<ARMCPU_ARM9>(REG_BASE_DISPB+subRegenAddr[i]));
 	// no need to restore 0x60 since control and MMU.ARM9_REG are both in the savestates, and restoring it could mess up the ack bits anyway
 
-	SetupMMU(nds.debugConsole);
+	SetupMMU(nds.Is_DebugConsole(),nds.Is_DSI());
 
 	execute = !driver->EMU_IsEmulationPaused();
 }
@@ -1128,7 +1124,7 @@ bool savestate_load(EMUFILE* is)
 	_HACK_DONT_STOPMOVIE = false;
 
 	//reset some options to their old defaults which werent saved
-	nds.debugConsole = FALSE;
+	nds._DebugConsole = FALSE;
 
 	//GPU_Reset(MainScreen.gpu, 0);
 	//GPU_Reset(SubScreen.gpu, 1);
@@ -1151,10 +1147,15 @@ bool savestate_load(EMUFILE* is)
 
 	loadstate();
 
-	if((nds.debugConsole!=0) != CommonSettings.DebugConsole) {
-		printf("WARNING: forcing console debug mode to: debugmode=%s\n",nds.debugConsole?"TRUE":"FALSE");
+	if(nds.ConsoleType != CommonSettings.ConsoleType) {
+		printf("WARNING: forcing console type to: ConsoleType=%d\n",nds.ConsoleType);
 	}
 
+	if((nds._DebugConsole!=0) != CommonSettings.DebugConsole) {
+			printf("WARNING: forcing console debug mode to: debugmode=%s\n",nds._DebugConsole?"TRUE":"FALSE");
+	}
+
+
 	return true;
 }
 
@@ -1201,14 +1202,11 @@ void rewindsave () {
 
 void dorewind()
 {
-
 	if(currFrameCounter % rewindinterval)
 		return;
 
 	//printf("rewind\n");
 
-	nds.debugConsole = FALSE;
-
 	int size = rewindbuffer.size();
 
 	if(size < 1) {
