@@ -1,12 +1,31 @@
+/*
+	Copyright (C) 2008-2011 DeSmuME team
+
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
+
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
 /* Predefined OpenGL shaders */
 
 /* Vertex shader */
 const char *vertexShader = {"\
+	varying vec4 pos; \n\
 	void main() \n\
 	{ \n\
 		gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex; \n\
 		gl_TexCoord[0] = gl_TextureMatrix[0] * gl_MultiTexCoord0; \n\
 		gl_FrontColor = gl_Color; \n\
+		pos = gl_Position; \n\
 	} \n\
 "};
 
@@ -16,8 +35,8 @@ const char *fragmentShader = {"\
 	uniform sampler2D tex2d; \n\
 	uniform int hasTexture; \n\
 	uniform int texBlending; \n\
-	\n\
-	\
+	uniform int oglWBuffer; \n\
+	varying vec4 pos; \n\
 	void main() \n\
 	{ \n\
 		vec4 texColor = vec4(1.0, 1.0, 1.0, 1.0); \n\
@@ -58,6 +77,12 @@ const char *fragmentShader = {"\
 						flagColor.rgb = texColor.rgb * gl_Color.rgb + toonColor.rgb; \n\
 						flagColor.a = texColor.a * gl_Color.a; \n\
 					} \n\
+		if (oglWBuffer == 1) \n\
+			// TODO \n\
+			gl_FragDepth = (pos.z / pos.w) * 0.5 + 0.5; \n\
+		else \n\
+			gl_FragDepth = (pos.z / pos.w) * 0.5 + 0.5; \n\
 		gl_FragColor = flagColor; \n\
 	} \n\
 "};
+
