@@ -1,30 +1,22 @@
-/*  Copyright (C) 2006 yopyop
-	Copyright (C) 2008-2010 DeSmuME team
+/*
+	Copyright (C) 2006 yopyop
+	Copyright (C) 2008 shash
+	Copyright (C) 2008-2012 DeSmuME team
 
-	Code added on 18/08/2006 by shash
-		- Missing missaligned addresses correction
-			(reference in http://nocash.emubase.de/gbatek.htm#cpumemoryalignments)
-
-	This file is part of DeSmuME 
-
-	DeSmuME is free software; you can redistribute it and/or modify
+	This file is free software: you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2 of the License, or
+	the Free Software Foundation, either version 2 of the License, or
 	(at your option) any later version.
 
-	DeSmuME is distributed in the hope that it will be useful,
+	This file is distributed in the hope that it will be useful,
 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	GNU General Public License for more details.
 
 	You should have received a copy of the GNU General Public License
-	along with DeSmuME; if not, write to the Free Software
-	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-// THUMB core TODO:
-// 
-
 #include "bios.h"
 #include "debug.h"
 #include "MMU.h"
@@ -163,7 +155,7 @@ TEMPLATE static  u32 FASTCALL OP_LSR_REG(const u32 i)
 TEMPLATE static  u32 FASTCALL OP_ASR_0(const u32 i)
 {
 	cpu->CPSR.bits.C = BIT31(cpu->R[REG_NUM(i, 3)]);
-	cpu->R[REG_NUM(i, 0)] = cpu->CPSR.bits.C * 0xFFFFFFFF;
+	cpu->R[REG_NUM(i, 0)] = BIT31(cpu->R[REG_NUM(i, 3)])*0xFFFFFFFF;
 	cpu->CPSR.bits.N = BIT31(cpu->R[REG_NUM(i, 0)]);
 	cpu->CPSR.bits.Z = cpu->R[REG_NUM(i, 0)] == 0;
 
@@ -201,7 +193,7 @@ TEMPLATE static  u32 FASTCALL OP_ASR_REG(const u32 i)
 	}
 	
 	cpu->CPSR.bits.C = BIT31(cpu->R[REG_NUM(i, 0)]);
-	cpu->R[REG_NUM(i, 0)] = cpu->CPSR.bits.C * 0xFFFFFFFF;
+	cpu->R[REG_NUM(i, 0)] = BIT31(cpu->R[REG_NUM(i, 0)])*0xFFFFFFFF;
 	cpu->CPSR.bits.N = BIT31(cpu->R[REG_NUM(i, 0)]);
 	cpu->CPSR.bits.Z = cpu->R[REG_NUM(i, 0)] == 0;
 	
@@ -642,7 +634,7 @@ TEMPLATE static  u32 FASTCALL OP_MUL_REG(const u32 i)
 	//In earlier versions of the architecture, the value of the C flag was UNPREDICTABLE
 	//after a MUL instruction.
 	
-	if (PROCNUM == 1)	// ARM4T 1S + mI, m = 3
+	if (!cpu->LDTBit)	// ARM4T 1S + mI, m = 3
 		return 4;
 
 	MUL_Mxx_END_THUMB(1);
@@ -914,7 +906,7 @@ TEMPLATE static  u32 FASTCALL OP_POP_PC(const u32 i)
 
 	v = READ32(cpu->mem_if->data, adr);
 	c += MMU_memAccessCycles<PROCNUM,32,MMU_AD_READ>(adr);
-	if(PROCNUM==0)
+	if(cpu->LDTBit)
 		cpu->CPSR.bits.T = BIT0(v);
 
 	cpu->R[15] = v & 0xFFFFFFFE;
@@ -1025,26 +1017,27 @@ TEMPLATE static  u32 FASTCALL OP_SWI_THUMB(const u32 i)
 		|| (cpu->intVector == 0xFFFF0000 && PROCNUM==1);
 
 	if(cpu->swi_tab && !bypassBuiltinSWI) {
-		 //zero 25-dec-2008 - in arm, we were masking to 0x1F. 
-		 //this is probably safer since an invalid opcode could crash the emu
-		 //zero 30-jun-2009 - but they say that the ideas 0xFF should crash the device...
-		 //u32 swinum = cpu->instruction & 0xFF;
+		//zero 25-dec-2008 - in arm, we were masking to 0x1F. 
+		//this is probably safer since an invalid opcode could crash the emu
+		//zero 30-jun-2009 - but they say that the ideas 0xFF should crash the device...
+		//u32 swinum = cpu->instruction & 0xFF;
 		swinum &= 0x1F;
 		//printf("%d ARM SWI %d\n",PROCNUM,swinum);
-	   return cpu->swi_tab[swinum]() + 3;  
+		return cpu->swi_tab[swinum]() + 3;  
 	}
 	else {
-	   /* we use an irq thats not in the irq tab, as
-	   it was replaced due to a changed intVector */
-	   Status_Reg tmp = cpu->CPSR;
-	   armcpu_switchMode(cpu, SVC);		  /* enter svc mode */
-	   cpu->R[14] = cpu->next_instruction;		  /* jump to swi Vector */
-	   cpu->SPSR = tmp;					/* save old CPSR as new SPSR */
-	   cpu->CPSR.bits.T = 0;				/* handle as ARM32 code */
-	   cpu->CPSR.bits.I = 1;
-	   cpu->R[15] = cpu->intVector + 0x08;
-	   cpu->next_instruction = cpu->R[15];
-	   return 3;
+		/* we use an irq thats not in the irq tab, as
+		it was replaced due to a changed intVector */
+		Status_Reg tmp = cpu->CPSR;
+		armcpu_switchMode(cpu, SVC);		  /* enter svc mode */
+		cpu->R[14] = cpu->next_instruction;		  /* jump to swi Vector */
+		cpu->SPSR = tmp;					/* save old CPSR as new SPSR */
+		cpu->CPSR.bits.T = 0;				/* handle as ARM32 code */
+		cpu->CPSR.bits.I = 1;
+		cpu->changeCPSR();
+		cpu->R[15] = cpu->intVector + 0x08;
+		cpu->next_instruction = cpu->R[15];
+		return 3;
 	}
 }
 
@@ -1066,16 +1059,14 @@ TEMPLATE static  u32 FASTCALL OP_B_COND(const u32 i)
 
 TEMPLATE static  u32 FASTCALL OP_B_UNCOND(const u32 i)
 {
-#ifdef DEVELOPER
 	//nocash message detection
 	const u16 last = _MMU_read16<PROCNUM,MMU_AT_DEBUG>(cpu->instruct_adr-2);
 	const u16 next = _MMU_read16<PROCNUM,MMU_AT_DEBUG>(cpu->instruct_adr+2);
 	static const u16 mov_r12_r12 = 0x46E4;
 	if(last == mov_r12_r12 && next == 0x6464)
 	{
-		NocashMessage(cpu);
+		NocashMessage(cpu,6);
 	}
-#endif
 
 	cpu->R[15] += (SIGNEEXT_IMM11(i)<<1);
 	cpu->next_instruction = cpu->R[15];
@@ -1135,7 +1126,7 @@ TEMPLATE static  u32 FASTCALL OP_BX_THUMB(const u32 i)
 	//----- the instruction are UNPREDICTABLE (because the value read for R15 has bits[1:0]==0b10).
 	if (Rm == 15)
 	{
-		printf("THUMB%c: BX using PC as operand\n", PROCNUM?'7':'9');
+		//printf("THUMB%c: BX using PC as operand\n", PROCNUM?'7':'9');
 		//emu_halt();
 	}
 	cpu->CPSR.bits.T = BIT0(Rm);
