@@ -1,33 +1,31 @@
-/*  Copyright 2009 DeSmuME team
+/*
+	Copyright (C) 2009-2011 DeSmuME team
 
-    This file is part of DeSmuME
+	This file is free software: you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation, either version 2 of the License, or
+	(at your option) any later version.
 
-    DeSmuME is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+	This file is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
 
-    DeSmuME is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with DeSmuME; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
+	You should have received a copy of the GNU General Public License
+	along with the this software.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "types.h"
 #include "task.h"
 #include <stdio.h>
 
-#ifdef _MSC_VER
-
 #ifdef _WINDOWS
 #include <windows.h>
+#else
+#include <pthread.h>
 #endif
 
-
+#ifdef _MSC_VER
 class Task::Impl {
 public:
 	Impl();
@@ -156,147 +154,152 @@ void* Task::Impl::finish()
 
 #else
 
-//just a stub impl that doesnt actually do any threading.
-//somebody needs to update the pthread implementation below
 class Task::Impl {
-public:
-	Impl() {}
-	~Impl() {}
-
-	void start(bool spinlock) {}
-	void shutdown() {}
-
-	void* ret;
-	void execute(const TWork &work, void* param) { ret = work(param); }
+private:
+	pthread_t _thread;
+	bool _isThreadRunning;
 	
-	void* finish() { return ret; }
-};
-
-
-/*
-#include <pthread.h>
-
-class Task::Impl {
 public:
 	Impl();
+	~Impl();
 
-	//execute some work
-	void execute(const TWork &work, void* param);
-
-	//wait for the work to complete
+	void start(bool spinlock);
+	void execute(const TWork &work, void *param);
 	void* finish();
-
-	pthread_t thread;
-	static void* s_taskProc(void *ptr);
-	void taskProc();
-	void start();
 	void shutdown();
 
-	//the work function that shall be executed
+	pthread_mutex_t mutex;
+	pthread_cond_t condWork;
 	TWork work;
-	void* param;
-
-	bool initialized;
-
-	struct WaitEvent
-	{
-		WaitEvent() 
-			: condition(PTHREAD_COND_INITIALIZER)
-			, mutex(PTHREAD_MUTEX_INITIALIZER)
-			, value(false)
-		{}
-		pthread_mutex_t mutex;
-		pthread_cond_t condition;
-		bool value;
-
-		//waits for the WaitEvent to be set
-		void waitAndClear()
-		{ 
-			lock();
-			if(!value)
-				pthread_cond_wait( &condition, &mutex );
-			value = false;
-			unlock();
-		}
-
-		//sets the WaitEvent
-		void signal()
-		{
-			lock();
-			if(!value) {
-				value = true;
-				pthread_cond_signal( &condition );
-			}
-			unlock();
-		}
-
-		//locks the condition's mutex
-		void lock() { pthread_mutex_lock(&mutex); }
-		
-		//unlocks the condition's mutex
-		void unlock() { pthread_mutex_unlock( &mutex ); }
-
-	} incomingWork, workDone;
-
+	void *param;
+	void *ret;
+	bool exitThread;
 };
 
+static void* taskProc(void *arg)
+{
+	Task::Impl *ctx = (Task::Impl *)arg;
+
+	do {
+		pthread_mutex_lock(&ctx->mutex);
+
+		while (ctx->work == NULL && !ctx->exitThread) {
+			pthread_cond_wait(&ctx->condWork, &ctx->mutex);
+		}
+
+		if (ctx->work != NULL) {
+			ctx->ret = ctx->work(ctx->param);
+		} else {
+			ctx->ret = NULL;
+		}
+
+		ctx->work = NULL;
+		pthread_cond_signal(&ctx->condWork);
+
+		pthread_mutex_unlock(&ctx->mutex);
+
+	} while(!ctx->exitThread);
+
+	return NULL;
+}
+
 Task::Impl::Impl()
-	: work(NULL)
-	, initialized(false)
 {
+	_isThreadRunning = false;
+	work = NULL;
+	param = NULL;
+	ret = NULL;
+	exitThread = false;
+
+	pthread_mutex_init(&mutex, NULL);
+	pthread_cond_init(&condWork, NULL);
 }
 
-void* Task::Impl::s_taskProc(void *ptr)
+Task::Impl::~Impl()
 {
-	//just past the buck to the instance method
-	((Task::Impl*)ptr)->taskProc();
-	return 0;
+	shutdown();
+	pthread_mutex_destroy(&mutex);
+	pthread_cond_destroy(&condWork);
 }
 
-void Task::Impl::taskProc()
+void Task::Impl::start(bool spinlock)
 {
-	for(;;) {
-		//wait for a chunk of work
-		incomingWork.waitAndClear();
-		//execute the work
-		param = work(param);
-		//signal completion
-		workDone.signal();
-	}
-}
+	pthread_mutex_lock(&this->mutex);
 
-void Task::Impl::start()
-{
-	pthread_create( &thread, NULL, Task::Impl::s_taskProc, (void*)this );     
-	initialized = true;
-}
-void Task::Impl::shutdown()
-{
-	if(!initialized)
+	if (this->_isThreadRunning) {
+		pthread_mutex_unlock(&this->mutex);
 		return;
-	// pthread_join or something, NYI, this code is all disabled anyway at the time of this writing
-	initialized = false;
+	}
+
+	this->work = NULL;
+	this->param = NULL;
+	this->ret = NULL;
+	this->exitThread = false;
+	pthread_create(&this->_thread, NULL, &taskProc, this);
+	this->_isThreadRunning = true;
+
+	pthread_mutex_unlock(&this->mutex);
 }
 
-void Task::Impl::execute(const TWork &work, void* param) 
+void Task::Impl::execute(const TWork &work, void *param)
 {
-	//initialization is deferred to the first execute to give win32 time to startup
-	if(!initialized) init();
-	//setup the work
+	pthread_mutex_lock(&this->mutex);
+
+	if (work == NULL || !this->_isThreadRunning) {
+		pthread_mutex_unlock(&this->mutex);
+		return;
+	}
+
 	this->work = work;
 	this->param = param;
-	//signal it to start
-	incomingWork.signal();
+	pthread_cond_signal(&this->condWork);
+
+	pthread_mutex_unlock(&this->mutex);
 }
 
 void* Task::Impl::finish()
 {
-	//just wait for the work to be done
-	workDone.waitAndClear();
-	return param;
-}
-*/
+	void *returnValue = NULL;
 
+	pthread_mutex_lock(&this->mutex);
+
+	if (!this->_isThreadRunning) {
+		pthread_mutex_unlock(&this->mutex);
+		return returnValue;
+	}
+
+	while (this->work != NULL) {
+		pthread_cond_wait(&this->condWork, &this->mutex);
+	}
+
+	returnValue = this->ret;
+
+	pthread_mutex_unlock(&this->mutex);
+
+	return returnValue;
+}
+
+void Task::Impl::shutdown()
+{
+	pthread_mutex_lock(&this->mutex);
+
+	if (!this->_isThreadRunning) {
+		pthread_mutex_unlock(&this->mutex);
+		return;
+	}
+
+	this->work = NULL;
+	this->exitThread = true;
+	pthread_cond_signal(&this->condWork);
+
+	pthread_mutex_unlock(&this->mutex);
+
+	pthread_join(this->_thread, NULL);
+
+	pthread_mutex_lock(&this->mutex);
+	this->_isThreadRunning = false;
+	pthread_mutex_unlock(&this->mutex);
+}
 #endif
 
 void Task::start(bool spinlock) { impl->start(spinlock); }
